@@ -68,7 +68,6 @@ Notcurses_new(PyTypeObject *subtype, PyObject *args, PyObject *kwds)
     PyObject *main_fd_object = NULL;
 
     const char *term_type = NULL;
-    PyObject *render_fd_object = NULL;
     int log_level = {0};
     const char *margins_str = NULL;
     PyObject *margin_top = NULL, *margin_right = NULL, *margin_bottom = NULL, *margin_left = NULL;
@@ -82,7 +81,7 @@ Notcurses_new(PyTypeObject *subtype, PyObject *args, PyObject *kwds)
 
     GNU_PY_CHECK_BOOL(PyArg_ParseTupleAndKeywords(args, kwds, "|O!sO!isO!O!O!O!K", keywords,
                                                   &PyLong_Type, &main_fd_object,
-                                                  &term_type, &PyLong_Type, &render_fd_object, &log_level,
+                                                  &term_type, &PyLong_Type, &log_level,
                                                   &margins_str,
                                                   &PyLong_Type, &margin_top, &PyLong_Type, &margin_right, &PyLong_Type, &margin_bottom, &PyLong_Type, &margin_left,
                                                   &flags));
@@ -91,20 +90,6 @@ Notcurses_new(PyTypeObject *subtype, PyObject *args, PyObject *kwds)
 
     options.termtype = term_type;
 
-    if (NULL != render_fd_object)
-    {
-        long render_fd = GNU_PY_LONG_CHECK(render_fd_object);
-
-        FILE *renderfp = fdopen((int)render_fd, "w");
-        if (NULL == renderfp)
-        {
-            PyErr_SetString(PyExc_ValueError, "Failed to open render file descriptor.");
-            return NULL;
-        }
-
-        options.renderfp = renderfp;
-    }
-
     options.loglevel = (ncloglevel_e)log_level;
 
     if (NULL != margins_str)
@@ -114,22 +99,22 @@ Notcurses_new(PyTypeObject *subtype, PyObject *args, PyObject *kwds)
 
     if (NULL != margin_top)
     {
-        options.margin_t = (int)GNU_PY_LONG_CHECK(margin_top);
+        options.margin_t = GNU_PY_UINT_CHECK(margin_top);
     }
 
     if (NULL != margin_right)
     {
-        options.margin_r = (int)GNU_PY_LONG_CHECK(margin_right);
+        options.margin_r = GNU_PY_UINT_CHECK(margin_right);
     }
 
     if (NULL != margin_bottom)
     {
-        options.margin_b = (int)GNU_PY_LONG_CHECK(margin_bottom);
+        options.margin_b = GNU_PY_UINT_CHECK(margin_bottom);
     }
 
     if (NULL != margin_left)
     {
-        options.margin_l = (int)GNU_PY_LONG_CHECK(margin_left);
+        options.margin_l = GNU_PY_UINT_CHECK(margin_left);
     }
 
     options.flags = (uint64_t)flags;
@@ -195,13 +180,6 @@ Notcurses_bottom(NotcursesObject *self, PyObject *Py_UNUSED(args))
     return new_object;
 }
 
-static PyObject *
-Notcurses_getc(NotcursesObject *Py_UNUSED(self), PyObject *Py_UNUSED(args), PyObject *Py_UNUSED(kwds))
-{
-    PyErr_SetString(PyExc_NotImplementedError, "TODO when ncinput is implemented");
-    return NULL;
-}
-
 static PyObject *
 Notcurses_inputready_fd(NotcursesObject *self, PyObject *Py_UNUSED(args))
 {
@@ -211,29 +189,100 @@ Notcurses_inputready_fd(NotcursesObject *self, PyObject *Py_UNUSED(args))
 }
 
 static PyObject *
-Notcurses_getc_nblock(NotcursesObject *Py_UNUSED(self), PyObject *Py_UNUSED(args))
+build_NcInput(uint32_t const id, ncinput const *const ni)
 {
-    PyErr_SetString(PyExc_NotImplementedError, "TODO when ncinput is implemented");
-    return NULL;
+    if (id == (uint32_t)-1)
+    {
+        PyErr_Format(PyExc_RuntimeError, "notcurses_get_nblock return -1");
+        return NULL;
+    }
+    else if (id == 0)
+        // No input event.
+        Py_RETURN_NONE;
+    else
+    {
+        PyObject *input = GNU_PY_CHECK(PyStructSequence_New(NcInput_Type));
+        PyObject *utf8 = GNU_PY_CHECK(PyUnicode_FromStringAndSize(ni->utf8, (Py_ssize_t)strlen(ni->utf8)));
+        PyStructSequence_SET_ITEM(input, 0, PyLong_FromLong(ni->id));
+        PyStructSequence_SET_ITEM(input, 1, PyLong_FromLong(ni->y));
+        PyStructSequence_SET_ITEM(input, 2, PyLong_FromLong(ni->x));
+        PyStructSequence_SET_ITEM(input, 3, utf8);
+        PyStructSequence_SET_ITEM(input, 4, PyLong_FromLong(ni->evtype));
+        PyStructSequence_SET_ITEM(input, 5, PyLong_FromLong(ni->modifiers));
+        PyStructSequence_SET_ITEM(input, 6, PyLong_FromLong(ni->ypx));
+        PyStructSequence_SET_ITEM(input, 7, PyLong_FromLong(ni->xpx));
+        return input;
+    }
+}
+
+static inline struct timespec secs_to_timespec(double const sec) {
+    assert(sec >= 0);
+    struct timespec const timespec = {
+        .tv_sec = (time_t)sec,
+        .tv_nsec = (long)((sec - (double)(time_t)sec) * 1E+9) };
+    return timespec;
 }
 
 static PyObject *
-Notcurses_getc_blocking(NotcursesObject *Py_UNUSED(self), PyObject *Py_UNUSED(args))
+Notcurses_get(NotcursesObject *self, PyObject *args, PyObject *kw)
 {
-    PyErr_SetString(PyExc_NotImplementedError, "TODO when ncinput is implemented");
-    return NULL;
+    static char *keywords[] = {"deadline", NULL};
+    PyObject* deadline_arg;
+    if (!PyArg_ParseTupleAndKeywords(args, kw, "O", keywords, &deadline_arg))
+        return NULL;
+
+    struct timespec timespec;
+    struct timespec *ts;
+    if (deadline_arg == Py_None)
+        // No deadline.
+        ts = NULL;
+    else
+    {
+        double const deadline = PyFloat_AsDouble(deadline_arg);
+        if (PyErr_Occurred())
+            // Can't convert to float sec.
+            return NULL;
+        if (deadline < 0)
+        {
+            PyErr_Format(PyExc_ValueError, "negative deadline");
+            return NULL;
+        }
+        timespec = secs_to_timespec(deadline);
+        ts = &timespec;
+    }
+
+    struct ncinput ni;
+    uint32_t const id = notcurses_get(self->notcurses_ptr, ts, &ni);
+    return build_NcInput(id, &ni);
 }
 
 static PyObject *
-Notcurses_mouse_enable(NotcursesObject *self, PyObject *Py_UNUSED(args))
+Notcurses_get_nblock(NotcursesObject *self, PyObject *Py_UNUSED(args))
 {
-    CHECK_NOTCURSES(notcurses_mouse_enable(self->notcurses_ptr));
+    struct ncinput ni;
+    uint32_t const id = notcurses_get_nblock(self->notcurses_ptr, &ni);
+    return build_NcInput(id, &ni);
+}
+
+static PyObject *
+Notcurses_get_blocking(NotcursesObject *self, PyObject *Py_UNUSED(args))
+{
+    struct ncinput ni;
+    uint32_t const id = notcurses_get_blocking(self->notcurses_ptr, &ni);
+    return build_NcInput(id, &ni);
+}
+
+static PyObject *
+Notcurses_mice_enable(NotcursesObject *self, PyObject *Py_UNUSED(args))
+{
+    CHECK_NOTCURSES(notcurses_mice_enable(self->notcurses_ptr, NCMICE_BUTTON_EVENT));
     Py_RETURN_NONE;
 }
+
 static PyObject *
-Notcurses_mouse_disable(NotcursesObject *self, PyObject *Py_UNUSED(args))
+Notcurses_mice_disable(NotcursesObject *self, PyObject *Py_UNUSED(args))
 {
-    CHECK_NOTCURSES(notcurses_mouse_disable(self->notcurses_ptr));
+    CHECK_NOTCURSES(notcurses_mice_disable(self->notcurses_ptr));
     Py_RETURN_NONE;
 }
 
@@ -254,10 +303,10 @@ Notcurses_linesigs_enable(NotcursesObject *self, PyObject *Py_UNUSED(args))
 static PyObject *
 Notcurses_refresh(NotcursesObject *self, PyObject *Py_UNUSED(args))
 {
-    int rows = 0, collumns = 0;
-    CHECK_NOTCURSES(notcurses_refresh(self->notcurses_ptr, &rows, &collumns));
+    unsigned rows = 0, columns = 0;
+    CHECK_NOTCURSES(notcurses_refresh(self->notcurses_ptr, &rows, &columns));
 
-    return Py_BuildValue("ii", rows, collumns);
+    return Py_BuildValue("II", rows, columns);
 }
 
 static PyObject *
@@ -274,22 +323,22 @@ Notcurses_stdplane(NotcursesObject *self, PyObject *Py_UNUSED(args))
 static PyObject *
 Notcurses_stddim_yx(NotcursesObject *self, PyObject *Py_UNUSED(args))
 {
-    int y = 0, x = 0;
+    unsigned y = 0, x = 0;
     PyObject *new_object CLEANUP_PY_OBJ = NcPlane_Type.tp_alloc((PyTypeObject *)&NcPlane_Type, 0);
     NcPlaneObject *new_plane = (NcPlaneObject *)new_object;
     new_plane->ncplane_ptr = CHECK_NOTCURSES_PTR(notcurses_stddim_yx(self->notcurses_ptr, &y, &x));
 
     Py_INCREF(new_object);
-    return Py_BuildValue("Oii", new_object, y, x);
+    return Py_BuildValue("OII", new_object, y, x);
 }
 
 static PyObject *
 Notcurses_term_dim_yx(NotcursesObject *self, PyObject *Py_UNUSED(args))
 {
-    int rows = 0, collumns = 0;
-    notcurses_term_dim_yx(self->notcurses_ptr, &rows, &collumns);
+    unsigned rows = 0, columns = 0;
+    notcurses_term_dim_yx(self->notcurses_ptr, &rows, &columns);
 
-    return Py_BuildValue("ii", rows, collumns);
+    return Py_BuildValue("II", rows, columns);
 }
 
 static PyObject *
@@ -303,11 +352,11 @@ static PyObject *
 Notcurses_pile_create(NotcursesObject *self, PyObject *args, PyObject *kwds)
 {
     int y = 0, x = 0;
-    int rows = 0, cols = 0;
+    unsigned rows = 0, cols = 0;
     const char *name = NULL;
     // TODO reseize callback
     unsigned long long flags = 0;
-    int margin_b = 0, margin_r = 0;
+    unsigned int margin_b = 0, margin_r = 0;
 
     char *keywords[] = {"y_pos", "x_pos",
                         "rows", "cols",
@@ -315,7 +364,7 @@ Notcurses_pile_create(NotcursesObject *self, PyObject *args, PyObject *kwds)
                         "flags",
                         "margin_b", "margin_r", NULL};
 
-    GNU_PY_CHECK_BOOL(PyArg_ParseTupleAndKeywords(args, kwds, "|iiiisKii", keywords,
+    GNU_PY_CHECK_BOOL(PyArg_ParseTupleAndKeywords(args, kwds, "|iiIIsKii", keywords,
                                                   &y, &x,
                                                   &rows, &cols,
                                                   &name,
@@ -454,13 +503,13 @@ static PyMethodDef Notcurses_methods[] = {
     {"top", (PyCFunction)Notcurses_top, METH_NOARGS, "Return the topmost ncplane of the standard pile."},
     {"bottom", (PyCFunction)Notcurses_bottom, METH_NOARGS, "Return the bottommost ncplane of the standard pile."},
 
-    {"getc", (void *)Notcurses_getc, METH_VARARGS | METH_KEYWORDS, "See ppoll(2) for more detail. Provide a None 'ts' to block at length, a 'ts' of 0 for non-blocking operation, and otherwise a timespec to bound blocking. Signals in sigmask (less several we handle internally) will be atomically masked and unmasked per ppoll(2). It should generally contain all signals. Returns a single Unicode code point, or (char32_t)-1 on error. 'sigmask' may be NULL. Returns 0 on a timeout. If an event is processed, the return value is the 'id' field from that event. 'ni' may be NULL."},
+    {"get", (void *)Notcurses_get, METH_VARARGS | METH_KEYWORDS, "See ppoll(2) for more detail. Provide a None 'ts' to block at length, a 'ts' of 0 for non-blocking operation, and otherwise a timespec to bound blocking. Signals in sigmask (less several we handle internally) will be atomically masked and unmasked per ppoll(2). It should generally contain all signals. Returns a single Unicode code point, or (char32_t)-1 on error. 'sigmask' may be NULL. Returns 0 on a timeout. If an event is processed, the return value is the 'id' field from that event. 'ni' may be NULL."},
     {"inputready_fd", (PyCFunction)Notcurses_inputready_fd, METH_NOARGS, "Get a file descriptor suitable for input event poll()ing. When this descriptor becomes available, you can call notcurses_getc_nblock(), and input ought be ready. This file descriptor is *not* necessarily the file descriptor associated with stdin (but it might be!)."},
-    {"getc_nblock", (PyCFunction)Notcurses_getc_nblock, METH_NOARGS, "Get input event without blocking. If no event is ready, returns None."},
-    {"getc_blocking", (PyCFunction)Notcurses_getc_blocking, METH_NOARGS, "Get input event completely blocking until and event or signal received."},
+    {"get_nblock", (PyCFunction)Notcurses_get_nblock, METH_NOARGS, "Get input event without blocking. If no event is ready, returns None."},
+    {"get_blocking", (PyCFunction)Notcurses_get_blocking, METH_NOARGS, "Get input event completely blocking until and event or signal received."},
 
-    {"mouse_enable", (PyCFunction)Notcurses_mouse_enable, METH_NOARGS, "Enable the mouse in \"button-event tracking\" mode with focus detection and UTF8-style extended coordinates. On success mouse events will be published to getc()"},
-    {"mouse_disable", (PyCFunction)Notcurses_mouse_disable, METH_NOARGS, "Disable mouse events. Any events in the input queue can still be delivered."},
+    {"mice_enable", (PyCFunction)Notcurses_mice_enable, METH_NOARGS, "Enable the mouse in \"button-event tracking\" mode with focus detection and UTF8-style extended coordinates. On success mouse events will be published to getc()"},
+    {"mice_disable", (PyCFunction)Notcurses_mice_disable, METH_NOARGS, "Disable mouse events. Any events in the input queue can still be delivered."},
     {"linesigs_disable", (PyCFunction)Notcurses_linesigs_disable, METH_NOARGS, "Disable signals originating from the terminal's line discipline, i.e. SIGINT (^C), SIGQUIT (^\\), and SIGTSTP (^Z). They are enabled by default."},
     {"linesigs_enable", (PyCFunction)Notcurses_linesigs_enable, METH_NOARGS, "Restore signals originating from the terminal's line discipline, i.e. SIGINT (^C), SIGQUIT (^\\), and SIGTSTP (^Z), if disabled."},
 
