@@ -1,6 +1,8 @@
 #include "demo.h"
 #include <pthread.h>
 #include <inttypes.h>
+#include "lib/fbuf.h"
+#include "lib/internal.h"
 
 // we provide a heads-up display throughout the demo, detailing the demos we're
 // about to run, running, and just runned. the user can move this HUD with
@@ -27,8 +29,6 @@ static int plot_grab_y = -1;
 // position of the plot *when grab started*
 static int plot_pos_y;
 
-#define FPSHZ 2
-
 #define FPSGRAPH_MAX_COLS 72 // give it some room on each side of an 80-column term
 
 // how many columns for runtime?
@@ -49,6 +49,7 @@ static struct ncplane* debug; // "debug info" modal popup
 
 #define MENUSTR_TOGGLE_HUD "Toggle HUD"
 #define MENUSTR_TOGGLE_PLOT "Toggle FPS plot"
+#define MENUSTR_REDRAW_SCREEN "Redraw the screen"
 #define MENUSTR_RESTART "Restart"
 #define MENUSTR_ABOUT "About"
 #define MENUSTR_DEBUG "Debug info"
@@ -69,13 +70,13 @@ hud_standard_bg_rgb(struct ncplane* n){
 
 static int
 count_debug_lines(const char* output, size_t outputlen){
-  int lines = 0;
+  int ll = 0;
   for(size_t i = 0 ; i < outputlen ; ++i){
     if(output[i] == '\n'){
-      ++lines;
+      ++ll;
     }
   }
-  return lines;
+  return ll;
 }
 
 static void
@@ -86,22 +87,17 @@ debug_toggle(struct notcurses* nc){
     debug = NULL;
     return;
   }
-  int dimy, dimx;
+  unsigned dimy, dimx;
   notcurses_term_dim_yx(nc, &dimy, &dimx);
-  char* output = NULL;
-  size_t outputlen = 0;
-  FILE* mstream = open_memstream(&output, &outputlen);
-  if(mstream == NULL){
-    return;
-  }
-  notcurses_debug(nc, mstream);
-  if(fclose(mstream)){
+  fbuf f;
+  if(fbuf_init_small(&f)){
     return;
   }
+  notcurses_debug_fbuf(nc, &f);
   ncplane_options nopts = {
     .y = 3,
     .x = NCALIGN_CENTER,
-    .rows = count_debug_lines(output, outputlen) + 1,
+    .rows = count_debug_lines(f.buf, f.used) + 1,
     // make it one column longer than the maximum debug output, so that a full
     // line of output doesn't cause trigger a newline. we'll make the last
     // column transparent.
@@ -111,7 +107,7 @@ debug_toggle(struct notcurses* nc){
   };
   struct ncplane* n = ncplane_create(notcurses_stdplane(nc), &nopts);
   if(n == NULL){
-    free(output);
+    fbuf_free(&f);
     return;
   }
   uint64_t channels = 0;
@@ -121,20 +117,21 @@ debug_toggle(struct notcurses* nc){
   ncplane_set_scrolling(n, true);
   ncplane_set_fg_rgb(n, 0x0a0a0a);
   ncplane_set_bg_rgb(n, 0xffffe5);
-  if(ncplane_puttext(n, 0, NCALIGN_LEFT, output, &outputlen) < 0){
-    free(output);
+  size_t b = f.used;
+  if(ncplane_puttext(n, 0, NCALIGN_LEFT, f.buf, &b) < 0){
+    fbuf_free(&f);
     ncplane_destroy(n);
     return;
   }
-  for(int y = 0 ; y < ncplane_dim_y(n) ; ++y){
-    nccell c = CELL_TRIVIAL_INITIALIZER;
+  fbuf_free(&f);
+  for(unsigned y = 0 ; y < ncplane_dim_y(n) ; ++y){
+    nccell c = NCCELL_TRIVIAL_INITIALIZER;
     nccell_set_fg_alpha(&c, NCALPHA_TRANSPARENT);
     nccell_set_bg_alpha(&c, NCALPHA_TRANSPARENT);
     ncplane_putc_yx(n, y, ncplane_dim_x(n) - 1, &c);
     nccell_release(n, &c);
   }
   ncplane_putstr_aligned(n, ncplane_dim_y(n) - 1, NCALIGN_CENTER, "Press Alt+d to hide this window");
-  free(output);
   debug = n;
 }
 
@@ -148,7 +145,7 @@ about_toggle(struct notcurses* nc){
   }
   const int ABOUT_ROWS = 9;
   const int ABOUT_COLS = 40;
-  int dimy;
+  unsigned dimy;
   notcurses_term_dim_yx(nc, &dimy, NULL);
   ncplane_options nopts = {
     .y = 3,
@@ -175,9 +172,9 @@ about_toggle(struct notcurses* nc){
     ncplane_printf_aligned(n, 4, NCALIGN_RIGHT, "restart Ctrl+R  ");
     ncplane_printf_aligned(n, 5, NCALIGN_CENTER, "q quit");
     ncplane_putstr_aligned(n, 7, NCALIGN_CENTER, "\u00a9 nick black <nickblack@linux.com>");
-    nccell ul = CELL_TRIVIAL_INITIALIZER, ur = CELL_TRIVIAL_INITIALIZER;
-    nccell lr = CELL_TRIVIAL_INITIALIZER, ll = CELL_TRIVIAL_INITIALIZER;
-    nccell hl = CELL_TRIVIAL_INITIALIZER, vl = CELL_TRIVIAL_INITIALIZER;
+    nccell ul = NCCELL_TRIVIAL_INITIALIZER, ur = NCCELL_TRIVIAL_INITIALIZER;
+    nccell lr = NCCELL_TRIVIAL_INITIALIZER, ll = NCCELL_TRIVIAL_INITIALIZER;
+    nccell hl = NCCELL_TRIVIAL_INITIALIZER, vl = NCCELL_TRIVIAL_INITIALIZER;
     channels = 0;
     ncchannels_set_fg_rgb(&channels, 0xc020c0);
     ncchannels_set_bg_rgb(&channels, 0);
@@ -229,10 +226,10 @@ fpsplot_toggle(struct notcurses* nc){
   }
   plot_hidden = !plot_hidden;
   if(plot_hidden){
-    ncplane_reparent(ncuplot_plane(plot), ncuplot_plane(plot));
+    ncplane_reparent_family(ncuplot_plane(plot), ncuplot_plane(plot));
   }else{
-    ncplane_reparent(ncuplot_plane(plot), notcurses_stdplane(nc));
-    ncplane_move_top(ncuplot_plane(plot));
+    ncplane_reparent_family(ncuplot_plane(plot), notcurses_stdplane(nc));
+    ncplane_move_family_top(ncuplot_plane(plot));
   }
   return demo_render(nc);
 }
@@ -246,39 +243,49 @@ bool menu_or_hud_key(struct notcurses *nc, const struct ncinput *ni){
     if(sel == NULL){
       return false;
     }
-  }else if(menu && ni->id == NCKEY_RELEASE){
+  }else if(menu && ni->id == NCKEY_BUTTON1 && ni->evtype == NCTYPE_RELEASE){
     const char* sel = ncmenu_mouse_selected(menu, ni, &tmpni);
     if(sel == NULL){
       memcpy(&tmpni, ni, sizeof(tmpni));
     }
   }else{
+    if(ni->evtype == NCTYPE_RELEASE){
+      return false;
+    }
     memcpy(&tmpni, ni, sizeof(tmpni));
   }
   // toggle the HUD
-  if(tmpni.id == 'H' && !tmpni.alt && !tmpni.ctrl){
+  if(tmpni.id == 'H' && !ncinput_alt_p(&tmpni) && !ncinput_ctrl_p(&tmpni)){
     hud_toggle(nc);
     return true;
   }
-  if(tmpni.id == 'P' && !tmpni.alt && !tmpni.ctrl){
+  if(tmpni.id == 'P' && !ncinput_alt_p(&tmpni) && !ncinput_ctrl_p(&tmpni)){
     fpsplot_toggle(nc);
     return true;
   }
-  if(tmpni.id == 'U' && !tmpni.alt && tmpni.ctrl){
+  if(tmpni.id == 'U' && !ncinput_alt_p(&tmpni) && ncinput_ctrl_p(&tmpni)){
     about_toggle(nc);
     return true;
   }
-  if(tmpni.id == 'd' && tmpni.alt && !tmpni.ctrl){
+  if(tmpni.id == 'd' && ncinput_alt_p(&tmpni)){
     debug_toggle(nc);
     return true;
   }
-  if(tmpni.id == 'R' && !tmpni.alt && tmpni.ctrl){
+  if(tmpni.id == 'L' && !ncinput_alt_p(&tmpni) && ncinput_ctrl_p(&tmpni)){
+    if(menu){
+      ncmenu_rollup(menu);
+    }
+    notcurses_refresh(nc, NULL, NULL);
+    return true;
+  }
+  if(tmpni.id == 'R' && !ncinput_alt_p(&tmpni) && ncinput_ctrl_p(&tmpni)){
     if(menu){
       ncmenu_rollup(menu);
     }
     interrupt_and_restart_demos();
     return true;
   }
-  if(tmpni.id == 'q' && !tmpni.alt && !tmpni.ctrl){
+  if(tmpni.id == 'q' && !ncinput_alt_p(&tmpni)){
     if(menu){
       ncmenu_rollup(menu);
     }
@@ -298,22 +305,23 @@ struct ncmenu* menu_create(struct notcurses* nc){
   struct ncmenu_item demo_items[] = {
     { .desc = MENUSTR_TOGGLE_HUD, .shortcut = { .id = 'H', }, },
     { .desc = MENUSTR_TOGGLE_PLOT, .shortcut = { .id = 'P', }, },
+    { .desc = MENUSTR_REDRAW_SCREEN, .shortcut = { .id = 'L', .modifiers = NCKEY_MOD_CTRL }, },
     { .desc = NULL, },
-    { .desc = MENUSTR_RESTART, .shortcut = { .id = 'R', .ctrl = true, }, },
+    { .desc = MENUSTR_RESTART, .shortcut = { .id = 'R', .modifiers = NCKEY_MOD_CTRL, }, },
     { .desc = MENUSTR_QUIT, .shortcut = { .id = 'q', }, },
   };
   struct ncmenu_item help_items[] = {
-    { .desc = MENUSTR_ABOUT, .shortcut = { .id = 'U', .ctrl = true, }, },
-    { .desc = MENUSTR_DEBUG, .shortcut = { .id = 'd', .alt = true, }, },
+    { .desc = MENUSTR_ABOUT, .shortcut = { .id = 'U', .modifiers = NCKEY_MOD_CTRL, }, },
+    { .desc = MENUSTR_DEBUG, .shortcut = { .id = 'd', .modifiers = NCKEY_MOD_ALT, }, },
   };
   struct ncmenu_section sections[] = {
     { .name = "notcurses-demo", .items = demo_items,
       .itemcount = sizeof(demo_items) / sizeof(*demo_items),
-      .shortcut = { .id = 'o', .alt = true, }, },
+      .shortcut = { .id = 'o', .modifiers = NCKEY_MOD_ALT, }, },
     { .name = NULL, .items = NULL, .itemcount = 0, },
     { .name = "help", .items = help_items,
       .itemcount = sizeof(help_items) / sizeof(*help_items),
-      .shortcut = { .id = 'h', .alt = true, }, },
+      .shortcut = { .id = 'h', .modifiers = NCKEY_MOD_ALT, }, },
   };
   uint64_t headerchannels = 0;
   uint64_t sectionchannels = 0;
@@ -338,9 +346,9 @@ struct ncmenu* menu_create(struct notcurses* nc){
 static int
 hud_refresh(struct ncplane* n){
   ncplane_erase(n);
-  nccell ul = CELL_TRIVIAL_INITIALIZER, ur = CELL_TRIVIAL_INITIALIZER;
-  nccell lr = CELL_TRIVIAL_INITIALIZER, ll = CELL_TRIVIAL_INITIALIZER;
-  nccell hl = CELL_TRIVIAL_INITIALIZER, vl = CELL_TRIVIAL_INITIALIZER;
+  nccell ul = NCCELL_TRIVIAL_INITIALIZER, ur = NCCELL_TRIVIAL_INITIALIZER;
+  nccell lr = NCCELL_TRIVIAL_INITIALIZER, ll = NCCELL_TRIVIAL_INITIALIZER;
+  nccell hl = NCCELL_TRIVIAL_INITIALIZER, vl = NCCELL_TRIVIAL_INITIALIZER;
   if(nccells_rounded_box(n, NCSTYLE_NONE, 0, &ul, &ur, &ll, &lr, &hl, &vl)){
     return -1;
   }
@@ -383,7 +391,7 @@ hud_print_finished(elem* list){
       break;
     }
     if(hud){
-      nccell c = CELL_TRIVIAL_INITIALIZER;
+      nccell c = NCCELL_TRIVIAL_INITIALIZER;
       ncplane_base(hud, &c);
       ncplane_set_bg_rgb(hud, nccell_bg_rgb(&c));
       ncplane_set_bg_alpha(hud, NCALPHA_BLEND);
@@ -393,9 +401,13 @@ hud_print_finished(elem* list){
       if(ncplane_printf_yx(hud, line, 1, "%d", e->frames) < 0){
         return -1;
       }
-      if(ncplane_printf_yx(hud, line, 7, "%"PRIu64".%03"PRIu64"s",
-                           e->totalns / NANOSECS_IN_SEC,
-                           (e->totalns % NANOSECS_IN_SEC) / 1000000) < 0){
+      char buf[NCPREFIXCOLUMNS + 2];
+      ncnmetric(e->totalns, sizeof(buf), NANOSECS_IN_SEC, buf, 0, 1000, '\0');
+      for(int x = 6 ; x < 14 - ncstrwidth(buf, NULL, NULL) ; ++x){
+        nccell ci = NCCELL_TRIVIAL_INITIALIZER;
+        ncplane_putc_yx(hud, 1, x, &ci);
+      }
+      if(ncplane_printf_yx(hud, line, 14 - ncstrwidth(buf, NULL, NULL), "%ss", buf) < 0){
         return -1;
       }
       if(ncplane_putstr_yx(hud, line, 16, e->name) < 0){
@@ -411,7 +423,7 @@ struct ncplane* hud_create(struct notcurses* nc){
   if(hud){
     return NULL;
   }
-  int dimx, dimy;
+  unsigned dimx, dimy;
   notcurses_term_dim_yx(nc, &dimy, &dimx);
   struct ncplane_options nopts = {
     // FPS graph is 6 rows tall; we want one row above it
@@ -421,7 +433,7 @@ struct ncplane* hud_create(struct notcurses* nc){
     .cols = HUD_COLS,
     .userptr = NULL,
     .name = "hud",
-    .resizecb = NULL,
+    .resizecb = ncplane_resize_placewithin,
     .flags = NCPLANE_OPTION_HORALIGNED |
              NCPLANE_OPTION_FIXED,
   };
@@ -509,8 +521,8 @@ int hud_completion_notify(const demoresult* result){
   return 0;
 }
 
-// inform the HUD of an upcoming demo
-int hud_schedule(const char* demoname){
+// inform the HUD of an upcoming demo, starting at startns.
+int hud_schedule(const char* demoname, uint64_t startns){
   elem* cure = malloc(sizeof(*cure));
   if(!cure){
     return -1;
@@ -519,10 +531,8 @@ int hud_schedule(const char* demoname){
   cure->name = strdup(demoname);
   cure->totalns = 0;
   cure->frames = 0;
+  cure->startns = startns;
   elems = cure;
-  struct timespec cur;
-  clock_gettime(CLOCK_MONOTONIC, &cur);
-  cure->startns = timespec_to_ns(&cur);
   return hud_print_finished(elems);
 }
 
@@ -584,9 +594,9 @@ int demo_render(struct notcurses* nc){
   clock_gettime(CLOCK_MONOTONIC, &ts);
   if(plot){
     if(!plot_hidden){
-      ncplane_move_top(ncuplot_plane(plot));
+      ncplane_move_family_top(ncuplot_plane(plot));
     }
-    uint64_t ns = (timespec_to_ns(&ts) - plottimestart) / (NANOSECS_IN_SEC / FPSHZ);
+    uint64_t ns = (timespec_to_ns(&ts) - plottimestart) / NANOSECS_IN_SEC;
     ncuplot_add_sample(plot, ns, 1);
   }
   if(menu){
@@ -598,7 +608,7 @@ int demo_render(struct notcurses* nc){
     }
     uint64_t ns = timespec_to_ns(&ts) - elems->startns;
     ++elems->frames;
-    nccell c = CELL_TRIVIAL_INITIALIZER;
+    nccell c = NCCELL_TRIVIAL_INITIALIZER;
     ncplane_base(hud, &c);
     ncplane_set_bg_rgb(hud, nccell_bg_rgb(&c));
     ncplane_set_bg_alpha(hud, NCALPHA_BLEND);
@@ -609,8 +619,14 @@ int demo_render(struct notcurses* nc){
     if(ncplane_printf_yx(hud, 1, 1, "%d", elems->frames) < 0){
       return -1;
     }
-    if(ncplane_printf_yx(hud, 1, 7, "%"PRIu64".%03"PRIu64"s", ns / NANOSECS_IN_SEC,
-                         (ns % NANOSECS_IN_SEC) / 1000000) < 0){
+    char buf[NCPREFIXCOLUMNS + 2];
+    ncnmetric(ns, sizeof(buf), NANOSECS_IN_SEC, buf, 0, 1000, '\0');
+    for(int x = 6 ; x < 14 - ncstrwidth(buf, NULL, NULL) ; ++x){
+      nccell ci = NCCELL_TRIVIAL_INITIALIZER;
+      ncplane_putc_yx(hud, 1, x, &ci);
+    }
+//fprintf(stderr, "[%s] %zu %d\n", buf, strlen(buf), ncstrwidth(buf, NULL, NULL));
+    if(ncplane_printf_yx(hud, 1, 14 - ncstrwidth(buf, NULL, NULL), "%ss", buf) < 0){
       return -1;
     }
     if(ncplane_putstr_yx(hud, 1, 16, elems->name) < 0){
@@ -633,7 +649,7 @@ int demo_render(struct notcurses* nc){
 
 int fpsgraph_init(struct notcurses* nc){
   const int PLOTHEIGHT = 6;
-  int dimy, dimx;
+  unsigned dimy, dimx;
   struct ncplane* stdn = notcurses_stddim_yx(nc, &dimy, &dimx);
   ncplane_options nopts = {
     .y = NCALIGN_BOTTOM,
@@ -653,23 +669,24 @@ int fpsgraph_init(struct notcurses* nc){
   uint32_t style = 0;
   uint64_t channels = 0;
   ncchannels_set_fg_alpha(&channels, NCALPHA_BLEND);
-  ncchannels_set_fg_rgb(&channels, 0x201020);
+  ncchannels_set_fg_rgb(&channels, 0x201040);
   ncchannels_set_bg_alpha(&channels, NCALPHA_BLEND);
-  ncchannels_set_bg_rgb(&channels, 0x201020);
+  ncchannels_set_bg_rgb(&channels, 0x201040);
   ncplane_set_base(newp, "", style, channels);
   ncplot_options opts;
   memset(&opts, 0, sizeof(opts));
   opts.flags = NCPLOT_OPTION_LABELTICKSD |
                NCPLOT_OPTION_EXPONENTIALD |
+               NCPLOT_OPTION_DETECTMAXONLY |
                NCPLOT_OPTION_PRINTSAMPLE;
   opts.gridtype = NCBLIT_BRAILLE;
   opts.legendstyle = NCSTYLE_ITALIC | NCSTYLE_BOLD;
-  opts.title = "frames per semisecond";
+  opts.title = "frames per second";
   ncchannels_set_fg_rgb8(&opts.minchannels, 0x80, 0x80, 0xff);
-  ncchannels_set_bg_rgb(&opts.minchannels, 0x201020);
+  ncchannels_set_bg_rgb(&opts.minchannels, 0x201040);
   ncchannels_set_bg_alpha(&opts.minchannels, NCALPHA_BLEND);
   ncchannels_set_fg_rgb8(&opts.maxchannels, 0x80, 0xff, 0x80);
-  ncchannels_set_bg_rgb(&opts.maxchannels, 0x201020);
+  ncchannels_set_bg_rgb(&opts.maxchannels, 0x201040);
   ncchannels_set_bg_alpha(&opts.maxchannels, NCALPHA_BLEND);
   // takes ownership of newp on all paths
   struct ncuplot* fpsplot = ncuplot_create(newp, &opts, 0, 0);
@@ -681,7 +698,7 @@ int fpsgraph_init(struct notcurses* nc){
   clock_gettime(CLOCK_MONOTONIC, &ts);
   plottimestart = timespec_to_ns(&ts);
   if(plot_hidden){
-    ncplane_reparent(ncuplot_plane(plot), ncuplot_plane(plot));
+    ncplane_reparent_family(ncuplot_plane(plot), ncuplot_plane(plot));
   }
   return 0;
 }
