@@ -112,11 +112,10 @@ static const char* luigis[] = {
 static int
 draw_luigi(struct ncplane* n, const char* sprite){
   uint64_t channels = 0;
-  channels_set_fg_alpha(&channels, CELL_ALPHA_TRANSPARENT);
-  channels_set_bg_alpha(&channels, CELL_ALPHA_TRANSPARENT);
+  ncchannels_set_fg_alpha(&channels, NCALPHA_TRANSPARENT);
+  ncchannels_set_bg_alpha(&channels, NCALPHA_TRANSPARENT);
   ncplane_set_base(n, "", 0, channels);
   size_t s;
-  int sbytes;
   // optimization so we can elide more color changes, see README's "#perf"
   ncplane_set_bg_rgb8(n, 0x00, 0x00, 0x00);
   for(s = 0 ; sprite[s] ; ++s){
@@ -134,7 +133,7 @@ draw_luigi(struct ncplane* n, const char* sprite){
         break;
     }
     if(sprite[s] != '0'){
-      if(ncplane_putegc_yx(n, s / 16, s % 16, " ", &sbytes) != 1){
+      if(ncplane_putegc_yx(n, s / 16, s % 16, " ", NULL) != 1){
         return -1;
       }
     }
@@ -142,11 +141,12 @@ draw_luigi(struct ncplane* n, const char* sprite){
   return 0;
 }
 
-int luigi_demo(struct notcurses* nc){
+int luigi_demo(struct notcurses* nc, uint64_t startns){
+  (void)startns;
   if(!notcurses_canopen_images(nc)){
     return 0;
   }
-  int rows, cols;
+  unsigned rows, cols;
   char* map = find_data("megaman2.bmp");
   struct ncvisual* nv = ncvisual_from_file(map);
   free(map);
@@ -156,8 +156,9 @@ int luigi_demo(struct notcurses* nc){
   struct ncvisual_options vopts = {
     .n = notcurses_stddim_yx(nc, &rows, &cols),
     .scaling = NCSCALE_STRETCH,
+    .flags = NCVISUAL_OPTION_NOINTERPOLATE,
   };
-  if(ncvisual_render(nc, nv, &vopts) == NULL){
+  if(ncvisual_blit(nc, nv, &vopts) == NULL){
     return -1;
   }
   assert(1 == ncvisual_decode(nv));
@@ -165,7 +166,7 @@ int luigi_demo(struct notcurses* nc){
   const int height = 32;
   int yoff = rows * 4 / 5 - height + 1; // tuned
   struct ncplane* lns[3];
-  int i;
+  unsigned i;
   struct ncplane* lastseen = NULL;
   for(i = 0 ; i < 3 ; ++i){
     struct ncplane_options nopts = {
@@ -197,9 +198,13 @@ int luigi_demo(struct notcurses* nc){
     return -1;
   }
   uint64_t channels = 0;
-  channels_set_fg_alpha(&channels, CELL_ALPHA_TRANSPARENT);
-  channels_set_bg_alpha(&channels, CELL_ALPHA_TRANSPARENT);
-  struct ncplane* wmplane = ncvisual_render(nc, wmncv, NULL);
+  ncchannels_set_fg_alpha(&channels, NCALPHA_TRANSPARENT);
+  ncchannels_set_bg_alpha(&channels, NCALPHA_TRANSPARENT);
+  struct ncvisual_options wvopts = {
+    .n = notcurses_stdplane(nc),
+    .flags = NCVISUAL_OPTION_CHILDPLANE,
+  };
+  struct ncplane* wmplane = ncvisual_blit(nc, wmncv, &wvopts);
   if(wmplane == NULL){
     ncvisual_destroy(wmncv);
     return -1;
@@ -214,10 +219,11 @@ int luigi_demo(struct notcurses* nc){
       ncplane_move_top(lastseen);
     }
     ncplane_move_yx(lastseen, yoff, i);
-    int dimy, scaley;
-    ncvisual_geom(nc, wmncv, NULL, &dimy, NULL, &scaley, NULL);
-    dimy /= scaley;
-    ncplane_move_yx(wmplane, rows * 4 / 5 - dimy + 1 + (i % 2), i - 60);
+    ncvgeom geom;
+    ncvisual_geom(nc, wmncv, NULL, &geom);
+    geom.pixy /= geom.scaley;
+    // FIXME what the fuck is this
+    ncplane_move_yx(wmplane, rows * 4 / 5 - geom.pixy + 1 + (i % 2), i - 60);
     DEMO_RENDER(nc);
     demo_nanosleep(nc, &stepdelay);
   }
