@@ -26,7 +26,7 @@ grow_rgb8(uint32_t* rgb){
 
 static struct ncplane*
 legend(struct notcurses* nc, const char* msg){
-  int dimx, dimy;
+  unsigned dimx, dimy;
   notcurses_term_dim_yx(nc, &dimy, &dimx);
   ncplane_options nopts = {
     .rows = 3,
@@ -39,7 +39,7 @@ legend(struct notcurses* nc, const char* msg){
   if(n == NULL){
     return NULL;
   }
-  nccell c = CELL_TRIVIAL_INITIALIZER;
+  nccell c = NCCELL_TRIVIAL_INITIALIZER;
   nccell_set_fg_rgb8(&c, 0, 0, 0); // darken surrounding characters by half
   nccell_set_fg_alpha(&c, NCALPHA_BLEND);
   nccell_set_bg_alpha(&c, NCALPHA_TRANSPARENT); // don't touch background
@@ -62,10 +62,10 @@ legend(struct notcurses* nc, const char* msg){
 static int
 slideitslideit(struct notcurses* nc, struct ncplane* n, uint64_t deadline,
                int* vely, int* velx){
-  int dimy, dimx;
   int yoff, xoff;
-  int ny, nx;
+  unsigned dimy, dimx;
   notcurses_term_dim_yx(nc, &dimy, &dimx);
+  unsigned ny, nx;
   ncplane_dim_yx(n, &ny, &nx);
   ncplane_yx(n, &yoff, &xoff);
   struct timespec iterdelay;
@@ -78,14 +78,14 @@ slideitslideit(struct notcurses* nc, struct ncplane* n, uint64_t deadline,
     if(xoff <= 1){
       xoff = 1;
       *velx = -*velx;
-    }else if(xoff >= dimx - nx){
+    }else if((unsigned)xoff >= dimx - nx){
       xoff = dimx - nx - 1;
       *velx = -*velx;
     }
     if(yoff <= 2){
       yoff = 2;
       *vely = -*vely;
-    }else if(yoff >= dimy - ny){
+    }else if((unsigned)yoff >= dimy - ny){
       yoff = dimy - ny - 1;
       *vely = -*vely;
     }
@@ -99,7 +99,7 @@ slideitslideit(struct notcurses* nc, struct ncplane* n, uint64_t deadline,
 // run panels atop the display in an exploration of transparency
 static int
 slidepanel(struct notcurses* nc, struct ncplane* stdn){
-  int dimy, dimx;
+  unsigned dimy, dimx;
   notcurses_term_dim_yx(nc, &dimy, &dimx);
   int ny = dimy / 4;
   int nx = dimx / 3;
@@ -120,7 +120,7 @@ slidepanel(struct notcurses* nc, struct ncplane* stdn){
   // no glyph, we should show underlying glyphs in the default colors. The
   // background default might be transparent, at the window level (i.e. a copy
   // of the underlying desktop).
-  nccell c = CELL_CHAR_INITIALIZER(' ');
+  nccell c = NCCELL_CHAR_INITIALIZER(' ');
   struct timespec cur;
   ncplane_set_base_cell(n, &c);
   clock_gettime(CLOCK_MONOTONIC, &cur);
@@ -241,6 +241,33 @@ slidepanel(struct notcurses* nc, struct ncplane* stdn){
   }
   ncplane_destroy(l);
 
+  char* logop = find_data("notcurses.png");
+  struct ncvisual* ncv = ncvisual_from_file(logop);
+  if(ncv == NULL){
+    ncplane_destroy(n);
+    return err;
+  }
+  free(logop);
+  struct ncvisual_options vopts = {
+    .n = n,
+    .scaling = NCSCALE_STRETCH,
+    .blitter = NCBLIT_PIXEL,
+  };
+  if(ncvisual_blit(nc, ncv, &vopts) == NULL){
+    ncplane_destroy(n);
+    return err;
+  }
+  ncvisual_destroy(ncv);
+  clock_gettime(CLOCK_MONOTONIC, &cur);
+  l = legend(nc, "partially-transparent image");
+  deadlinens = timespec_to_ns(&cur) + 2 * timespec_to_ns(&demodelay);
+  if( (err = slideitslideit(nc, n, deadlinens, &vely, &velx)) ){
+    ncplane_destroy(n);
+    ncplane_destroy(l);
+    return err;
+  }
+  ncplane_destroy(l);
+
   return ncplane_destroy(n);
 }
 
@@ -248,14 +275,15 @@ slidepanel(struct notcurses* nc, struct ncplane* stdn){
 // markers, each a slightly different color. the goal is to make sure we can
 // have a great many colors, that they progress reasonably through the space,
 // and that we can write to every coordinate.
-int trans_demo(struct notcurses* nc){
-  int maxx, maxy;
+int trans_demo(struct notcurses* nc, uint64_t startns){
+  (void)startns;
+  unsigned maxx, maxy;
   struct ncplane* n = notcurses_stddim_yx(nc, &maxy, &maxx);
   ncplane_set_fg_rgb8(n, 255, 255, 255);
   uint64_t channels = 0;
   ncchannels_set_fg_rgb8(&channels, 0, 128, 128);
   ncchannels_set_bg_rgb8(&channels, 90, 0, 90);
-  int y = 1, x = 0;
+  unsigned y = 1, x = 0;
   ncplane_cursor_move_yx(n, y, x);
   if(ncplane_rounded_box_sized(n, 0, channels, maxy - 1, maxx, 0)){
     return -1;
