@@ -4,6 +4,20 @@ static struct marshal {
   struct ncvisual_options pipopts;
 } marsh;
 
+#define PIPCOLUMNS 18
+
+static int
+pip_resize_cb(struct ncplane* n){
+  int absx = ncplane_abs_x(n);
+  int width = ncplane_dim_x(n);
+  int pwidth = ncplane_dim_x(ncplane_parent(n));
+  int pabsx = ncplane_abs_x(ncplane_parent(n));
+  if(absx + width == pabsx + pwidth){
+    return 0;
+  }
+  return ncplane_move_yx(n, 1, pabsx + pwidth - width);
+}
+
 // pip is non-NULL iff we can do pixel rendering
 static inline int
 streamer(struct ncvisual* ncv, struct ncvisual_options* vopts,
@@ -14,9 +28,10 @@ streamer(struct ncvisual* ncv, struct ncvisual_options* vopts,
         .y = 1,
         .x = NCALIGN_RIGHT,
         .rows = 12,
-        .cols = 18,
+        .cols = PIPCOLUMNS,
         .flags = NCPLANE_OPTION_HORALIGNED,
         .name = "pip",
+        .resizecb = pip_resize_cb,
       };
       marsh.pipopts.n = ncplane_create(vopts->n, &nopts);
       if(marsh.pipopts.n == NULL){
@@ -26,7 +41,7 @@ streamer(struct ncvisual* ncv, struct ncvisual_options* vopts,
       marsh.pipopts.scaling = NCSCALE_STRETCH;
     }
     if(marsh.pipopts.n){
-      ncvisual_render(ncplane_notcurses(marsh.pipopts.n), ncv, &marsh.pipopts);
+      ncvisual_blit(ncplane_notcurses(marsh.pipopts.n), ncv, &marsh.pipopts);
       ncplane_move_above(marsh.pipopts.n, vopts->n);
     }
   }
@@ -37,7 +52,7 @@ static int
 view_video_demo(struct notcurses* nc){
   struct ncplane* ncp = notcurses_stdplane(nc);
   struct ncvisual* ncv;
-  char* fm6 = find_data("fm6.mkv");
+  char* fm6 = find_data("fm6.mov");
   ncv = ncvisual_from_file(fm6);
   if(!ncv){
     free(fm6);
@@ -130,7 +145,7 @@ view_images(struct notcurses* nc, struct ncplane* nstd, int dimy, int dimx){
     .scaling = NCSCALE_STRETCH,
     .y = 1,
   };
-  if(ncvisual_render(nc, ncv2, &vopts) == NULL){
+  if(ncvisual_blit(nc, ncv2, &vopts) == NULL){
     ncvisual_destroy(ncv2);
     ncplane_destroy(dsplane);
     return -1;
@@ -153,7 +168,7 @@ view_images(struct notcurses* nc, struct ncplane* nstd, int dimy, int dimx){
   }
   free(pic);
   vopts.n = notcurses_stdplane(nc);
-  if(ncvisual_render(nc, ncv, &vopts) == NULL){
+  if(ncvisual_blit(nc, ncv, &vopts) == NULL){
     ncvisual_destroy(ncv);
     ncplane_destroy(dsplane);
     return -1;
@@ -169,11 +184,13 @@ view_images(struct notcurses* nc, struct ncplane* nstd, int dimy, int dimx){
   return 0;
 }
 
-int view_demo(struct notcurses* nc){
+int view_demo(struct notcurses* nc, uint64_t startns){
+  (void)startns;
   if(!notcurses_canopen_images(nc)){
     return 0;
   }
-  int dimy, dimx;
+  memset(&marsh, 0, sizeof(marsh));
+  unsigned dimy, dimx;
   struct ncplane* nstd = notcurses_stddim_yx(nc, &dimy, &dimx);
   int ret = view_images(nc, nstd, dimy, dimx);
   if(ret){
