@@ -2,7 +2,7 @@
 #include <unistd.h>
 #include <stdlib.h>
 #include <notcurses/notcurses.h>
-#include "internal.h" // internal headers
+#include "lib/internal.h" // internal headers
 
 static inline wchar_t
 capboolbool(unsigned utf8, bool cap){
@@ -19,7 +19,7 @@ capboolbool(unsigned utf8, bool cap){
 }
 
 static void
-tinfo_debug_cap(struct ncplane* n, const char* name, bool yn, char ch){
+tinfo_debug_cap(struct ncplane* n, const char* name, bool yn){
   if(!yn){
     ncplane_set_styles(n, NCSTYLE_ITALIC);
   }
@@ -27,7 +27,7 @@ tinfo_debug_cap(struct ncplane* n, const char* name, bool yn, char ch){
   ncplane_set_styles(n, NCSTYLE_BOLD);
   ncplane_putwc(n, capboolbool(notcurses_canutf8(ncplane_notcurses(n)), yn));
   ncplane_set_styles(n, NCSTYLE_NONE);
-  ncplane_putchar(n, ch);
+  ncplane_putchar(n, ' ');
 }
 
 static void
@@ -46,8 +46,9 @@ tinfo_debug_style(struct ncplane* n, const char* name, int style, char ch){
 
 static void
 wviz(struct ncplane* n, const wchar_t* wp){
-  for(const wchar_t* w = wp ; *w ; ++w){
-    if(ncplane_putwc(n, *w) <= 0){
+  unsigned wchars;
+  for(const wchar_t* w = wp ; *w ; w += wchars){
+    if(ncplane_putwc_utf32(n, w, &wchars) <= 0){
       ncplane_putchar(n, ' ');
     }
   }
@@ -63,7 +64,7 @@ wvizn(struct ncplane* n, const wchar_t* wp, int nnn){
 }
 
 static int
-braille_viz(struct ncplane* n, wchar_t l, const wchar_t* egcs, wchar_t r,
+braille_viz(struct ncplane* n, wint_t l, const wchar_t* egcs, wchar_t r,
             const char* indent, const wchar_t* bounds, wchar_t r8, wchar_t l8,
             const wchar_t* trailer){
   ncplane_printf(n, "%s%lc", indent, l);
@@ -84,26 +85,51 @@ braille_viz(struct ncplane* n, wchar_t l, const wchar_t* egcs, wchar_t r,
   if(trailer){
     wviz(n, trailer);
   }
-  ncplane_putchar(n, '\n');
+  if(ncplane_dim_x(n) > 80){
+    ncplane_putchar(n, '\n');
+  }
   return 0;
 }
 
 static void
 finish_line(struct ncplane* n){
-  int x;
+  unsigned x;
   ncplane_cursor_yx(n, NULL, &x);
-  while(x++ < 78){
+  while(x++ < 80){
     ncplane_putchar(n, ' ');
   }
-  ncplane_putchar(n, '\n');
+  if(ncplane_dim_x(n) > 80){
+    ncplane_putchar(n, '\n');
+  }
 }
 
 static int
 emoji_viz(struct ncplane* n){
-  static const char emoji[] = "👾🏴🤘🚬🌍🌎🌏🥆💣🗡🔫⚗️⚛️☢️☣️🌿🎱🏧"
-                              "💉💊"
+  static const char emoji[] = "\U0001f47e" // alien monster
+                              "\U0001f3f4" // waving black flag
+                              "\U0001f918" // sign of the horns
+                              "\U0001f6ac" // cigarette, delicious
+                              "\U0001f30d" // globe europe/africa
+                              "\U0001f30e" // globe americas
+                              "\U0001f30f" // globe asia/australia
+                              "\U0001F946" // rifle
+                              "\U0001f4a3" // bomb
+                              "\U0001f5e1" // dagger
+                              "\U0001F52B" // pistol
+                              "\u2697\ufe0f" // alembic
+                              "\u269b\ufe0f" // atom
+                              "\u2622\ufe0f" // radiation sign
+                              "\u2623\ufe0f" // biohazard
+                              "\U0001F33F" // herb
+                              "\U0001F3B1" // billiards
+                              "\U0001F3E7" // automated teller machine
+                              "\U0001F489" // syringe
+                              "\U0001F48A" // pill
                               "\U0001f574\ufe0f" // man in suit levitating
-                              "📡🤻🦑🇮🇱🇦🇶"
+                              "\U0001F4E1" // satellite antenna
+                              "\U0001F93B" // modern pentathlon
+                              "\U0001F991" // squid
+                              "\U0001f1e6\U0001f1f6" // regional indicators AQ (antarctica)
                               "\U0001f469\u200d\U0001f52c" // woman scientist
                               "\U0001faa4" // mouse trap
                               "\U0001f6b1" // non-potable water
@@ -113,8 +139,8 @@ emoji_viz(struct ncplane* n){
                               "\U0001f3f4\u200d\u2620\ufe0f" // pirate flag
                               "\U0001f93d\U0001f3fc\u200d\u2640\ufe0f" // type-3 woman playing water polo
                               ;
-  ncplane_set_bg_rgb(n, 0);
-  int bytes;
+  ncplane_set_bg_default(n);
+  size_t bytes;
   for(const char* e = emoji ; *e ; e += bytes){
     if(ncplane_putegc(n, e, &bytes) < 0){
       if(ncplane_putchar(n, ' ') < 0){
@@ -122,7 +148,14 @@ emoji_viz(struct ncplane* n){
       }
     }
   }
-  finish_line(n);
+  unsigned x;
+  ncplane_cursor_yx(n, NULL, &x);
+  while(x++ < 80){
+    ncplane_putchar(n, ' ');
+  }
+  if(ncplane_dim_x(n) > 80){
+    ncplane_putchar(n, '\n');
+  }
   return 0;
 }
 
@@ -131,19 +164,24 @@ static int
 legacy_viz(struct ncplane* n, const char* indent, const wchar_t* eighths,
            const wchar_t* anglesr, const wchar_t* anglesl){
   ncplane_printf(n, "%s ", indent);
-  for(const wchar_t* e = eighths ; *e ; ++e){
-    if(ncplane_putwc(n, *e) <= 0){
+  unsigned wchars;
+  for(const wchar_t* e = eighths ; *e ; e += wchars){
+    if(ncplane_putwc_utf32(n, e, &wchars) <= 0){
       ncplane_putchar(n, ' ');
     }
   }
   ncplane_putchar(n, ' ');
-  for(const wchar_t* r = anglesr ; *r ; ++r){
-    if(ncplane_putwc(n, *r) <= 0){
+  const wchar_t* r = anglesr;
+  const wchar_t* l = anglesl;
+  while(*r){
+    if(ncplane_putwc_utf32(n, r, &wchars) <= 0){
       ncplane_putchar(n, ' ');
     }
-    if(ncplane_putwc(n, anglesl[r - anglesr]) <= 0){
+    r += wchars;
+    if(ncplane_putwc_utf32(n, l, &wchars) <= 0){
       ncplane_putchar(n, ' ');
     }
+    l += wchars;
     ncplane_putchar(n, ' ');
   }
   return 0;
@@ -159,8 +197,9 @@ sex_viz(struct ncplane* n, const wchar_t* sex, wchar_t r, const wchar_t* post){
   if(ncplane_putwc(n, r) <= 0){
     ncplane_putchar(n, ' ');
   }
-  for(const wchar_t* p = post ; *p ; ++p){
-    if(ncplane_putwc(n, *p) <= 0){
+  unsigned wchars = 0;
+  for(const wchar_t* p = post ; *p ; p += wchars){
+    if(ncplane_putwc_utf32(n, p, &wchars) <= 0){
       ncplane_putchar(n, ' ');
     }
   }
@@ -174,7 +213,7 @@ triviz(struct ncplane* n, const wchar_t* w1, const wchar_t* w2, const wchar_t* w
        const wchar_t* wa, const wchar_t* wb, const wchar_t* wc,
        const wchar_t* wd, const wchar_t* we, const wchar_t* wf,
        const wchar_t* w10, const wchar_t* w11, const wchar_t* w12,
-       const wchar_t* w13, const wchar_t* w14){
+       const wchar_t* w13, const wchar_t* w14, const wchar_t* w15){
   wvizn(n, w1, 2);
   ncplane_putchar(n, ' ');
   wvizn(n, w2, 2);
@@ -193,22 +232,20 @@ triviz(struct ncplane* n, const wchar_t* w1, const wchar_t* w2, const wchar_t* w
   wvizn(n, w9, 2);
   wvizn(n, wa, 1);
   ncplane_putchar(n, ' ');
-  ncplane_putchar(n, ' ');
   wvizn(n, wb, 2);
   wvizn(n, wc, 1);
   ncplane_putchar(n, ' ');
   wvizn(n, wd, 2);
   wvizn(n, we, 1);
   ncplane_putchar(n, ' ');
-  ncplane_putchar(n, ' ');
   wvizn(n, wf, 2);
   wvizn(n, w10, 1);
   ncplane_putchar(n, ' ');
   wvizn(n, w11, 2);
   wvizn(n, w12, 1);
-  ncplane_putchar(n, ' ');
-  wviz(n, w13);
+  wvizn(n, w13, 3); // chess
   wviz(n, w14);
+  wviz(n, w15);
 }
 
 static void
@@ -227,102 +264,84 @@ vertviz(struct ncplane* n, wchar_t l, wchar_t li, wchar_t ri, wchar_t r,
     ncplane_putchar(n, ' ');
   }
   wviz(n, trail);
-  ncplane_putchar(n, '\n');
+  if(ncplane_dim_x(n) > 80){
+    ncplane_putchar(n, '\n');
+  }
 }
 
 static int
-unicodedumper(struct ncplane* n, const char* indent){
+unicodedumper(struct notcurses* nc, struct ncplane* n, const char* indent){
   if(notcurses_canutf8(ncplane_notcurses_const(n))){
     // all NCHALFBLOCKS are contained within NCQUADBLOCKS
     ncplane_printf(n, "%s%ls⎧", indent, NCQUADBLOCKS);
-    sex_viz(n, NCSEXBLOCKS, L'⎫', L"♠♥🯰🯱🯲🯳🯴🯵🯶🯷🯸🯹\u2157\u2158\u2159\u215a\u215b");
-    vertviz(n, L'⎧', NCEIGHTHSR[0], NCEIGHTHSL[0], L'⎫', L"┌╥─╥─╥┐🭩");
+    // 🯰🯱🯲🯳🯴🯵🯶🯷🯸🯹 (on Windows, these will be encoded as UTF-16 surrogate
+    // pairs due to a 16-bit wchar_t.
+    sex_viz(n, NCSEXBLOCKS, L'⎫', L"♠♥" NCSEGDIGITS L"\u2157\u2158\u2159\u215a\u215b");
+    vertviz(n, L'⎧', NCEIGHTHSR[0], NCEIGHTHSL[0], L'⎫', L"┌╥─╥─╥┐🭩⎛⎞");
     ncplane_printf(n, "%s╲╿╱ ◨◧ ◪◩ ◖◗ ⫷⫸ ⎩", indent);
     sex_viz(n, &NCSEXBLOCKS[32], L'⎭', L"♦♣\u00bc\u00bd\u00be\u2150\u2151\u2152\u2153\u2154\u2155\u2156\u215c\u215d\u215e\u215f\u2189");
-    vertviz(n, L'⎪', NCEIGHTHSR[1], NCEIGHTHSL[1], L'⎪', L"├╜╓╫╖╙┤🭫");
+    vertviz(n, L'⎪', NCEIGHTHSR[1], NCEIGHTHSL[1], L'⎪', L"├╜╓╫╖╙┤🭫⎜⎟");
     ncplane_printf(n, "%s╾╳╼ ", indent);
     triviz(n, NCWHITESQUARESW, NCWHITECIRCLESW, NCDIAGONALSW, &NCDIAGONALSW[4],
            NCCIRCULARARCSW, NCWHITETRIANGLESW, NCSHADETRIANGLESW, NCBLACKTRIANGLESW,
            NCBOXLIGHTW, &NCBOXLIGHTW[4], NCBOXHEAVYW, &NCBOXHEAVYW[4], NCBOXROUNDW,
            &NCBOXROUNDW[4], NCBOXDOUBLEW, &NCBOXDOUBLEW[4], NCBOXOUTERW, &NCBOXOUTERW[4],
-           L"⩘▵△▹▷▿▽◃◁", NCARROWW);
-    vertviz(n, L'⎪', NCEIGHTHSR[2], NCEIGHTHSL[2], L'⎪', L"├─╨╫╨─┤┇");
+           NCCHESSBLACK, L"⩘▵△▹▷▿▽◃◁", NCARROWW);
+    vertviz(n, L'⎪', NCEIGHTHSR[2], NCEIGHTHSL[2], L'⎪', L"├─╨╫╨─┤┇⎜⎟");
     ncplane_printf(n, "%s╱╽╲ ", indent);
     triviz(n, &NCWHITESQUARESW[2], &NCWHITECIRCLESW[2], &NCDIAGONALSW[2], &NCDIAGONALSW[6],
            &NCCIRCULARARCSW[2], &NCWHITETRIANGLESW[2], &NCSHADETRIANGLESW[2], &NCBLACKTRIANGLESW[2],
            &NCBOXLIGHTW[2], &NCBOXLIGHTW[5], &NCBOXHEAVYW[2], &NCBOXHEAVYW[5], &NCBOXROUNDW[2],
            &NCBOXROUNDW[5], &NCBOXDOUBLEW[2], &NCBOXDOUBLEW[5], &NCBOXOUTERW[2], &NCBOXOUTERW[5],
-           L"⩗▴⏶⯅▲▸⏵⯈▶", L"▾⏷⯆▼◂⏴⯇◀");
-    vertviz(n, L'⎪', NCEIGHTHSR[3], NCEIGHTHSL[3], L'⎪', L"╞═╤╬╤═╡┋");
+           &NCCHESSBLACK[3], L"⩗▴⏶⯅▲▸⏵⯈▶", L"▾⏷⯆▼◂⏴⯇◀");
+    vertviz(n, L'⎪', NCEIGHTHSR[3], NCEIGHTHSL[3], L'⎪', L"╞═╤╬╤═╡┋⎜⎟");
     braille_viz(n, L'⎡', NCBRAILLEEGCS, L'⎤', indent, L"⎨⎬", NCEIGHTHSR[4], NCEIGHTHSL[4],
-                L"╞╕╘╬╛╒╡┊");
+                L"╞╕╘╬╛╒╡┊⎜⎟");
     braille_viz(n, L'⎢', &NCBRAILLEEGCS[64], L'⎥', indent, L"⎪⎪", NCEIGHTHSR[5], NCEIGHTHSL[5],
-                L"└┴─╨─┴┘╏");
+                L"└┴─╨─┴┘╏⎝⎠");
     braille_viz(n, L'⎢', &NCBRAILLEEGCS[128], L'⎥', indent, L"⎪⎪", NCEIGHTHSR[6], NCEIGHTHSL[6],
-                L"╭──╮⟬⟭╔╗");
+                L"╭──╮⟬⟭╔╗≶≷");
     braille_viz(n, L'⎣', &NCBRAILLEEGCS[192], L'⎦', indent, L"⎪⎪", NCEIGHTHSR[7], NCEIGHTHSL[7],
-                L"│╭╮│╔═╝║");
+                L"│╭╮│╔═╝║⊆⊇");
     legacy_viz(n, indent, L"▔🭶🭷🭸🭹🭺🭻▁", NCANGLESBR, NCANGLESBL);
     wviz(n, NCDIGITSSUBW);
     wviz(n, L" ⎛");
     wviz(n, NCEIGHTHSB);
-    wviz(n, L" ⎞⎪🭨🭪⎪╰╯││║╔═╝");
-    ncplane_putchar(n, '\n');
+    // 🭫⎞⎪🭨🭪⎪╰╯││║╔═╝⊴⊵
+    wviz(n, L"\U0001FB6B\u239e⎪🭨🭪⎪╰╯││║╔═╝⊴⊵");
+    if(ncplane_dim_x(n) > 80){
+      ncplane_putchar(n, '\n');
+    }
     legacy_viz(n, indent, L"▏🭰🭱🭲🭳🭴🭵▕", NCANGLESTR, NCANGLESTL);
     wviz(n, NCDIGITSSUPERW);
     wviz(n, L" ⎝");
     wviz(n, NCEIGHTHST);
-    wviz(n, L" ⎠⎩🭪🭨⎭⧒⧑╰╯╚╝❨❩");
-    ncplane_putchar(n, '\n');
+    // 🭩⎠⎩🭪🭨⎭⧒⧑╰╯╚╝❨❩⟃⟄
+    wviz(n, L"\U0001FB69\u23a0⎩🭪🭨⎭⧒⧑╰╯╚╝❨❩⟃⟄");
+    if(ncplane_dim_x(n) > 80){
+      ncplane_putchar(n, '\n');
+    }
     emoji_viz(n);
-    int y, x;
+    unsigned y, x;
     ncplane_cursor_yx(n, &y, &x);
-    /*
-    ncplane_printf_aligned(n, y - 9, NCALIGN_RIGHT, "⎡⎛⎞⎤");
-    ncplane_printf_aligned(n, y - 8, NCALIGN_RIGHT, "⎢⎜⎟⎥");
-    ncplane_printf_aligned(n, y - 7, NCALIGN_RIGHT, "⎢⎜⎟⎥");
-    ncplane_printf_aligned(n, y - 6, NCALIGN_RIGHT, "⎢⎜⎟⎥");
-    ncplane_printf_aligned(n, y - 5, NCALIGN_RIGHT, "⎢⎜⎟⎥");
-    ncplane_printf_aligned(n, y - 4, NCALIGN_RIGHT, "⎢⎜⎟⎥");
-    ncplane_printf_aligned(n, y - 3, NCALIGN_RIGHT, "⎢⎜⎟⎥");
-    ncplane_printf_aligned(n, y - 2, NCALIGN_RIGHT, "⎢⎜⎟⎥");
-    ncplane_printf_aligned(n, y - 1, NCALIGN_RIGHT, "⎣⎝⎠⎦");
-    */
-    // the symbols for legacy computing
-    ncplane_cursor_move_yx(n, y - 3, 0);
-    uint32_t ul = NCCHANNEL_INITIALIZER(0x30, 0x30, 0x30);
-    uint32_t lr = NCCHANNEL_INITIALIZER(0x80, 0x80, 0x80);
-    ncplane_stain(n, y - 2, 65, ul, lr, ul, lr);
-    // the horizontal eighths
-    ul = NCCHANNEL_INITIALIZER(0x02, 0x8a, 0x0f);
-    lr = NCCHANNEL_INITIALIZER(0x08, 0x3d, 0x3b);
-    ncplane_cursor_move_yx(n, y - 11, 66);
-    ncplane_stain(n, y - 2, 77, ul, ul, lr, lr);
-    // the braille
-    ncplane_cursor_move_yx(n, y - 7, 0);
-    ul = NCCHANNEL_INITIALIZER(0x2f, 0x25, 0x24);
-    lr = NCCHANNEL_INITIALIZER(0x74, 0x25, 0x2f);
-    ncplane_stain(n, y - 4, 65, ul, lr, ul, lr);
-    // the sextants
-    ncplane_cursor_move_yx(n, y - 11, 27);
-    lr = NCCHANNEL_INITIALIZER(0x7B, 0x68, 0xEE);
-    ul = NCCHANNEL_INITIALIZER(0x19, 0x19, 0x70);
-    ncplane_stain(n, y - 10, 57, lr, ul, lr, ul);
-    // the boxes + quadrants
-    ncplane_cursor_move_yx(n, y - 11, 0);
-    ncplane_stain(n, y - 8, 65, lr, ul, lr, ul);
-    // the capabilities
-    ul = NCCHANNEL_INITIALIZER(0x1B, 0xb8, 0x8E);
-    lr = NCCHANNEL_INITIALIZER(0x19, 0x19, 0x70);
-    ncplane_cursor_move_yx(n, y - 16, 0);
-    ncplane_stain(n, y - 12, 77, lr, ul, lr, ul);
-
-    ncplane_set_fg_rgb(n, 0x00c080);
+    ncpalette* npal = ncpalette_new(nc);
+    unsigned r, g, b;
+    // if we weren't able to detect the palette, we'll just have a black
+    // background throughout the unicode section. otherwise, our gradient
+    // will be based off the existing palette.
+    ncpalette_get_rgb8(npal, 4, &r, &g, &b);
+    uint64_t ur = NCCHANNELS_INITIALIZER(0xff, 0xff, 0xff, r, g, b);
+    ncpalette_get_rgb8(npal, 5, &r, &g, &b);
+    uint64_t lr = NCCHANNELS_INITIALIZER(0xff, 0xff, 0xff, r, g, b);
+    ncpalette_get_rgb8(npal, 6, &r, &g, &b);
+    uint64_t ul = NCCHANNELS_INITIALIZER(0xff, 0xff, 0xff, r, g, b);
+    ncpalette_get_rgb8(npal, 7, &r, &g, &b);
+    uint64_t ll = NCCHANNELS_INITIALIZER(0xff, 0xff, 0xff, r, g, b);
+    ncpalette_free(npal);
+    ncplane_stain(n, y - 16, 0, 15, 80, ul, ur, ll, lr);
     ncplane_set_styles(n, NCSTYLE_BOLD | NCSTYLE_ITALIC);
-    ncplane_cursor_move_yx(n, y - 12, 54);
-    ncplane_set_bg_rgb(n, 0);
+    ncplane_cursor_move_yx(n, y - 12, 55);
     wviz(n, L"🯁🯂🯃https://notcurses.com");
-    ncplane_set_fg_default(n);
     ncplane_set_styles(n, NCSTYLE_NONE);
   }
   return 0;
@@ -330,27 +349,26 @@ unicodedumper(struct ncplane* n, const char* indent){
 
 static int
 display_logo(struct ncplane* n, const char* path){
-  int cpixy, cpixx;
-  ncplane_pixelgeom(n, NULL, NULL, &cpixy, &cpixx, NULL, NULL);
+  unsigned cpixy, cpixx;
+  ncplane_pixel_geom(n, NULL, NULL, &cpixy, &cpixx, NULL, NULL);
   struct ncvisual* ncv = ncvisual_from_file(path);
   if(ncv == NULL){
     return -1;
   }
-  // FIXME ought be exactly 4:1
   if(ncvisual_resize(ncv, 3 * cpixy, 24 * cpixx)){
     ncvisual_destroy(ncv);
     return -1;
   }
-  int y;
+  unsigned y;
   ncplane_cursor_yx(n, &y, NULL);
   struct ncvisual_options vopts = {
     .n = n,
     .y = y - 3,
-    .x = 54,
+    .x = 55,
     .blitter = NCBLIT_PIXEL,
     .flags = NCVISUAL_OPTION_CHILDPLANE | NCVISUAL_OPTION_NODEGRADE,
   };
-  struct ncplane* bitm = ncvisual_render(ncplane_notcurses(n), ncv, &vopts);
+  struct ncplane* bitm = ncvisual_blit(ncplane_notcurses(n), ncv, &vopts);
   if(bitm == NULL){
     ncvisual_destroy(ncv);
     return -1;
@@ -361,57 +379,68 @@ display_logo(struct ncplane* n, const char* path){
 
 static void
 tinfo_debug_bitmaps(struct ncplane* n, const tinfo* ti, const char* indent){
-  ncplane_set_fg_rgb8(n, 0xc4, 0x5a, 0xec);
-  ncplane_printf(n, "%sdefbg 0x%06lx %sconsidered transparent", indent,
-                 ti->bg_collides_default & 0xfffffful,
-                 (ti->bg_collides_default & 0x01000000) ? "" : "not ");
+  uint32_t fg = 0;
+  int r = notcurses_default_foreground(ncplane_notcurses(n), &fg);
+  if(r){
+    ncplane_printf(n, "%sno known default fg ", indent);
+  }else{
+    ncplane_printf(n, "%sdefault fg 0x%06x ", indent, fg);
+  }
+  uint32_t bg = 0;
+  r = notcurses_default_background(ncplane_notcurses(n), &bg);
+  if(r){
+    ncplane_printf(n, "no known default bg");
+  }else{
+    ncplane_printf(n, "default bg 0x%06x", bg);
+  }
+  tinfo_debug_cap(n, " pmouse", ti->pixelmice);
   finish_line(n);
-  ncplane_set_fg_default(n);
-  ncplane_set_fg_rgb(n, 0x5efa80);
-  if(!ti->pixel_draw){
-    ncplane_printf(n, "%sno bitmap graphics detected", indent);
-  }else{ // we do have support; draw one
-    if(ti->color_registers){
+  ncpixelimpl_e blit = notcurses_check_pixel_support(ncplane_notcurses(n));
+  switch(blit){
+    case NCPIXEL_NONE:
+      ncplane_printf(n, "%sno bitmap graphics detected", indent);
+      break;
+    case NCPIXEL_SIXEL:
       if(ti->sixel_maxy){
         ncplane_printf(n, "%smax sixel size: %dx%d colorregs: %u",
                       indent, ti->sixel_maxy, ti->sixel_maxx, ti->color_registers);
       }else{
         ncplane_printf(n, "%ssixel colorregs: %u", indent, ti->color_registers);
       }
-    }else if(ti->linux_fb_fd >= 0){
+      break;
+    case NCPIXEL_LINUXFB:
       ncplane_printf(n, "%sframebuffer graphics supported", indent);
-    }else if(ti->pixel_move == NULL){
-      ncplane_printf(n, "%siTerm2 graphics support", indent);
-    }else if(ti->sixel_maxy_pristine){
+      break;
+    case NCPIXEL_ITERM2:
+      ncplane_printf(n, "%siTerm2 graphics supported", indent);
+      break;
+    case NCPIXEL_KITTY_STATIC:
       ncplane_printf(n, "%srgba pixel graphics support", indent);
-    }else{
-      ncplane_printf(n, "%srgba pixel animation support", indent);
-    }
+      break;
+    case NCPIXEL_KITTY_ANIMATED:
+      ncplane_printf(n, "%s1st gen rgba pixel animation support", indent);
+      break;
+    case NCPIXEL_KITTY_SELFREF:
+      ncplane_printf(n, "%s2nd gen rgba pixel animation support", indent);
+      break;
   }
   finish_line(n);
-  /*
-  ncplane_putstr(n, "\U0001F918");
-  ncplane_putstr(n, "\U0001F918\u200d\U0001F3FB");
-  ncplane_putstr(n, "\U0001F918\u200d\U0001F3FC");
-  ncplane_putstr(n, "\U0001F918\u200d\U0001F3FD");
-  ncplane_putstr(n, "\U0001F918\u200d\U0001F3FE");
-  ncplane_putstr(n, "\U0001F918\u200d\U0001F3FF");
-  */
 }
 
 static void
 tinfo_debug_caps(struct ncplane* n, const tinfo* ti, const char* indent){
   ncplane_printf(n, "%s", indent);
-  tinfo_debug_cap(n, "af", get_escape(ti, ESCAPE_SETAF), ' ');
-  tinfo_debug_cap(n, "ab", get_escape(ti, ESCAPE_SETAB), ' ');
-  tinfo_debug_cap(n, "sum", get_escape(ti, ESCAPE_BSUM), ' ');
-  tinfo_debug_cap(n, "cup", get_escape(ti, ESCAPE_CUP), ' ');
-  tinfo_debug_cap(n, "vpa", get_escape(ti, ESCAPE_VPA), ' ');
-  tinfo_debug_cap(n, "hpa", get_escape(ti, ESCAPE_HPA), ' ');
-  tinfo_debug_cap(n, "sgr0", get_escape(ti, ESCAPE_SGR0), ' ');
-  tinfo_debug_cap(n, "op", get_escape(ti, ESCAPE_OP), ' ');
-  tinfo_debug_cap(n, "fgop", get_escape(ti, ESCAPE_FGOP), ' ');
-  tinfo_debug_cap(n, "bgop", get_escape(ti, ESCAPE_BGOP), ' ');
+  tinfo_debug_cap(n, "af", get_escape(ti, ESCAPE_SETAF));
+  tinfo_debug_cap(n, "ab", get_escape(ti, ESCAPE_SETAB));
+  tinfo_debug_cap(n, "sum", get_escape(ti, ESCAPE_BSUM));
+  tinfo_debug_cap(n, "vpa", get_escape(ti, ESCAPE_VPA));
+  tinfo_debug_cap(n, "hpa", get_escape(ti, ESCAPE_HPA));
+  tinfo_debug_cap(n, "sgr0", get_escape(ti, ESCAPE_SGR0));
+  tinfo_debug_cap(n, "op", get_escape(ti, ESCAPE_OP));
+  tinfo_debug_cap(n, "fgop", get_escape(ti, ESCAPE_FGOP));
+  tinfo_debug_cap(n, "bgop", get_escape(ti, ESCAPE_BGOP));
+  tinfo_debug_cap(n, "bce", ti->bce);
+  tinfo_debug_cap(n, "rect", get_escape(ti, ESCAPE_DECERA));
   finish_line(n);
 }
 
@@ -424,20 +453,23 @@ tinfo_debug_styles(const notcurses* nc, struct ncplane* n, const char* indent){
   tinfo_debug_style(n, "struck", NCSTYLE_STRUCK, ' ');
   tinfo_debug_style(n, "ucurl", NCSTYLE_UNDERCURL, ' ');
   tinfo_debug_style(n, "uline", NCSTYLE_UNDERLINE, ' ');
-  tinfo_debug_cap(n, "u7", get_escape(ti, ESCAPE_U7), ' ');
-  tinfo_debug_cap(n, "ccc", ti->caps.can_change_colors, ' ');
-  tinfo_debug_cap(n, "rgb", ti->caps.rgb, ' ');
+  tinfo_debug_cap(n, "u7", get_escape(ti, ESCAPE_U7));
+  tinfo_debug_cap(n, "ccc", ti->caps.can_change_colors);
+  tinfo_debug_cap(n, "rgb", ti->caps.rgb);
+  tinfo_debug_cap(n, "el", get_escape(ti, ESCAPE_EL));
   finish_line(n);
-  ncplane_set_fg_default(n);
   ncplane_putstr(n, indent);
-  tinfo_debug_cap(n, "utf8", ti->caps.utf8, ' ');
-  tinfo_debug_cap(n, "quad", ti->caps.quadrants, ' ');
-  tinfo_debug_cap(n, "sex", ti->caps.sextants, ' ');
-  tinfo_debug_cap(n, "braille", ti->caps.braille, ' ');
-  tinfo_debug_cap(n, "images", notcurses_canopen_images(nc), ' ');
-  tinfo_debug_cap(n, "video", notcurses_canopen_videos(nc), ' ');
+  tinfo_debug_cap(n, "utf8", notcurses_canutf8(nc));
+  tinfo_debug_cap(n, "2x1", notcurses_canhalfblock(nc));
+  tinfo_debug_cap(n, "2x2", notcurses_canquadrant(nc));
+  tinfo_debug_cap(n, "3x2", notcurses_cansextant(nc));
+  tinfo_debug_cap(n, "4x2", notcurses_canbraille(nc));
+  tinfo_debug_cap(n, "img", notcurses_canopen_images(nc));
+  tinfo_debug_cap(n, "vid", notcurses_canopen_videos(nc));
+  tinfo_debug_cap(n, "indn", get_escape(ti, ESCAPE_INDN));
+  tinfo_debug_cap(n, "gpm", ti->gpmfd >= 0);
+  tinfo_debug_cap(n, "kbd", ti->kittykbdsupport);
   finish_line(n);
-  ncplane_set_fg_default(n);
 }
 
 static int
@@ -452,7 +484,8 @@ int main(int argc, const char** argv){
   notcurses_options nopts = {
     .flags = NCOPTION_NO_ALTERNATE_SCREEN
              | NCOPTION_PRESERVE_CURSOR
-             | NCOPTION_NO_CLEAR_BITMAPS,
+             | NCOPTION_NO_CLEAR_BITMAPS
+             | NCOPTION_DRAIN_INPUT,
   };
   if(argc > 2){
     usage(*argv, stderr, EXIT_FAILURE);
@@ -467,16 +500,31 @@ int main(int argc, const char** argv){
   if(nc == NULL){
     return EXIT_FAILURE;
   }
+  // so that we know whether we're talking to gpm
+  notcurses_mice_enable(nc, NCMICE_ALL_EVENTS);
   const char indent[] = "";
-  struct ncplane* stdn = notcurses_stdplane(nc);
+  unsigned dimx;
+  struct ncplane* stdn = notcurses_stddim_yx(nc, NULL, &dimx);
+  if(dimx < 80){
+    ncplane_set_fg_rgb(stdn, 0xff5349);
+    ncplane_set_styles(stdn, NCSTYLE_BOLD);
+    ncplane_putstr(stdn, "This program requires at least 80 columns.\n");
+    notcurses_render(nc);
+    notcurses_stop(nc);
+    return EXIT_FAILURE;
+  }
+  ncplane_set_fg_alpha(stdn, NCALPHA_HIGHCONTRAST);
+  ncplane_set_fg_rgb(stdn, 0xffffff);
   ncplane_set_scrolling(stdn, true);
   tinfo_debug_caps(stdn, &nc->tcache, indent);
   tinfo_debug_styles(nc, stdn, indent);
   tinfo_debug_bitmaps(stdn, &nc->tcache, indent);
-  unicodedumper(stdn, indent);
-  char* path = prefix_data("notcurses.png");
+  unicodedumper(nc, stdn, indent);
+  char* path = notcurses_data_path(NULL, "notcurses.png");
   if(path){
-    display_logo(stdn, path); // let it fail
+    if(notcurses_canpixel(nc)){
+      display_logo(stdn, path); // let it fail
+    }
     free(path);
   }
   if(notcurses_render(nc)){
