@@ -67,24 +67,18 @@ tria_blit_ascii(ncplane* nc, int linesize, const void* data,
                 int leny, int lenx, const blitterargs* bargs){
 //fprintf(stderr, "ASCII %d X %d @ %d X %d (%p) place: %d X %d\n", leny, lenx, bargs->begy, bargs->begx, data, bargs->u.cell.placey, bargs->u.cell.placex);
   const bool blendcolors = bargs->flags & NCVISUAL_OPTION_BLEND;
-  int dimy, dimx, x, y;
+  unsigned dimy, dimx, x, y;
   int total = 0; // number of cells written
   ncplane_dim_yx(nc, &dimy, &dimx);
   // FIXME not going to necessarily be safe on all architectures hrmmm
   const unsigned char* dat = data;
   int visy = bargs->begy;
   for(y = bargs->u.cell.placey ; visy < (bargs->begy + leny) && y < dimy ; ++y, ++visy){
-    if(y < 0){
-      continue;
-    }
     if(ncplane_cursor_move_yx(nc, y, bargs->u.cell.placex < 0 ? 0 : bargs->u.cell.placex)){
       return -1;
     }
     int visx = bargs->begx;
     for(x = bargs->u.cell.placex ; visx < (bargs->begx + lenx) && x < dimx ; ++x, ++visx){
-      if(x < 0){
-        continue;
-      }
       const unsigned char* rgbbase_up = dat + (linesize * visy) + (visx * 4);
 //fprintf(stderr, "[%04d/%04d] lsize: %d %02x %02x %02x %02x\n", y, x, linesize, rgbbase_up[0], rgbbase_up[1], rgbbase_up[2], rgbbase_up[3]);
       nccell* c = ncplane_cell_ref_yx(nc, y, x);
@@ -100,6 +94,7 @@ tria_blit_ascii(ncplane* nc, int linesize, const void* data,
         nccell_set_bg_alpha(c, NCALPHA_TRANSPARENT);
         nccell_set_fg_alpha(c, NCALPHA_TRANSPARENT);
         cell_set_blitquadrants(c, 0, 0, 0, 0);
+        nccell_release(nc, c);
       }else{
         nccell_set_fg_rgb8(c, rgbbase_up[0], rgbbase_up[1], rgbbase_up[2]);
         nccell_set_bg_rgb8(c, rgbbase_up[0], rgbbase_up[1], rgbbase_up[2]);
@@ -122,24 +117,18 @@ tria_blit(ncplane* nc, int linesize, const void* data, int leny, int lenx,
   const bool blendcolors = bargs->flags & NCVISUAL_OPTION_BLEND;
 //fprintf(stderr, "HALF %d X %d @ %d X %d (%p) place: %d X %d\n", leny, lenx, bargs->begy, bargs->begx, data, bargs->u.cell.placey, bargs->u.cell.placex);
   uint32_t transcolor = bargs->transcolor;
-  int dimy, dimx, x, y;
+  unsigned dimy, dimx, x, y;
   int total = 0; // number of cells written
   ncplane_dim_yx(nc, &dimy, &dimx);
   // FIXME not going to necessarily be safe on all architectures hrmmm
   const unsigned char* dat = data;
   int visy = bargs->begy;
   for(y = bargs->u.cell.placey ; visy < (bargs->begy + leny) && y < dimy ; ++y, visy += 2){
-    if(y < 0){
-      continue;
-    }
     if(ncplane_cursor_move_yx(nc, y, bargs->u.cell.placex < 0 ? 0 : bargs->u.cell.placex)){
       return -1;
     }
     int visx = bargs->begx;
     for(x = bargs->u.cell.placex ; visx < (bargs->begx + lenx) && x < dimx ; ++x, ++visx){
-      if(x < 0){
-        continue;
-      }
       const unsigned char* rgbbase_up = dat + (linesize * visy) + (visx * 4);
       const unsigned char* rgbbase_down = zeroes;
       if(visy < bargs->begy + leny - 1){
@@ -159,6 +148,7 @@ tria_blit(ncplane* nc, int linesize, const void* data, int leny, int lenx,
         nccell_set_bg_alpha(c, NCALPHA_TRANSPARENT);
         if(rgba_trans_q(rgbbase_up, transcolor) && rgba_trans_q(rgbbase_down, transcolor)){
           nccell_set_fg_alpha(c, NCALPHA_TRANSPARENT);
+          nccell_release(nc, c);
         }else if(rgba_trans_q(rgbbase_up, transcolor)){ // down has the color
           if(pool_blit_direct(&nc->pool, c, "\u2584", strlen("\u2584"), 1) <= 0){
             return -1;
@@ -453,7 +443,7 @@ quadrant_blit(ncplane* nc, int linesize, const void* data, int leny, int lenx,
               const blitterargs* bargs){
   const unsigned nointerpolate = bargs->flags & NCVISUAL_OPTION_NOINTERPOLATE;
   const bool blendcolors = bargs->flags & NCVISUAL_OPTION_BLEND;
-  int dimy, dimx, x, y;
+  unsigned dimy, dimx, x, y;
   int total = 0; // number of cells written
   ncplane_dim_yx(nc, &dimy, &dimx);
 //fprintf(stderr, "quadblitter %dx%d -> %d/%d+%d/%d\n", leny, lenx, dimy, dimx, bargs->u.cell.placey, bargs->u.cell.placex);
@@ -461,17 +451,11 @@ quadrant_blit(ncplane* nc, int linesize, const void* data, int leny, int lenx,
   const unsigned char* dat = data;
   int visy = bargs->begy;
   for(y = bargs->u.cell.placey ; visy < (bargs->begy + leny) && y < dimy ; ++y, visy += 2){
-    if(y < 0){
-      continue;
-    }
     if(ncplane_cursor_move_yx(nc, y, bargs->u.cell.placex < 0 ? 0 : bargs->u.cell.placex)){
       return -1;
     }
     int visx = bargs->begx;
     for(x = bargs->u.cell.placex ; visx < (bargs->begx + lenx) && x < dimx ; ++x, visx += 2){
-      if(x < 0){
-        continue;
-      }
       const unsigned char* rgbbase_tl = dat + (linesize * visy) + (visx * 4);
       const unsigned char* rgbbase_tr = zeroes;
       const unsigned char* rgbbase_bl = zeroes;
@@ -516,6 +500,8 @@ quadrant_blit(ncplane* nc, int linesize, const void* data, int leny, int lenx,
           return -1;
         }
         ++total;
+      }else{
+        nccell_release(nc, c);
       }
     }
   }
@@ -613,7 +599,7 @@ sex_solver(const uint32_t rgbas[6], uint64_t* channels, unsigned blendcolors,
 }
 
 static const char*
-sex_trans_check(cell* c, const uint32_t rgbas[6], unsigned blendcolors,
+sex_trans_check(nccell* c, const uint32_t rgbas[6], unsigned blendcolors,
                 uint32_t transcolor, unsigned nointerpolate){
   // bit is *set* where sextant *is not*
   // 32: bottom right 16: bottom left
@@ -674,24 +660,18 @@ sextant_blit(ncplane* nc, int linesize, const void* data, int leny, int lenx,
              const blitterargs* bargs){
   const unsigned nointerpolate = bargs->flags & NCVISUAL_OPTION_NOINTERPOLATE;
   const bool blendcolors = bargs->flags & NCVISUAL_OPTION_BLEND;
-  int dimy, dimx, x, y;
+  unsigned dimy, dimx, x, y;
   int total = 0; // number of cells written
   ncplane_dim_yx(nc, &dimy, &dimx);
 //fprintf(stderr, "sexblitter %dx%d -> %d/%d+%d/%d\n", leny, lenx, dimy, dimx, bargs->u.cell.placey, bargs->u.cell.placex);
   const unsigned char* dat = data;
   int visy = bargs->begy;
   for(y = bargs->u.cell.placey ; visy < (bargs->begy + leny) && y < dimy ; ++y, visy += 3){
-    if(y < 0){
-      continue;
-    }
     if(ncplane_cursor_move_yx(nc, y, bargs->u.cell.placex < 0 ? 0 : bargs->u.cell.placex)){
       return -1;
     }
     int visx = bargs->begx;
     for(x = bargs->u.cell.placex ; visx < (bargs->begx + lenx) && x < dimx ; ++x, visx += 2){
-      if(x < 0){
-        continue;
-      }
       uint32_t rgbas[6] = { 0, 0, 0, 0, 0, 0 };
       memcpy(&rgbas[0], (dat + (linesize * visy) + (visx * 4)), sizeof(*rgbas));
       if(visx < bargs->begx + lenx - 1){
@@ -723,6 +703,8 @@ sextant_blit(ncplane* nc, int linesize, const void* data, int leny, int lenx,
           return -1;
         }
         ++total;
+      }else{
+        nccell_release(nc, c);
       }
     }
   }
@@ -748,24 +730,18 @@ static inline int
 braille_blit(ncplane* nc, int linesize, const void* data, int leny, int lenx,
              const blitterargs* bargs){
   const bool blendcolors = bargs->flags & NCVISUAL_OPTION_BLEND;
-  int dimy, dimx, x, y;
+  unsigned dimy, dimx, x, y;
   int total = 0; // number of cells written
   ncplane_dim_yx(nc, &dimy, &dimx);
   // FIXME not going to necessarily be safe on all architectures hrmmm
   const unsigned char* dat = data;
   int visy = bargs->begy;
   for(y = bargs->u.cell.placey ; visy < (bargs->begy + leny) && y < dimy ; ++y, visy += 4){
-    if(y < 0){
-      continue;
-    }
     if(ncplane_cursor_move_yx(nc, y, bargs->u.cell.placex < 0 ? 0 : bargs->u.cell.placex)){
       return -1;
     }
     int visx = bargs->begx;
     for(x = bargs->u.cell.placex ; visx < (bargs->begx + lenx) && x < dimx ; ++x, visx += 2){
-      if(x < 0){
-        continue;
-      }
       const uint32_t* rgbbase_l0 = (const uint32_t*)(dat + (linesize * visy) + (visx * 4));
       const uint32_t* rgbbase_r0 = &zeroes32;
       const uint32_t* rgbbase_l1 = &zeroes32;
@@ -917,51 +893,75 @@ void set_pixel_blitter(ncblitter blitfxn){
   b->blit = blitfxn;
 }
 
-const struct blitset* lookup_blitset(const tinfo* tcache, ncblitter_e setid, bool may_degrade) {
+const struct blitset* lookup_blitset(const tinfo* tcache, ncblitter_e setid,
+                                     bool may_degrade){
   if(setid == NCBLIT_DEFAULT){ // ought have resolved NCBLIT_DEFAULT before now
     return NULL;
   }
   // without braille support, NCBLIT_BRAILLE decays to NCBLIT_3x2
-  if(!tcache->caps.braille && setid == NCBLIT_BRAILLE){
-    if(may_degrade){
-      setid = NCBLIT_3x2;
-    }else{
+  if(setid == NCBLIT_BRAILLE){
+    if(tcache->caps.braille){
+      return &notcurses_blitters[setid - 1];
+    }else if(!may_degrade){
       return NULL;
     }
+    setid = NCBLIT_3x2;
   }
   // without bitmap support, NCBLIT_PIXEL decays to NCBLIT_3x2
-  if(!tcache->pixel_draw && setid == NCBLIT_PIXEL){
-    if(may_degrade){
-      setid = NCBLIT_3x2;
-    }else{
+  if(setid == NCBLIT_PIXEL){
+    if(tcache->pixel_draw || tcache->pixel_draw_late){
+      return &notcurses_blitters[setid - 1];
+    }else if(!may_degrade){
       return NULL;
     }
+    setid = NCBLIT_3x2;
+  }
+  // without eighths support, NCBLIT_8x1 decays to NCBLIT_4x1
+  if(setid == NCBLIT_8x1){ // plotter only
+    if(tcache->caps.quadrants){
+       return &notcurses_blitters[setid - 1];
+    }else if(!may_degrade){
+      return NULL;
+    }
+    setid = NCBLIT_4x1;
+  }
+  // without quarters support, NCBLIT_4x1 decays to NCBLIT_2x1
+  if(setid == NCBLIT_4x1){ // plotter only
+    if(tcache->caps.quadrants){
+       return &notcurses_blitters[setid - 1];
+    }else if(!may_degrade){
+      return NULL;
+    }
+    setid = NCBLIT_2x1;
   }
   // without sextant support, NCBLIT_3x2 decays to NCBLIT_2x2
-  if(!tcache->caps.sextants && setid == NCBLIT_3x2){
-    if(may_degrade){
-      setid = NCBLIT_2x2;
-    }else{
+  if(setid == NCBLIT_3x2){
+    if(tcache->caps.sextants){
+       return &notcurses_blitters[setid - 1];
+    }else if(!may_degrade){
       return NULL;
     }
+    setid = NCBLIT_2x2;
   }
   // without quadrant support, NCBLIT_2x2 decays to NCBLIT_2x1
-  if(!tcache->caps.quadrants && setid == NCBLIT_2x2){
-    if(may_degrade){
-      setid = NCBLIT_2x1;
-    }else{
+  if(setid == NCBLIT_2x2){
+    if(tcache->caps.quadrants){
+       return &notcurses_blitters[setid - 1];
+    }else if(!may_degrade){
       return NULL;
     }
+    setid = NCBLIT_2x1;
   }
-  // the only viable blitters in ASCII are NCBLIT_1x1 and NCBLIT_PIXEL
-  if(!tcache->caps.utf8 && (setid != NCBLIT_1x1 && setid != NCBLIT_PIXEL)){
-    if(may_degrade){
-      setid = NCBLIT_1x1;
-    }else{
+  // without halfblock support, NCBLIT_2x1 decays to NCBLIT_1x1
+  if(setid == NCBLIT_2x1){
+    if(tcache->caps.halfblocks){
+       return &notcurses_blitters[setid - 1];
+    }else if(!may_degrade){
       return NULL;
     }
+    setid = NCBLIT_1x1;
   }
-  assert(setid == notcurses_blitters[setid - 1].geom);
+  assert(NCBLIT_1x1 == setid);
   return &notcurses_blitters[setid - 1];
 }
 
@@ -997,6 +997,11 @@ const char* notcurses_str_blitter(ncblitter_e blitfxn){
 
 int ncblit_bgrx(const void* data, int linesize, const struct ncvisual_options* vopts){
   if(vopts->leny <= 0 || vopts->lenx <= 0){
+    logerror("invalid lengths %u %u", vopts->leny, vopts->lenx);
+    return -1;
+  }
+  if(vopts->n == NULL){
+    logerror("prohibited null plane");
     return -1;
   }
   void* rdata = bgra_to_rgba(data, vopts->leny, &linesize, vopts->lenx, 0xff);
@@ -1037,46 +1042,24 @@ int ncblit_rgb_packed(const void* data, int linesize,
 }
 
 int ncblit_rgba(const void* data, int linesize, const struct ncvisual_options* vopts){
-  if(vopts->flags > NCVISUAL_OPTION_BLEND){
-    fprintf(stderr, "Warning: unknown ncvisual options %016" PRIx64 "\n", vopts->flags);
-  }
-  if(linesize <= 0 || (size_t)linesize < vopts->lenx * sizeof(uint32_t)){
+  if(vopts->leny <= 0 || vopts->lenx <= 0){
+    logerror("invalid lengths %u %u", vopts->leny, vopts->lenx);
     return -1;
   }
-  struct ncplane* nc = vopts->n;
-  if(nc == NULL){
+  if(vopts->n == NULL){
+    logerror("prohibited null plane");
     return -1;
   }
-  int lenx = vopts->lenx;
-  int leny = vopts->leny;
-  int begy = vopts->begy;
-  int begx = vopts->begx;
-//fprintf(stderr, "render %dx%d+%dx%d %p\n", begy, begx, leny, lenx, ncv->data);
-  if(begy < 0 || begx < 0 || lenx < -1 || leny < -1){
+  struct ncvisual* ncv = ncvisual_from_rgba(data, vopts->leny, linesize, vopts->lenx);
+  if(ncv == NULL){
     return -1;
   }
-  ncblitter_e blitfxn;
-  if(!vopts || vopts->blitter == NCBLIT_DEFAULT){
-    blitfxn = ncvisual_media_defblitter(ncplane_notcurses(nc), NCSCALE_NONE);
-  }else{
-    blitfxn = vopts->blitter;
-  }
-  const bool degrade = !(vopts->flags & NCVISUAL_OPTION_NODEGRADE);
-  const notcurses* notc = ncplane_notcurses(nc);
-  const struct blitset* bset = lookup_blitset(&notc->tcache, blitfxn, degrade);
-  if(bset == NULL){
+  if(ncvisual_blit(ncplane_notcurses(vopts->n), ncv, vopts) == NULL){
+    ncvisual_destroy(ncv);
     return -1;
   }
-  blitterargs bargs = {
-    .flags = vopts->flags,
-    .u = {
-      .cell = {
-        .placey = vopts->y,
-        .placex = vopts->x,
-      },
-    },
-  };
-  return bset->blit(nc, linesize, data, leny, lenx, &bargs);
+  ncvisual_destroy(ncv);
+  return 0;
 }
 
 ncblitter_e ncvisual_media_defblitter(const notcurses* nc, ncscale_e scale){
