@@ -9,8 +9,9 @@
 #include <assert.h>
 #include <stdlib.h>
 #include <string.h>
-#include <unigbrk.h>
 #include <stdbool.h>
+#include <unigbrk.h>
+#include <unictype.h>
 #include "notcurses/notcurses.h"
 #include "compat/compat.h"
 #include "logging.h"
@@ -64,7 +65,8 @@ egcpool_grow(egcpool* pool, size_t len){
 
 // get the expected length of the encoded codepoint from the first byte of a
 // utf-8 character. if the byte is illegal as a first byte, 1 is returned.
-// Table 3.1B, Legal UTF8 Byte Sequences, Corrigendum #1: UTF-8 Shortest Form
+// Table 3.1B, Legal UTF8 Byte Sequences, Corrigendum #1: UTF-8 Shortest Form.
+// subsequent ("continuation") bytes must start with the bit pattern 10.
 static inline size_t
 utf8_codepoint_length(unsigned char c){
   if(c <= 0x7f){        // 0x000000...0x00007f
@@ -99,29 +101,47 @@ utf8_egc_len(const char* gcluster, int* colcount){
   wchar_t wc, prevw = 0;
   bool injoin = false;
   do{
-    r = mbrtowc(&wc, gcluster, MB_CUR_MAX, &mbt);
+    r = mbrtowc(&wc, gcluster, MB_LEN_MAX, &mbt);
     if(r < 0){
       // FIXME probably ought escape this somehow
-      logerror("Invalid UTF8: %s\n", gcluster);
+      logerror("invalid UTF8: %s", gcluster);
       return -1;
     }
     if(prevw && !injoin && uc_is_grapheme_break(prevw, wc)){
       break; // starts a new EGC, exit and do not claim
     }
-    int cols = wcwidth(wc);
-    if(cols < 0){
-      if(iswspace(wc)){ // newline or tab
-        return ret + 1;
+    int cols;
+    if(uc_is_property_variation_selector(wc)){ // ends EGC
+      ret += r;
+      break;
+    }else if(wc == L'\u200d' || injoin){ // ZWJ is iswcntrl, so check it first
+      injoin = true;
+      cols = 0;
+    }else{
+      cols = wcwidth(wc);
+      if(cols < 0){
+        injoin = false;
+        if(iswspace(wc)){ // newline or tab
+          *colcount = 1;
+          return ret + 1;
+        }
+        cols = 1;
+        if(iswcntrl(wc)){
+          logerror("prohibited or invalid unicode: 0x%08x", (unsigned)wc);
+          return -1;
+        }
       }
-      logerror("Prohibited or invalid Unicode: 0x%x\n", wc);
-      return -1;
     }
-    injoin = (wc == L'\u200d');
-    *colcount += cols;
+    if(*colcount == 0){
+      *colcount += cols;
+    }
     ret += r;
     gcluster += r;
-    prevw = wc;
+    if(!prevw){
+      prevw = wc;
+    }
   }while(r);
+  // FIXME what if injoin is set? incomplete EGC!
   return ret;
 }
 
@@ -160,9 +180,12 @@ egcpool_stash(egcpool* pool, const char* egc, size_t ulen){
   do{
     if(egcpool_alloc_justified(pool, len) || searched){
       if(!duplicated){
-        if((duplicated = strndup(egc, ulen)) == NULL){
+        // cast (and avoidance of strndup) to facilitate c++ inclusions
+        if((duplicated = (char *)malloc(ulen + 1)) == NULL){
           return -1;
         }
+        memcpy(duplicated, egc, ulen);
+        duplicated[ulen] = '\0';
       }
       if(egcpool_grow(pool, len) && searched){
         free(duplicated);
@@ -218,32 +241,6 @@ egcpool_stash(egcpool* pool, const char* egc, size_t ulen){
   return -1; // should never get here
 }
 
-// Run a consistency check on the offset; ensure it's a valid, non-empty EGC.
-static inline bool
-egcpool_check_validity(const egcpool* pool, int offset){
-  if(offset >= pool->poolsize){
-    fprintf(stderr, "Offset 0x%06x greater than size (%d)\n", offset, pool->poolsize);
-    return false;
-  }
-  const char* egc = pool->pool + offset;
-  if(*egc == '\0'){
-    fprintf(stderr, "Bad offset 0x%06x: empty\n", offset);
-    return false;
-  }
-  mbstate_t mbstate;
-  memset(&mbstate, 0, sizeof(mbstate));
-  do{
-    wchar_t wcs;
-    int r = mbrtowc(&wcs, egc, strlen(egc), &mbstate);
-    if(r < 0){
-      fprintf(stderr, "Invalid UTF8 at offset 0x%06x [%s]\n", offset, strerror(errno));
-      return false;
-    }
-    egc += r;
-  }while(*egc);
-  return true;
-}
-
 // remove the egc from the pool. start at offset, and zero out everything until
 // we find a zero (our own NUL terminator). remove that number of bytes from
 // the usedcount.
@@ -300,15 +297,17 @@ egcpool_extended_gcluster(const egcpool* pool, const nccell* c) {
 // contents in 'dst'.
 static inline int
 egcpool_dup(egcpool* dst, const egcpool* src){
-  char* tmp;
-  if((tmp = (char*)realloc(dst->pool, src->poolsize)) == NULL){
-    return -1;
+  if(src->pool){
+    char* tmp;
+    if((tmp = (char*)realloc(dst->pool, src->poolsize)) == NULL){
+      return -1;
+    }
+    dst->pool = tmp;
+    memcpy(dst->pool, src->pool, src->poolsize);
   }
-  dst->pool = tmp;
   dst->poolsize = src->poolsize;
   dst->poolused = src->poolused;
   dst->poolwrite = src->poolwrite;
-  memcpy(dst->pool, src->pool, src->poolsize);
   return 0;
 }
 
