@@ -1,11 +1,46 @@
 #include "internal.h"
 
+// ncmenu_item and ncmenu_section have internal and (minimal) external forms
+typedef struct ncmenu_int_item {
+  char* desc;           // utf-8 menu item, NULL for horizontal separator
+  ncinput shortcut;     // shortcut, all should be distinct
+  int shortcut_offset;  // column offset with desc of shortcut EGC
+  char* shortdesc;      // description of shortcut, can be NULL
+  int shortdesccols;    // columns occupied by shortcut description
+  bool disabled;        // disabled?
+} ncmenu_int_item;
+
+typedef struct ncmenu_int_section {
+  char* name;             // utf-8 c string
+  unsigned itemcount;
+  ncmenu_int_item* items; // items, NULL iff itemcount == 0
+  ncinput shortcut;       // shortcut, will be underlined if present in name
+  int xoff;               // column offset from beginning of menu bar
+  int bodycols;           // column width of longest item
+  int itemselected;       // current item selected, -1 for no selection
+  int shortcut_offset;    // column offset within name of shortcut EGC
+  int enabled_item_count; // number of enabled items: section is disabled iff 0
+} ncmenu_int_section;
+
+typedef struct ncmenu {
+  ncplane* ncp;
+  int sectioncount;         // must be positive
+  ncmenu_int_section* sections; // NULL iff sectioncount == 0
+  int unrolledsection;      // currently unrolled section, -1 if none
+  int headerwidth;          // minimum space necessary to display all sections
+  uint64_t headerchannels;  // styling for header
+  uint64_t dissectchannels; // styling for disabled section headers
+  uint64_t sectionchannels; // styling for sections
+  uint64_t disablechannels; // styling for disabled entries
+  bool bottom;              // are we on the bottom (vs top)?
+} ncmenu;
+
 // Search the provided multibyte (UTF8) string 's' for the provided unicode
 // codepoint 'cp'. If found, return the column offset of the EGC in which the
 // codepoint appears in 'col', and the byte offset as the return value. If not
 // found, -1 is returned, and 'col' is meaningless.
 static int
-mbstr_find_codepoint(const char* s, char32_t cp, int* col){
+mbstr_find_codepoint(const char* s, uint32_t cp, int* col){
   mbstate_t ps;
   memset(&ps, 0, sizeof(ps));
   size_t bytes = 0;
@@ -27,7 +62,7 @@ mbstr_find_codepoint(const char* s, char32_t cp, int* col){
 
 static void
 free_menu_section(ncmenu_int_section* ms){
-  for(int i = 0 ; i < ms->itemcount ; ++i){
+  for(unsigned i = 0 ; i < ms->itemcount ; ++i){
     free(ms->items[i].desc);
     free(ms->items[i].shortdesc);
   }
@@ -57,10 +92,10 @@ dup_menu_item(ncmenu_int_item* dst, const struct ncmenu_item* src){
     return 0;
   }
   size_t bytes = 1; // NUL terminator
-  if(src->shortcut.alt){
+  if(ncinput_alt_p(&src->shortcut)){
     bytes += strlen(ALTMOD);
   }
-  if(src->shortcut.ctrl){
+  if(ncinput_ctrl_p(&src->shortcut)){
     bytes += strlen(CTLMOD);
   }
   mbstate_t ps;
@@ -72,8 +107,8 @@ dup_menu_item(ncmenu_int_item* dst, const struct ncmenu_item* src){
   }
   bytes += shortsize + 1;
   char* sdup = malloc(bytes);
-  int n = snprintf(sdup, bytes, "%s%s", src->shortcut.alt ? ALTMOD : "",
-                   src->shortcut.ctrl ? CTLMOD : "");
+  int n = snprintf(sdup, bytes, "%s%s", ncinput_alt_p(&src->shortcut) ? ALTMOD : "",
+                   ncinput_ctrl_p(&src->shortcut) ? CTLMOD : "");
   if(n < 0 || (size_t)n >= bytes){
     free(sdup);
     free(dst->desc);
@@ -88,7 +123,7 @@ dup_menu_item(ncmenu_int_item* dst, const struct ncmenu_item* src){
   }
   sdup[n + mbbytes] = '\0';
   dst->shortdesc = sdup;
-  dst->shortdesccols = ncstrwidth(dst->shortdesc);
+  dst->shortdesccols = ncstrwidth(dst->shortdesc, NULL, NULL);
   return 0;
 #undef CTLMOD
 #undef ALTMOD
@@ -101,7 +136,7 @@ dup_menu_section(ncmenu_int_section* dst, const struct ncmenu_section* src){
     return -1;
   }
   dst->bodycols = 0;
-  dst->itemselected = 0;
+  dst->itemselected = -1;
   dst->items = NULL;
   // we must reject any section which is entirely separators
   bool gotitem = false;
@@ -121,7 +156,7 @@ dup_menu_section(ncmenu_int_section* dst, const struct ncmenu_section* src){
         return -1;
       }
       gotitem = true;
-      int cols = ncstrwidth(dst->items[i].desc);
+      int cols = ncstrwidth(dst->items[i].desc, NULL, NULL);
       if(dst->items[i].shortdesc){
         cols += 2 + dst->items[i].shortdesccols; // two spaces minimum
       }
@@ -153,7 +188,7 @@ dup_menu_section(ncmenu_int_section* dst, const struct ncmenu_section* src){
 
 // Duplicates all menu sections in opts, adding their length to '*totalwidth'.
 static int
-dup_menu_sections(ncmenu* ncm, const ncmenu_options* opts, int* totalwidth, int* totalheight){
+dup_menu_sections(ncmenu* ncm, const ncmenu_options* opts, unsigned* totalwidth, unsigned* totalheight){
   if(opts->sectioncount == 0){
     return -1;
   }
@@ -162,13 +197,13 @@ dup_menu_sections(ncmenu* ncm, const ncmenu_options* opts, int* totalwidth, int*
     return -1;
   }
   bool rightaligned = false; // can only right-align once. twice is error.
-  int maxheight = 0;
-  int maxwidth = *totalwidth;
-  int xoff = 2;
+  unsigned maxheight = 0;
+  unsigned maxwidth = *totalwidth;
+  unsigned xoff = 2;
   int i;
   for(i = 0 ; i < opts->sectioncount ; ++i){
     if(opts->sections[i].name){
-      int cols = ncstrwidth(opts->sections[i].name);
+      int cols = ncstrwidth(opts->sections[i].name, NULL, NULL);
       if(rightaligned){ // FIXME handle more than one right-aligned section
         ncm->sections[i].xoff = -(cols + 2);
       }else{
@@ -224,6 +259,7 @@ err:
   while(i--){
     free_menu_section(&ncm->sections[i]);
   }
+  free(ncm->sections);
   return -1;
 }
 
@@ -241,14 +277,14 @@ section_x(const ncmenu* ncm, int x){
       if(x < pos){
         break;
       }
-      if(x < pos + ncstrwidth(ncm->sections[i].name)){
+      if(x < pos + ncstrwidth(ncm->sections[i].name, NULL, NULL)){
         return i;
       }
     }else{
       if(x < ncm->sections[i].xoff){
         break;
       }
-      if(x < ncm->sections[i].xoff + ncstrwidth(ncm->sections[i].name)){
+      if(x < ncm->sections[i].xoff + ncstrwidth(ncm->sections[i].name, NULL, NULL)){
         return i;
       }
     }
@@ -259,14 +295,14 @@ section_x(const ncmenu* ncm, int x){
 static int
 write_header(ncmenu* ncm){
   ncplane_set_channels(ncm->ncp, ncm->headerchannels);
-  int dimy, dimx;
+  unsigned dimy, dimx;
   ncplane_dim_yx(ncm->ncp, &dimy, &dimx);
-  int xoff = 0; // 2-column margin on left
+  unsigned xoff = 0; // 2-column margin on left
   int ypos = ncm->bottom ? dimy - 1 : 0;
   if(ncplane_cursor_move_yx(ncm->ncp, ypos, 0)){
     return -1;
   }
-  nccell c = CELL_INITIALIZER(' ', 0, ncm->headerchannels);
+  nccell c = NCCELL_INITIALIZER(' ', 0, ncm->headerchannels);
   ncplane_set_styles(ncm->ncp, 0);
   if(ncplane_putc(ncm->ncp, &c) < 0){
     return -1;
@@ -299,17 +335,17 @@ write_header(ncmenu* ncm){
         return -1;
       }
       if(ncm->sections[i].shortcut_offset >= 0){
-        nccell cl = CELL_TRIVIAL_INITIALIZER;
+        nccell cl = NCCELL_TRIVIAL_INITIALIZER;
         if(ncplane_at_yx_cell(ncm->ncp, ypos, xoff + ncm->sections[i].shortcut_offset, &cl) < 0){
           return -1;
         }
-        cell_on_styles(&cl, NCSTYLE_UNDERLINE|NCSTYLE_BOLD);
+        nccell_on_styles(&cl, NCSTYLE_UNDERLINE|NCSTYLE_BOLD);
         if(ncplane_putc_yx(ncm->ncp, ypos, xoff + ncm->sections[i].shortcut_offset, &cl) < 0){
           return -1;
         }
-        cell_release(ncm->ncp, &cl);
+        nccell_release(ncm->ncp, &cl);
       }
-      xoff += ncstrwidth(ncm->sections[i].name);
+      xoff += ncstrwidth(ncm->sections[i].name, NULL, NULL);
     }
   }
   while(xoff < dimx){
@@ -339,24 +375,24 @@ resize_menu(ncplane* n){
 }
 
 ncmenu* ncmenu_create(ncplane* n, const ncmenu_options* opts){
-  ncmenu_options zeroed = {};
+  ncmenu_options zeroed = {0};
   if(!opts){
     opts = &zeroed;
   }
   if(opts->sectioncount <= 0 || !opts->sections){
-    logerror(ncplane_notcurses(n), "Invalid %d-ary section information\n", opts->sectioncount);
+    logerror("invalid %d-ary section information", opts->sectioncount);
     return NULL;
   }
   if(opts->flags >= (NCMENU_OPTION_HIDING << 1u)){
-    logwarn(ncplane_notcurses(n), "Provided unsupported flags %016jx\n", (uintmax_t)opts->flags);
+    logwarn("provided unsupported flags %016" PRIx64, opts->flags);
   }
-  int totalheight = 1;
-  int totalwidth = 2; // start with two-character margin on the left
+  unsigned totalheight = 1;
+  unsigned totalwidth = 2; // start with two-character margin on the left
   ncmenu* ret = malloc(sizeof(*ret));
   ret->sectioncount = opts->sectioncount;
   ret->sections = NULL;
-  int dimy, dimx;
-  ncplane_dim_yx(notcurses_stdplane(ncplane_notcurses(n)), &dimy, &dimx);
+  unsigned dimy, dimx;
+  ncplane_dim_yx(n, &dimy, &dimx);
   if(ret){
     ret->bottom = !!(opts->flags & NCMENU_OPTION_BOTTOM);
     if(dup_menu_sections(ret, opts, &totalwidth, &totalheight) == 0){
@@ -372,23 +408,26 @@ ncmenu* ncmenu_create(ncplane* n, const ncmenu_options* opts){
         .userptr = ret,
         .name = "menu",
         .resizecb = resize_menu,
+        .flags = NCPLANE_OPTION_FIXED,
       };
       ret->ncp = ncplane_create(n, &nopts);
       if(ret->ncp){
-        ret->unrolledsection = -1;
-        ret->headerchannels = opts->headerchannels;
-        ret->dissectchannels = opts->headerchannels;
-        channels_set_fg_rgb(&ret->dissectchannels, 0xdddddd);
-        ret->sectionchannels = opts->sectionchannels;
-        ret->disablechannels = ret->sectionchannels;
-        channels_set_fg_rgb(&ret->disablechannels, 0xdddddd);
-        nccell c = CELL_TRIVIAL_INITIALIZER;
-        cell_set_fg_alpha(&c, CELL_ALPHA_TRANSPARENT);
-        cell_set_bg_alpha(&c, CELL_ALPHA_TRANSPARENT);
-        ncplane_set_base_cell(ret->ncp, &c);
-        cell_release(ret->ncp, &c);
-        if(write_header(ret) == 0){
-          return ret;
+        if(ncplane_set_widget(ret->ncp, ret, (void(*)(void*))ncmenu_destroy) == 0){
+          ret->unrolledsection = -1;
+          ret->headerchannels = opts->headerchannels;
+          ret->dissectchannels = opts->headerchannels;
+          ncchannels_set_fg_rgb(&ret->dissectchannels, 0xdddddd);
+          ret->sectionchannels = opts->sectionchannels;
+          ret->disablechannels = ret->sectionchannels;
+          ncchannels_set_fg_rgb(&ret->disablechannels, 0xdddddd);
+          nccell c = NCCELL_TRIVIAL_INITIALIZER;
+          nccell_set_fg_alpha(&c, NCALPHA_TRANSPARENT);
+          nccell_set_bg_alpha(&c, NCALPHA_TRANSPARENT);
+          ncplane_set_base_cell(ret->ncp, &c);
+          nccell_release(ret->ncp, &c);
+          if(write_header(ret) == 0){
+            return ret;
+          }
         }
         ncplane_destroy(ret->ncp);
       }
@@ -396,7 +435,7 @@ ncmenu* ncmenu_create(ncplane* n, const ncmenu_options* opts){
     }
     free(ret);
   }
-  logerror(ncplane_notcurses(n), "Error creating ncmenu\n");
+  logerror("error creating ncmenu");
   return NULL;
 }
 
@@ -415,7 +454,7 @@ int ncmenu_unroll(ncmenu* n, int sectionidx){
     return -1;
   }
   if(sectionidx < 0 || sectionidx >= n->sectioncount){
-    logerror(ncplane_notcurses(n->ncp), "Unrolled invalid sectionidx %d\n", sectionidx);
+    logerror("unrolled invalid sectionidx %d", sectionidx);
     return -1;
   }
   if(n->sections[sectionidx].enabled_item_count <= 0){
@@ -425,13 +464,13 @@ int ncmenu_unroll(ncmenu* n, int sectionidx){
     return -1;
   }
   n->unrolledsection = sectionidx;
-  int dimy, dimx;
+  unsigned dimy, dimx;
   ncplane_dim_yx(n->ncp, &dimy, &dimx);
   const int height = section_height(n, sectionidx);
   const int width = section_width(n, sectionidx);
   int xpos = n->sections[sectionidx].xoff < 0 ?
-    dimx + (n->sections[sectionidx].xoff - 2) : n->sections[sectionidx].xoff;
-  if(xpos + width >= dimx){
+    (int)dimx + (n->sections[sectionidx].xoff - 2) : n->sections[sectionidx].xoff;
+  if(xpos + width >= (int)dimx){
     xpos = dimx - (width + 2);
   }
   int ypos = n->bottom ? dimy - height - 1 : 1;
@@ -441,21 +480,25 @@ int ncmenu_unroll(ncmenu* n, int sectionidx){
   if(ncplane_rounded_box_sized(n->ncp, 0, n->headerchannels, height, width, 0)){
     return -1;
   }
-  const ncmenu_int_section* sec = &n->sections[sectionidx];
-  for(int i = 0 ; i < sec->itemcount ; ++i){
+  ncmenu_int_section* sec = &n->sections[sectionidx];
+  for(unsigned i = 0 ; i < sec->itemcount ; ++i){
     ++ypos;
     if(sec->items[i].desc){
       // FIXME the user ought be able to configure the disabled channel
       if(!sec->items[i].disabled){
         ncplane_set_channels(n->ncp, n->sectionchannels);
+        if(sec->itemselected < 0){
+          sec->itemselected = i;
+        }
       }else{
         ncplane_set_channels(n->ncp, n->disablechannels);
       }
-      if(i == sec->itemselected){
-        ncplane_set_styles(n->ncp, NCSTYLE_REVERSE);
-      }else{
-        ncplane_set_styles(n->ncp, 0);
+      if(sec->itemselected >= 0){
+        if(i == (unsigned)sec->itemselected){
+          ncplane_set_channels(n->ncp, ncchannels_reverse(ncplane_channels(n->ncp)));
+        }
       }
+      ncplane_set_styles(n->ncp, 0);
       int cols = ncplane_putstr_yx(n->ncp, ypos, xpos + 1, sec->items[i].desc);
       if(cols < 0){
         return -1;
@@ -479,15 +522,15 @@ int ncmenu_unroll(ncmenu* n, int sectionidx){
         }
       }
       if(sec->items[i].shortcut_offset >= 0){
-        nccell cl = CELL_TRIVIAL_INITIALIZER;
+        nccell cl = NCCELL_TRIVIAL_INITIALIZER;
         if(ncplane_at_yx_cell(n->ncp, ypos, xpos + 1 + sec->items[i].shortcut_offset, &cl) < 0){
           return -1;
         }
-        cell_on_styles(&cl, NCSTYLE_UNDERLINE|NCSTYLE_BOLD);
+        nccell_on_styles(&cl, NCSTYLE_UNDERLINE|NCSTYLE_BOLD);
         if(ncplane_putc_yx(n->ncp, ypos, xpos + 1 + sec->items[i].shortcut_offset, &cl) < 0){
           return -1;
         }
-        cell_release(n->ncp, &cl);
+        nccell_release(n->ncp, &cl);
       }
     }else{
       n->ncp->channels = n->headerchannels;
@@ -519,11 +562,14 @@ int ncmenu_rollup(ncmenu* n){
 
 int ncmenu_nextsection(ncmenu* n){
   int nextsection = n->unrolledsection;
-  // FIXME probably best to detect cycles
+  int origselected = n->unrolledsection;
   do{
     if(++nextsection == n->sectioncount){
       nextsection = 0;
     }
+    if(nextsection == origselected){
+      break;
+    }
   }while(n->sections[nextsection].name == NULL ||
          n->sections[nextsection].enabled_item_count == 0);
   return ncmenu_unroll(n, nextsection);
@@ -531,11 +577,14 @@ int ncmenu_nextsection(ncmenu* n){
 
 int ncmenu_prevsection(ncmenu* n){
   int prevsection = n->unrolledsection;
-  // FIXME probably best to detect cycles
+  int origselected = n->unrolledsection;
   do{
     if(--prevsection < 0){
       prevsection = n->sectioncount - 1;
     }
+    if(prevsection == origselected){
+      break;
+    }
   }while(n->sections[prevsection].name == NULL ||
          n->sections[prevsection].enabled_item_count == 0);
   return ncmenu_unroll(n, prevsection);
@@ -548,11 +597,14 @@ int ncmenu_nextitem(ncmenu* n){
     }
   }
   ncmenu_int_section* sec = &n->sections[n->unrolledsection];
-  // FIXME probably best to detect cycles
+  int origselected = sec->itemselected;
   do{
-    if(++sec->itemselected == sec->itemcount){
+    if((unsigned)++sec->itemselected == sec->itemcount){
       sec->itemselected = 0;
     }
+    if(sec->itemselected == origselected){
+      break;
+    }
   }while(!sec->items[sec->itemselected].desc || sec->items[sec->itemselected].disabled);
   return ncmenu_unroll(n, n->unrolledsection);
 }
@@ -564,11 +616,14 @@ int ncmenu_previtem(ncmenu* n){
     }
   }
   ncmenu_int_section* sec = &n->sections[n->unrolledsection];
-  // FIXME probably best to detect cycles
+  int origselected = sec->itemselected;
   do{
     if(sec->itemselected-- == 0){
       sec->itemselected = sec->itemcount - 1;
     }
+    if(sec->itemselected == origselected){
+      break;
+    }
   }while(!sec->items[sec->itemselected].desc || sec->items[sec->itemselected].disabled);
   return ncmenu_unroll(n, n->unrolledsection);
 }
@@ -579,38 +634,66 @@ const char* ncmenu_selected(const ncmenu* n, ncinput* ni){
   }
   const struct ncmenu_int_section* sec = &n->sections[n->unrolledsection];
   const int itemidx = sec->itemselected;
+  if(itemidx < 0){
+    return NULL;
+  }
   if(ni){
     memcpy(ni, &sec->items[itemidx].shortcut, sizeof(*ni));
   }
   return sec->items[itemidx].desc;
 }
 
+// given the active section, return the line on which we clicked, or -1 if the
+// click was not within said section. |y| and |x| ought be translated for the
+// menu plane |n|->ncp.
+static int
+ncsection_click_index(const ncmenu* n, const ncmenu_int_section* sec,
+                      unsigned dimy, unsigned dimx, int y, int x){
+  // don't allow a click on the side boundaries
+  if(sec->xoff < 0){
+    if(x > (int)dimx - 4 || x <= (int)dimx - 4 - sec->bodycols){
+      return -1;
+    }
+  }else{
+    if(x <= sec->xoff || x > sec->xoff + sec->bodycols){
+      return -1;
+    }
+  }
+  const int itemidx = n->bottom ? y - ((int)dimy - (int)sec->itemcount) + 2 : y - 2;
+  if(itemidx < 0 || itemidx >= (int)sec->itemcount){
+    return -1;
+  }
+  return itemidx;
+}
+
 const char* ncmenu_mouse_selected(const ncmenu* n, const ncinput* click,
                                   ncinput* ni){
-  if(click->id != NCKEY_RELEASE){
+  if(click->id != NCKEY_BUTTON1){
+    return NULL;
+  }
+  if(click->evtype != NCTYPE_RELEASE){
     return NULL;
   }
-  int y, x, dimy, dimx;
   struct ncplane* nc = n->ncp;
-  y = click->y;
-  x = click->x;
+  int y = click->y;
+  int x = click->x;
+  unsigned dimy, dimx;
   ncplane_dim_yx(nc, &dimy, &dimx);
   if(!ncplane_translate_abs(nc, &y, &x)){
     return NULL;
   }
-  // FIXME section_x() works only off the section header lengths, meaning that
-  // if we click an item outside of those columns covered by the header, it will
-  // read as a -1 from section_x(). we want to instead get the unrolled section,
-  // find its boundaries, and verify that we are within them.
-  int i = section_x(n, x);
-  if(i < 0 || i != n->unrolledsection){
+  if(n->unrolledsection < 0){
     return NULL;
   }
   const struct ncmenu_int_section* sec = &n->sections[n->unrolledsection];
-  if(y < 2 || y - 2 >= sec->itemcount){
+  int itemidx = ncsection_click_index(n, sec, dimy, dimx, y, x);
+  if(itemidx < 0){
+    return NULL;
+  }
+  // don't allow a disabled item to be selected
+  if(sec->items[itemidx].disabled){
     return NULL;
   }
-  const int itemidx = y - 2;
   if(ni){
     memcpy(ni, &sec->items[itemidx].shortcut, sizeof(*ni));
   }
@@ -620,15 +703,26 @@ const char* ncmenu_mouse_selected(const ncmenu* n, const ncinput* click,
 bool ncmenu_offer_input(ncmenu* n, const ncinput* nc){
   // we can't actually select menu items in this function, since we need to
   // invoke an arbitrary function as a result.
-  if(nc->id == NCKEY_RELEASE){
-    int y, x, dimy, dimx;
-    y = nc->y;
-    x = nc->x;
+  if(nc->id == NCKEY_BUTTON1 && nc->evtype == NCTYPE_RELEASE){
+    int y = nc->y;
+    int x = nc->x;
+    unsigned dimy, dimx;
     ncplane_dim_yx(n->ncp, &dimy, &dimx);
     if(!ncplane_translate_abs(n->ncp, &y, &x)){
       return false;
     }
-    if(y != (n->bottom ? dimy - 1 : 0)){
+    if(n->unrolledsection >= 0){
+      struct ncmenu_int_section* sec = &n->sections[n->unrolledsection];
+      int itemidx = ncsection_click_index(n, sec, dimy, dimx, y, x);
+      if(itemidx >= 0){
+        if(!sec->items[itemidx].disabled){
+          sec->itemselected = itemidx;
+          ncmenu_unroll(n, n->unrolledsection);
+          return false;
+        }
+      }
+    }
+    if(y != (n->bottom ? (int)dimy - 1 : 0)){
       return false;
     }
     int i = section_x(n, x);
@@ -638,6 +732,8 @@ bool ncmenu_offer_input(ncmenu* n, const ncinput* nc){
       ncmenu_unroll(n, i);
     }
     return true;
+  }else if(nc->evtype == NCTYPE_RELEASE){
+    return false;
   }
   for(int si = 0 ; si < n->sectioncount ; ++si){
     const ncmenu_int_section* sec = &n->sections[si];
@@ -686,10 +782,10 @@ int ncmenu_item_set_status(ncmenu* n, const char* section, const char* item,
   for(int si = 0 ; si < n->sectioncount ; ++si){
     struct ncmenu_int_section* sec = &n->sections[si];
     if(strcmp(sec->name, section) == 0){
-      for(int ii = 0 ; ii < sec->itemcount ; ++ii){
+      for(unsigned ii = 0 ; ii < sec->itemcount ; ++ii){
         struct ncmenu_int_item* i = &sec->items[ii];
         if(strcmp(i->desc, item) == 0){
-          const bool changed = i->disabled == enabled;
+          const bool changed = (i->disabled != enabled);
           i->disabled = !enabled;
           if(changed){
             if(i->disabled){
@@ -722,12 +818,12 @@ ncplane* ncmenu_plane(ncmenu* menu){
   return menu->ncp;
 }
 
-int ncmenu_destroy(ncmenu* n){
-  int ret = 0;
+void ncmenu_destroy(ncmenu* n){
   if(n){
     free_menu_sections(n);
-    ncplane_destroy(n->ncp);
+    if(ncplane_set_widget(n->ncp, NULL, NULL) == 0){
+      ncplane_destroy(n->ncp);
+    }
     free(n);
   }
-  return ret;
 }
