@@ -5,16 +5,20 @@
 int main(void){
   setlocale(LC_ALL, "");
   notcurses_options nopts = {
-    .flags = NCOPTION_INHIBIT_SETLOCALE,
+    .flags = NCOPTION_INHIBIT_SETLOCALE
+             | NCOPTION_CLI_MODE
+             | NCOPTION_DRAIN_INPUT,
   };
   struct notcurses* nc = notcurses_core_init(&nopts, NULL);
   if(nc == NULL){
     return EXIT_FAILURE;
   }
-  int dimy, dimx;
+  unsigned dimy, dimx;
   struct ncplane* n = notcurses_stddim_yx(nc, &dimy, &dimx);
   wchar_t wc = 0x4e00;
-  ncplane_set_scrolling(n, true);
+  ncplane_set_styles(n, NCSTYLE_BOLD);
+  ncplane_putstr(n, "This program is *not* indicative of real scrolling speed.\n");
+  ncplane_set_styles(n, NCSTYLE_NONE);
   while(true){
     struct timespec req = { .tv_sec = 0, .tv_nsec = 1000000, };
     nanosleep(&req, NULL);
