@@ -1,5 +1,5 @@
 #include "main.h"
-#include "egcpool.h"
+#include "lib/egcpool.h"
 
 TEST_CASE("Cell") {
   auto nc_ = testing_notcurses();
@@ -14,72 +14,62 @@ TEST_CASE("Cell") {
   REQUIRE(nullptr != n_);
 
   SUBCASE("EGCs") {
-    nccell c = CELL_TRIVIAL_INITIALIZER;
-    CHECK(2 == nccell_load(n_, &c, "é"));
+    nccell c = NCCELL_TRIVIAL_INITIALIZER;
+    CHECK(2 == nccell_load(n_, &c, u8"é"));
     CHECK(1 == nccell_cols(&c));
     int cols;
-    CHECK(3 == nccell_load(n_, &c, "\x41\u0301"));
+    CHECK(3 == nccell_load(n_, &c, u8"\x41\u0301"));
     CHECK(1 == nccell_cols(&c));
     CHECK(4 == nccell_load(n_, &c, " ி"));
     cols = nccell_cols(&c);
-#if defined(__linux__) || defined(__APPLE__)
-    CHECK(2 == cols);
-#else
     CHECK(1 == cols);
-#endif
     CHECK(4 == nccell_load(n_, &c, " ि"));
     cols = nccell_cols(&c);
-#if defined(__linux__) || defined(__APPLE__)
-    CHECK(2 == cols);
-#else
     CHECK(1 == cols);
-#endif
     // musl+s390x (alpine) is reporting these EGCs to be 0 columns wide (they
     // ought be 1). not sure whether i've got a bug (s390x is big-endian), or
     // whether it does. just relaxed the tests for now FIXME.
-    CHECK(5 == nccell_load(n_, &c, "◌̈"));
+    CHECK(5 == nccell_load(n_, &c, u8"◌̈"));
     WARN(1 == nccell_cols(&c));
-    CHECK(9 == nccell_load(n_, &c, "นี้"));
+    CHECK(9 == nccell_load(n_, &c, u8"นี้"));
     WARN(1 == nccell_cols(&c));
 
-    // type-3 woman playing water polo, 17 bytes (5 characters)
+    // type-3 woman playing water polo, 17 bytes (5 characters, 2 columns)
 #ifdef __linux__
-    CHECK(17 == nccell_load(n_, &c, "\U0001f93d\U0001f3fc\u200d\u2640\ufe0f"));
+    CHECK(17 == nccell_load(n_, &c, u8"\U0001f93d\U0001f3fc\u200d\u2640\ufe0f"));
     WARN(2 == nccell_cols(&c));
     nccell_release(n_, &c);
 #endif
   }
 
   SUBCASE("Loadchar") {
-    nccell c = CELL_TRIVIAL_INITIALIZER;
+    nccell c = NCCELL_TRIVIAL_INITIALIZER;
     CHECK(1 == nccell_load(n_, &c, " "));
     CHECK(cell_simple_p(&c));
     nccell_release(n_, &c);
   }
 
   SUBCASE("MultibyteWidth") {
-    CHECK(0 == ncstrwidth(""));       // zero bytes, zero columns
-    CHECK(-1 == ncstrwidth("\x7"));   // single byte, non-printable
-    CHECK(1 == ncstrwidth(" "));      // single byte, one column
-    CHECK(5 == ncstrwidth("abcde"));  // single byte, one column
-    CHECK(1 == ncstrwidth("µ"));      // two bytes, one column
-    CHECK(1 <= ncstrwidth("\U0001f982"));     // four bytes, two columns
-    CHECK(3 <= ncstrwidth("平仮名")); // nine bytes, six columns
-    CHECK(1 == ncstrwidth("\U00012008")); // four bytes, 1 column
+    CHECK(0 == ncstrwidth(u8"", NULL, NULL));       // zero bytes, zero columns
+    CHECK(-1 == ncstrwidth(u8"\x7", NULL, NULL));   // single byte, non-printable
+    CHECK(1 == ncstrwidth(u8" ", NULL, NULL));      // single byte, one column
+    CHECK(5 == ncstrwidth(u8"abcde", NULL, NULL));  // single byte, one column
+    CHECK(1 == ncstrwidth(u8"µ", NULL, NULL));      // two bytes, one column
+    CHECK(1 <= ncstrwidth(u8"\U0001f982", NULL, NULL));     // four bytes, two columns
+    CHECK(3 <= ncstrwidth(u8"平仮名", NULL, NULL)); // nine bytes, six columns
+    CHECK(1 == ncstrwidth(u8"\U00012008", NULL, NULL)); // four bytes, 1 column
   }
 
   // test combining characters and ZWJs
   SUBCASE("MultiglyphWidth") {
-    CHECK(2 == ncstrwidth("\U0001F471"));
-#ifndef __APPLE__ // FIXME
-    CHECK(2 == ncstrwidth("\U0001F471\u200D"));
-    CHECK(3 == ncstrwidth("\U0001F471\u200D\u2640")); // *not* a single EGC!
-#endif
+    CHECK(2 == ncstrwidth(u8"\U0001F471", NULL, NULL));
+    CHECK(2 == ncstrwidth(u8"\U0001F471\u200D", NULL, NULL));
+    CHECK(2 == ncstrwidth(u8"\U0001F471\u200D\u2640", NULL, NULL));
   }
 
   SUBCASE("SetItalic") {
-    nccell c = CELL_TRIVIAL_INITIALIZER;
-    int dimy, dimx;
+    nccell c = NCCELL_TRIVIAL_INITIALIZER;
+    unsigned dimy, dimx;
     notcurses_term_dim_yx(nc_, &dimy, &dimx);
     nccell_set_styles(&c, NCSTYLE_ITALIC);
     CHECK(1 == nccell_load(n_, &c, "i"));
@@ -91,8 +81,8 @@ TEST_CASE("Cell") {
   }
 
   SUBCASE("SetBold") {
-    nccell c = CELL_TRIVIAL_INITIALIZER;
-    int dimy, dimx;
+    nccell c = NCCELL_TRIVIAL_INITIALIZER;
+    unsigned dimy, dimx;
     notcurses_term_dim_yx(nc_, &dimy, &dimx);
     nccell_set_styles(&c, NCSTYLE_BOLD);
     CHECK(1 == nccell_load(n_, &c, "b"));
@@ -104,8 +94,8 @@ TEST_CASE("Cell") {
   }
 
   SUBCASE("SetUnderline") {
-    nccell c = CELL_TRIVIAL_INITIALIZER;
-    int dimy, dimx;
+    nccell c = NCCELL_TRIVIAL_INITIALIZER;
+    unsigned dimy, dimx;
     notcurses_term_dim_yx(nc_, &dimy, &dimx);
     nccell_set_styles(&c, NCSTYLE_UNDERLINE);
     CHECK(1 == nccell_load(n_, &c, "u"));
@@ -118,7 +108,7 @@ TEST_CASE("Cell") {
 
   /*SUBCASE("CellLoadTamil") {
     const char zerodeg[] = "\u0bb8\u0bc0\u0bb0\u0bc7\u0bb3\u0b95\u0bbf\u0b95\u0bbf\u0bb0\u0bbf";
-    nccell c = CELL_TRIVIAL_INITIALIZER;
+    nccell c = NCCELL_TRIVIAL_INITIALIZER;
     size_t ulen = nccell_load(n_, &c, zerodeg);
     // First have U+0BB8 TAMIL LETTER SA U+0BC0 TAMIL VOWEL SIGN II
     // // e0 ae b8 e0 af 80
@@ -131,7 +121,7 @@ TEST_CASE("Cell") {
   }*/
 
   SUBCASE("CellSetFGAlpha"){
-    nccell c = CELL_TRIVIAL_INITIALIZER;
+    nccell c = NCCELL_TRIVIAL_INITIALIZER;
     CHECK(0 > nccell_set_fg_alpha(&c, -1));
     CHECK(0 > nccell_set_fg_alpha(&c, 4));
     CHECK(0 == nccell_set_fg_alpha(&c, NCALPHA_OPAQUE));
@@ -145,7 +135,7 @@ TEST_CASE("Cell") {
   }
 
   SUBCASE("CellSetBGAlpha"){
-    nccell c = CELL_TRIVIAL_INITIALIZER;
+    nccell c = NCCELL_TRIVIAL_INITIALIZER;
     CHECK(0 > nccell_set_bg_alpha(&c, -1));
     CHECK(0 > nccell_set_bg_alpha(&c, 4));
     CHECK(0 == nccell_set_bg_alpha(&c, NCALPHA_OPAQUE));
@@ -159,7 +149,7 @@ TEST_CASE("Cell") {
 
   // white on a black background ought be unmolested for highcontrast
   SUBCASE("HighContrastWhiteOnBlackBackground"){
-    nccell c = CELL_CHAR_INITIALIZER('+');
+    nccell c = NCCELL_CHAR_INITIALIZER('+');
     CHECK(0 == nccell_set_fg_rgb8(&c, 0xff, 0xff, 0xff));
     CHECK(0 == nccell_set_fg_alpha(&c, NCALPHA_HIGHCONTRAST));
     CHECK(0 == nccell_set_bg_alpha(&c, NCALPHA_TRANSPARENT));
@@ -199,7 +189,7 @@ TEST_CASE("Cell") {
 
   // white on a white background ought be changed for highcontrast
   SUBCASE("HighContrastWhiteOnWhiteBackground"){
-    nccell c = CELL_CHAR_INITIALIZER('+');
+    nccell c = NCCELL_CHAR_INITIALIZER('+');
     CHECK(0 == nccell_set_fg_rgb8(&c, 0xff, 0xff, 0xff));
     CHECK(0 == nccell_set_fg_alpha(&c, NCALPHA_HIGHCONTRAST));
     CHECK(0 == nccell_set_bg_alpha(&c, NCALPHA_TRANSPARENT));
@@ -239,7 +229,7 @@ TEST_CASE("Cell") {
 
   // black on a black background must be changed for highcontrast
   SUBCASE("HighContrastBlackOnBlackBackground"){
-    nccell c = CELL_CHAR_INITIALIZER('+');
+    nccell c = NCCELL_CHAR_INITIALIZER('+');
     CHECK(0 == nccell_set_fg_rgb8(&c, 0x0, 0x0, 0x0));
     CHECK(0 == nccell_set_fg_alpha(&c, NCALPHA_HIGHCONTRAST));
     CHECK(0 == nccell_set_bg_alpha(&c, NCALPHA_TRANSPARENT));
@@ -279,7 +269,7 @@ TEST_CASE("Cell") {
 
   // black on a white background ought be unmolested for highcontrast
   SUBCASE("HighContrastBlackOnWhiteBackground"){
-    nccell c = CELL_CHAR_INITIALIZER('+');
+    nccell c = NCCELL_CHAR_INITIALIZER('+');
     CHECK(0 == nccell_set_fg_rgb8(&c, 0x0, 0x0, 0x0));
     CHECK(0 == nccell_set_fg_alpha(&c, NCALPHA_HIGHCONTRAST));
     CHECK(0 == nccell_set_bg_alpha(&c, NCALPHA_TRANSPARENT));
@@ -320,7 +310,7 @@ TEST_CASE("Cell") {
   // high contrast ought only be activated relevant to the background equal to
   // or below them, not above.
   SUBCASE("HighContrastBelowOnly"){
-    nccell c = CELL_TRIVIAL_INITIALIZER;
+    nccell c = NCCELL_TRIVIAL_INITIALIZER;
     // top has a background of white
     CHECK(0 == nccell_set_bg_rgb8(&c, 0xff, 0xff, 0xff));
     CHECK(0 == nccell_set_fg_alpha(&c, NCALPHA_TRANSPARENT));
@@ -359,36 +349,56 @@ TEST_CASE("Cell") {
   }
 
   SUBCASE("CellLoadCharPrinting") {
-    nccell c = CELL_TRIVIAL_INITIALIZER;
+    nccell c = NCCELL_TRIVIAL_INITIALIZER;
     CHECK(1 == nccell_load_char(n_, &c, '*'));
     CHECK(0 == strcmp(nccell_extended_gcluster(n_, &c), "*"));
   }
 
-  // only space is allowed
+  // only space/newline is allowed from control char whitespace
   SUBCASE("CellLoadCharWhitespace") {
-    nccell c = CELL_TRIVIAL_INITIALIZER;
-    CHECK(-1 == nccell_load_char(n_, &c, '\n'));
+    nccell c = NCCELL_TRIVIAL_INITIALIZER;
     CHECK(-1 == nccell_load_char(n_, &c, '\f'));
     CHECK(-1 == nccell_load_char(n_, &c, '\v'));
     CHECK(-1 == nccell_load_char(n_, &c, '\t'));
+    CHECK(1 == nccell_load_char(n_, &c, '\n'));
     CHECK(1 == nccell_load_char(n_, &c, ' '));
   }
 
   SUBCASE("CellLoadCharControl") {
-    nccell c = CELL_TRIVIAL_INITIALIZER;
+    nccell c = NCCELL_TRIVIAL_INITIALIZER;
     CHECK(0 == nccell_load_char(n_, &c, '\0'));
     CHECK(-1 == nccell_load_char(n_, &c, 1));
     CHECK(-1 == nccell_load_char(n_, &c, '\b'));
   }
 
   SUBCASE("CellLoadEGC32") {
-    nccell c = CELL_TRIVIAL_INITIALIZER;
+    nccell c = NCCELL_TRIVIAL_INITIALIZER;
     CHECK(0 == nccell_load_egc32(n_, &c, 0));
     CHECK(1 == nccell_load_egc32(n_, &c, 0x65));   // U+0061 LATIN SMALL LETTER A
     CHECK(2 == nccell_load_egc32(n_, &c, 0xb5c2)); // U+00B5 MICRO SIGN
     CHECK(4 == nccell_load_egc32(n_, &c, 0x82a69ff0)); // U+1F982 SCORPION
   }
 
+  // test the nccell_rgbequal_p() predicate. it ought return true only when
+  // both channels use RGB, and are equal.
+  SUBCASE("CellRGBEqual") {
+    nccell c = NCCELL_CHAR_INITIALIZER('x');
+    nccell_set_fg_default(&c);
+    nccell_set_bg_default(&c);
+    CHECK(0 == nccell_rgbequal_p(&c));
+    nccell_set_bg_palindex(&c, 0);
+    CHECK(nccell_bg_palindex_p(&c));
+    nccell_set_fg_palindex(&c, 0);
+    CHECK(nccell_fg_palindex_p(&c));
+    CHECK(0 == nccell_rgbequal_p(&c));
+    nccell_set_bg_rgb(&c, 0);
+    CHECK(0 == nccell_rgbequal_p(&c));
+    nccell_set_fg_rgb(&c, 0x1);
+    CHECK(0 == nccell_rgbequal_p(&c));
+    nccell_set_fg_rgb(&c, 0);
+    CHECK(nccell_rgbequal_p(&c));
+  }
+
   // common teardown
   CHECK(0 == notcurses_stop(nc_));
 }
