@@ -1,6 +1,6 @@
 #include <vector>
 #include "main.h"
-#include "egcpool.h"
+#include "lib/egcpool.h"
 
 TEST_CASE("EGCpool") {
   egcpool pool_{};
@@ -23,9 +23,9 @@ TEST_CASE("EGCpool") {
   auto n_ = notcurses_stdplane(nc_);
 
   SUBCASE("UTF8EGC") {
-    int c = ncstrwidth("☢");
+    int c = ncstrwidth("☢", NULL, NULL);
     CHECK(0 < c);
-    c = ncstrwidth("▓");
+    c = ncstrwidth("▓", NULL, NULL);
     CHECK(0 < c);
   }
 
@@ -35,7 +35,7 @@ TEST_CASE("EGCpool") {
     const char* w1 = "\u00e0"; // (utf8: c3 a0)
     const char* w2 = "\u0061\u0300"; // (utf8: 61 cc 80)
     const char* w3 = "\u0061"; // (utf8: 61)
-    nccell c = CELL_TRIVIAL_INITIALIZER;
+    nccell c = NCCELL_TRIVIAL_INITIALIZER;
     CHECK(2 == nccell_load(n_, &c, w1));
     CHECK(1 == nccell_cols(&c));
     CHECK(3 == nccell_load(n_, &c, w2));
@@ -46,7 +46,7 @@ TEST_CASE("EGCpool") {
 
   SUBCASE("AddAndRemove") {
     const char* wstr = "\U0001242B"; // cuneiform numeric sign nine shar2
-    nccell c = CELL_TRIVIAL_INITIALIZER;
+    nccell c = NCCELL_TRIVIAL_INITIALIZER;
     auto ulen = nccell_load(n_, &c, wstr);
     CHECK(1 == nccell_cols(&c)); // not considered wide, believe it or not
     REQUIRE(0 <= egcpool_stash(&pool_, wstr, ulen));
@@ -65,8 +65,8 @@ TEST_CASE("EGCpool") {
 
   SUBCASE("AddTwiceRemoveFirst") {
     const char* wstr = "\u8840"; // cjk unified ideograph, wide
-    nccell c1 = CELL_TRIVIAL_INITIALIZER;
-    nccell c2 = CELL_TRIVIAL_INITIALIZER;
+    nccell c1 = NCCELL_TRIVIAL_INITIALIZER;
+    nccell c2 = NCCELL_TRIVIAL_INITIALIZER;
     auto u1 = nccell_load(n_, &c1, wstr); // bytes consumed
     auto u2 = nccell_load(n_, &c2, wstr);
     int o1 = egcpool_stash(&pool_, wstr, u1);
@@ -90,8 +90,8 @@ TEST_CASE("EGCpool") {
 
   SUBCASE("AddTwiceRemoveSecond") {
     const char* wstr = "\u8840"; // cjk unified ideograph, wide
-    nccell c1 = CELL_TRIVIAL_INITIALIZER;
-    nccell c2 = CELL_TRIVIAL_INITIALIZER;
+    nccell c1 = NCCELL_TRIVIAL_INITIALIZER;
+    nccell c2 = NCCELL_TRIVIAL_INITIALIZER;
     auto u1 = nccell_load(n_, &c1, wstr); // bytes consumed
     auto u2 = nccell_load(n_, &c2, wstr);
     int o1 = egcpool_stash(&pool_, wstr, u1);
@@ -119,7 +119,7 @@ TEST_CASE("EGCpool") {
     std::vector<int> candidates;
     char* firstalloc = nullptr;
     for(auto i = 0u ; i < 1u << 20u ; ++i){
-      char mb[MB_CUR_MAX + 1];
+      char mb[MB_LEN_MAX + 1];
       wchar_t wcs = i + 0x80;
       auto r = wctomb(mb, wcs);
       if(r < 0){
@@ -138,7 +138,7 @@ TEST_CASE("EGCpool") {
     REQUIRE(pool_.pool != firstalloc);
     for(auto i = 0u ; i < candidates.size() ; ++i){
       auto stored = pool_.pool + candidates[i];
-      char mb[MB_CUR_MAX + 1];
+      char mb[MB_LEN_MAX + 1];
       wchar_t wcs = i + 0x80;
       auto r = wctomb(mb, wcs);
       if(r < 0){
@@ -158,7 +158,7 @@ TEST_CASE("EGCpool") {
     std::vector<int> candidates;
     char* curpool = nullptr;
     for(auto i = 0u ; i < 1u << 20u ; ++i){
-      char mb[MB_CUR_MAX + 1];
+      char mb[MB_LEN_MAX + 1];
       wchar_t wcs = (i % 0x1000) + 0x80;
       auto r = wctomb(mb, wcs);
       if(r < 0){
@@ -190,7 +190,7 @@ TEST_CASE("EGCpool") {
         continue;
       }
       auto stored = pool_.pool + candidates[i];
-      char mb[MB_CUR_MAX + 1];
+      char mb[MB_LEN_MAX + 1];
       wchar_t wcs = (i % 0x1000) + 0x80;
       auto r = wctomb(mb, wcs);
       if(r < 0){
@@ -219,7 +219,7 @@ TEST_CASE("EGCpoolLong" * doctest::skip(true)) {
     wchar_t wcs = 0x4e00;
     uint64_t total = 0;
     while(true){
-      char mb[MB_CUR_MAX + 1];
+      char mb[MB_LEN_MAX + 1];
       auto r = wctomb(mb, wcs);
       CHECK(0 < r);
       REQUIRE(sizeof(mb) >= r);
@@ -230,7 +230,6 @@ TEST_CASE("EGCpoolLong" * doctest::skip(true)) {
       }
       REQUIRE(loc == total);
       total += r + 1;
-      CHECK(egcpool_check_validity(&pool_, loc));
       CHECK((1u << 25) > loc);
       if(++wcs == 0x9fa5){
         wcs = 0x4e00;
