@@ -16,25 +16,25 @@ TEST_CASE("Fills") {
 
   // can't polyfill with a null glyph
   SUBCASE("PolyfillNullGlyph") {
-    int dimx, dimy;
+    unsigned dimx, dimy;
     ncplane_dim_yx(n_, &dimy, &dimx);
-    nccell c = CELL_TRIVIAL_INITIALIZER;
+    nccell c = NCCELL_TRIVIAL_INITIALIZER;
     CHECK(0 > ncplane_polyfill_yx(n_, dimy, dimx, &c));
   }
 
   // trying to polyfill an invalid cell ought be an error
   SUBCASE("PolyfillOffplane") {
-    int dimx, dimy;
+    unsigned dimx, dimy;
     ncplane_dim_yx(n_, &dimy, &dimx);
-    nccell c = CELL_CHAR_INITIALIZER('+');
+    nccell c = NCCELL_CHAR_INITIALIZER('+');
     CHECK(0 > ncplane_polyfill_yx(n_, dimy, 0, &c));
     CHECK(0 > ncplane_polyfill_yx(n_, 0, dimx, &c));
-    CHECK(0 > ncplane_polyfill_yx(n_, 0, -1, &c));
-    CHECK(0 > ncplane_polyfill_yx(n_, -1, 0, &c));
+    CHECK(0 > ncplane_polyfill_yx(n_, 0, -2, &c));
+    CHECK(0 > ncplane_polyfill_yx(n_, -2, 0, &c));
   }
 
   SUBCASE("PolyfillOnGlyph") {
-    nccell c = CELL_CHAR_INITIALIZER('+');
+    nccell c = NCCELL_CHAR_INITIALIZER('+');
     struct ncplane_options nopts = {
       .y = 0,
       .x = 0,
@@ -44,15 +44,16 @@ TEST_CASE("Fills") {
       .name = nullptr,
       .resizecb = nullptr,
       .flags = 0,
+      .margin_b = 0, .margin_r = 0,
     };
     struct ncplane* pfn = ncplane_create(n_, &nopts);
     REQUIRE(nullptr != pfn);
     CHECK(16 == ncplane_polyfill_yx(pfn, 0, 0, &c));
     CHECK(0 == notcurses_render(nc_));
     CHECK(0 < ncplane_putc_yx(pfn, 0, 0, &c));
-    CHECK(0 < cell_load(pfn, &c, "/"));
+    CHECK(0 < nccell_load(pfn, &c, "/"));
     CHECK(0 < ncplane_polyfill_yx(pfn, 0, 0, &c));
-    char* ncpc = ncplane_at_yx(pfn, 0, 0, NULL, NULL);
+    char* ncpc = ncplane_at_yx(pfn, 0, 0, nullptr, nullptr);
     CHECK(0 == strcmp(ncpc, "/"));
     free(ncpc);
     CHECK(0 == notcurses_render(nc_));
@@ -60,13 +61,13 @@ TEST_CASE("Fills") {
   }
 
   SUBCASE("PolyfillStandardPlane") {
-    nccell c = CELL_CHAR_INITIALIZER('-');
+    nccell c = NCCELL_CHAR_INITIALIZER('-');
     CHECK(0 < ncplane_polyfill_yx(n_, 0, 0, &c));
     CHECK(0 == notcurses_render(nc_));
   }
 
   SUBCASE("PolyfillEmptyPlane") {
-    nccell c = CELL_CHAR_INITIALIZER('+');
+    nccell c = NCCELL_CHAR_INITIALIZER('+');
     struct ncplane_options nopts = {
       .y = 0,
       .x = 0,
@@ -76,6 +77,7 @@ TEST_CASE("Fills") {
       .name = nullptr,
       .resizecb = nullptr,
       .flags = 0,
+      .margin_b = 0, .margin_r = 0,
     };
     struct ncplane* pfn = ncplane_create(n_, &nopts);
     REQUIRE(nullptr != pfn);
@@ -85,7 +87,7 @@ TEST_CASE("Fills") {
   }
 
   SUBCASE("PolyfillWalledPlane") {
-    nccell c = CELL_CHAR_INITIALIZER('+');
+    nccell c = NCCELL_CHAR_INITIALIZER('+');
     struct ncplane_options nopts = {
       .y = 0,
       .x = 0,
@@ -95,6 +97,7 @@ TEST_CASE("Fills") {
       .name = nullptr,
       .resizecb = nullptr,
       .flags = 0,
+      .margin_b = 0, .margin_r = 0,
     };
     struct ncplane* pfn = ncplane_create(n_, &nopts);
     REQUIRE(nullptr != pfn);
@@ -111,18 +114,18 @@ TEST_CASE("Fills") {
 
   SUBCASE("GradientMonochromatic") {
     uint64_t c = 0;
-    channels_set_fg_rgb(&c, 0x40f040);
-    channels_set_bg_rgb(&c, 0x40f040);
-    int dimy, dimx;
+    ncchannels_set_fg_rgb(&c, 0x40f040);
+    ncchannels_set_bg_rgb(&c, 0x40f040);
+    unsigned dimy, dimx;
     ncplane_dim_yx(n_, &dimy, &dimx);
-    REQUIRE(0 < ncplane_gradient_sized(n_, "M", 0, c, c, c, c, dimy, dimx));
-    nccell cl = CELL_TRIVIAL_INITIALIZER;
+    REQUIRE(0 < ncplane_gradient(n_, -1, -1, dimy, dimx, "M", 0, c, c, c, c));
+    nccell cl = NCCELL_TRIVIAL_INITIALIZER;
     uint64_t channels = 0;
-    channels_set_fg_rgb(&channels, 0x40f040);
-    channels_set_bg_rgb(&channels, 0x40f040);
+    ncchannels_set_fg_rgb(&channels, 0x40f040);
+    ncchannels_set_bg_rgb(&channels, 0x40f040);
     // check all squares 
-    for(int y = 0 ; y < dimy ; ++y){
-      for(int x = 0 ; x < dimx ; ++x){
+    for(unsigned y = 0 ; y < dimy ; ++y){
+      for(unsigned x = 0 ; x < dimx ; ++x){
         REQUIRE(0 <= ncplane_at_yx_cell(n_, y, x, &cl));
         CHECK(htole('M') == cl.gcluster);
         CHECK(0 == cl.stylemask);
@@ -135,28 +138,28 @@ TEST_CASE("Fills") {
   SUBCASE("GradientVertical") {
     uint64_t ul, ur, ll, lr;
     ul = ur = ll = lr = 0;
-    channels_set_fg_rgb(&ul, 0x40f040);
-    channels_set_bg_rgb(&ul, 0x40f040);
-    channels_set_fg_rgb(&ll, 0xf040f0);
-    channels_set_bg_rgb(&ll, 0xf040f0);
-    channels_set_fg_rgb(&ur, 0x40f040);
-    channels_set_bg_rgb(&ur, 0x40f040);
-    channels_set_fg_rgb(&lr, 0xf040f0);
-    channels_set_bg_rgb(&lr, 0xf040f0);
-    int dimy, dimx;
+    ncchannels_set_fg_rgb(&ul, 0x40f040);
+    ncchannels_set_bg_rgb(&ul, 0x40f040);
+    ncchannels_set_fg_rgb(&ll, 0xf040f0);
+    ncchannels_set_bg_rgb(&ll, 0xf040f0);
+    ncchannels_set_fg_rgb(&ur, 0x40f040);
+    ncchannels_set_bg_rgb(&ur, 0x40f040);
+    ncchannels_set_fg_rgb(&lr, 0xf040f0);
+    ncchannels_set_bg_rgb(&lr, 0xf040f0);
+    unsigned dimy, dimx;
     ncplane_dim_yx(n_, &dimy, &dimx);
-    REQUIRE(0 < ncplane_gradient_sized(n_, "V", 0, ul, ur, ll, lr, dimy, dimx));
-    nccell c = CELL_TRIVIAL_INITIALIZER;
+    REQUIRE(0 < ncplane_gradient(n_, 0, 0, dimy, dimx, "V", 0, ul, ur, ll, lr));
+    nccell c = NCCELL_TRIVIAL_INITIALIZER;
     uint64_t channels = 0;
-    channels_set_fg_rgb(&channels, 0x40f040);
-    channels_set_bg_rgb(&channels, 0x40f040);
+    ncchannels_set_fg_rgb(&channels, 0x40f040);
+    ncchannels_set_bg_rgb(&channels, 0x40f040);
     // check all squares. all rows ought be the same across their breadth, and
     // the components ought be going in the correct direction.
     uint64_t lastyrgb, lastxrgb;
     lastyrgb = -1;
-    for(int y = 0 ; y < dimy ; ++y){
+    for(unsigned y = 0 ; y < dimy ; ++y){
       lastxrgb = -1;
-      for(int x = 0 ; x < dimx ; ++x){
+      for(unsigned x = 0 ; x < dimx ; ++x){
         REQUIRE(0 <= ncplane_at_yx_cell(n_, y, x, &c));
         CHECK(htole('V') == c.gcluster);
         CHECK(0 == c.stylemask);
@@ -186,17 +189,17 @@ TEST_CASE("Fills") {
   SUBCASE("GradientHorizontal") {
     uint64_t ul, ur, ll, lr;
     ul = ur = ll = lr = 0;
-    channels_set_fg_rgb(&ul, 0x40f040);
-    channels_set_bg_rgb(&ul, 0x40f040);
-    channels_set_fg_rgb(&ur, 0xf040f0);
-    channels_set_bg_rgb(&ur, 0xf040f0);
-    channels_set_fg_rgb(&ll, 0x40f040);
-    channels_set_bg_rgb(&ll, 0x40f040);
-    channels_set_fg_rgb(&lr, 0xf040f0);
-    channels_set_bg_rgb(&lr, 0xf040f0);
-    int dimy, dimx;
+    ncchannels_set_fg_rgb(&ul, 0x40f040);
+    ncchannels_set_bg_rgb(&ul, 0x40f040);
+    ncchannels_set_fg_rgb(&ur, 0xf040f0);
+    ncchannels_set_bg_rgb(&ur, 0xf040f0);
+    ncchannels_set_fg_rgb(&ll, 0x40f040);
+    ncchannels_set_bg_rgb(&ll, 0x40f040);
+    ncchannels_set_fg_rgb(&lr, 0xf040f0);
+    ncchannels_set_bg_rgb(&lr, 0xf040f0);
+    unsigned dimy, dimx;
     ncplane_dim_yx(n_, &dimy, &dimx);
-    REQUIRE(0 < ncplane_gradient_sized(n_, "H", 0, ul, ur, ll, lr, dimy, dimx));
+    REQUIRE(0 < ncplane_gradient(n_, 0, 0, dimy, dimx, "H", 0, ul, ur, ll, lr));
     // check corners FIXME
     CHECK(0 == notcurses_render(nc_));
   }
@@ -204,17 +207,17 @@ TEST_CASE("Fills") {
   SUBCASE("GradientX") {
     uint64_t ul, ur, ll, lr;
     ul = ur = ll = lr = 0;
-    channels_set_fg_rgb(&ul, 0x000000);
-    channels_set_bg_rgb(&ul, 0xffffff);
-    channels_set_fg_rgb(&ll, 0x40f040);
-    channels_set_bg_rgb(&ll, 0x40f040);
-    channels_set_fg_rgb(&ur, 0xf040f0);
-    channels_set_bg_rgb(&ur, 0xf040f0);
-    channels_set_fg_rgb(&lr, 0xffffff);
-    channels_set_bg_rgb(&lr, 0x000000);
-    int dimy, dimx;
+    ncchannels_set_fg_rgb(&ul, 0x000000);
+    ncchannels_set_bg_rgb(&ul, 0xffffff);
+    ncchannels_set_fg_rgb(&ll, 0x40f040);
+    ncchannels_set_bg_rgb(&ll, 0x40f040);
+    ncchannels_set_fg_rgb(&ur, 0xf040f0);
+    ncchannels_set_bg_rgb(&ur, 0xf040f0);
+    ncchannels_set_fg_rgb(&lr, 0xffffff);
+    ncchannels_set_bg_rgb(&lr, 0x000000);
+    unsigned dimy, dimx;
     ncplane_dim_yx(n_, &dimy, &dimx);
-    REQUIRE(0 < ncplane_gradient_sized(n_, "X", 0, ul, ur, ll, lr, dimy, dimx));
+    REQUIRE(0 < ncplane_gradient(n_, 0, 0, dimy, dimx, "X", 0, ul, ur, ll, lr));
     // check corners FIXME
     CHECK(0 == notcurses_render(nc_));
   }
@@ -222,58 +225,55 @@ TEST_CASE("Fills") {
   SUBCASE("GradientS") {
     uint64_t ul, ur, ll, lr;
     ul = ur = ll = lr = 0;
-    channels_set_fg_rgb(&ul, 0xffffff);
-    channels_set_bg_rgb(&ul, 0xffffff);
-    channels_set_fg_rgb(&lr, 0x000000);
-    channels_set_bg_rgb(&lr, 0x000000);
-    channels_set_fg_rgb(&ll, 0x00ffff);
-    channels_set_bg_rgb(&ll, 0xff0000);
-    channels_set_fg_rgb(&ur, 0xff00ff);
-    channels_set_bg_rgb(&ur, 0x00ff00);
-    int dimy, dimx;
+    ncchannels_set_fg_rgb(&ul, 0xffffff);
+    ncchannels_set_bg_rgb(&ul, 0xffffff);
+    ncchannels_set_fg_rgb(&lr, 0x000000);
+    ncchannels_set_bg_rgb(&lr, 0x000000);
+    ncchannels_set_fg_rgb(&ll, 0x00ffff);
+    ncchannels_set_bg_rgb(&ll, 0xff0000);
+    ncchannels_set_fg_rgb(&ur, 0xff00ff);
+    ncchannels_set_bg_rgb(&ur, 0x00ff00);
+    unsigned dimy, dimx;
     ncplane_dim_yx(n_, &dimy, &dimx);
-    REQUIRE(0 < ncplane_gradient_sized(n_, "S", 0, ul, ur, ll, lr, dimy, dimx));
+    REQUIRE(0 < ncplane_gradient(n_, 0, 0, dimy, dimx, "S", 0, ul, ur, ll, lr));
     // check corners FIXME
     CHECK(0 == notcurses_render(nc_));
   }
 
-  SUBCASE("Ncplane_Format") {
-    int sbytes;
+  SUBCASE("Format") {
     CHECK(0 == ncplane_set_fg_rgb(n_, 0x444444));
-    CHECK(1 == ncplane_putegc(n_, "A", &sbytes));
+    CHECK(1 == ncplane_putegc(n_, "A", nullptr));
     CHECK(0 == ncplane_set_fg_rgb(n_, 0x888888));
-    CHECK(1 == ncplane_putegc(n_, "B", &sbytes));
+    CHECK(1 == ncplane_putegc(n_, "B", nullptr));
     CHECK(0 == notcurses_render(nc_));
     // attr should change, but not the EGC/color
     CHECK(0 == ncplane_cursor_move_yx(n_, 0, 0));
-    nccell c = CELL_TRIVIAL_INITIALIZER;
-    cell_on_styles(&c, NCSTYLE_BOLD);
-    CHECK(0 < ncplane_format(n_, 0, 0, c.stylemask));
-    nccell d = CELL_TRIVIAL_INITIALIZER;
+    nccell c = NCCELL_TRIVIAL_INITIALIZER;
+    nccell_on_styles(&c, NCSTYLE_BOLD);
+    CHECK(0 < ncplane_format(n_, 0, 0, 0, 0, c.stylemask));
+    nccell d = NCCELL_TRIVIAL_INITIALIZER;
     CHECK(1 == ncplane_at_yx_cell(n_, 0, 0, &d));
     CHECK(d.stylemask == c.stylemask);
-    CHECK(0x444444 == cell_fg_rgb(&d));
+    CHECK(0x444444 == nccell_fg_rgb(&d));
   }
 
-  SUBCASE("Ncplane_Stain") {
-    int sbytes;
+  SUBCASE("Stain") {
     CHECK(0 == ncplane_set_fg_rgb(n_, 0x444444));
-    for(int y = 0 ; y < 8 ; ++y){
-      for(int x = 0 ; x < 8 ; ++x){
-        CHECK(1 == ncplane_putegc_yx(n_, y, x, "A", &sbytes));
+    for(unsigned y = 0 ; y < 8 ; ++y){
+      for(unsigned x = 0 ; x < 8 ; ++x){
+        CHECK(1 == ncplane_putegc_yx(n_, y, x, "A", nullptr));
       }
     }
     CHECK(0 == notcurses_render(nc_));
     // EGC/color should change, but nothing else
-    CHECK(0 == ncplane_cursor_move_yx(n_, 0, 0));
     uint64_t channels = 0;
-    channels_set_fg_rgb8(&channels, 0x88, 0x99, 0x77);
-    channels_set_bg_rgb(&channels, 0);
-    REQUIRE(0 < ncplane_stain(n_, 7, 7, channels, channels, channels, channels));
+    ncchannels_set_fg_rgb8(&channels, 0x88, 0x99, 0x77);
+    ncchannels_set_bg_rgb(&channels, 0);
+    REQUIRE(0 < ncplane_stain(n_, 0, 0, 7, 7, channels, channels, channels, channels));
     CHECK(0 == notcurses_render(nc_));
-    nccell d = CELL_TRIVIAL_INITIALIZER;
-    for(int y = 0 ; y < 8 ; ++y){
-      for(int x = 0 ; x < 8 ; ++x){
+    nccell d = NCCELL_TRIVIAL_INITIALIZER;
+    for(unsigned y = 0 ; y < 7 ; ++y){
+      for(unsigned x = 0 ; x < 7 ; ++x){
         CHECK(1 == ncplane_at_yx_cell(n_, y, x, &d));
         CHECK(channels == d.channels);
         REQUIRE(cell_simple_p(&d));
@@ -284,17 +284,15 @@ TEST_CASE("Fills") {
 
   // test the single-cell (1x1) special case
   SUBCASE("GradientSingleCell") {
-    int sbytes;
     CHECK(0 == ncplane_set_fg_rgb(n_, 0x444444));
-    CHECK(1 == ncplane_putegc_yx(n_, 0, 0, "A", &sbytes));
+    CHECK(1 == ncplane_putegc_yx(n_, 0, 0, "A", nullptr));
     CHECK(0 == notcurses_render(nc_));
-    CHECK(0 == ncplane_cursor_move_yx(n_, 0, 0));
     uint64_t channels = 0;
-    channels_set_fg_rgb8(&channels, 0x88, 0x99, 0x77);
-    channels_set_bg_rgb(&channels, 0);
-    REQUIRE(0 < ncplane_gradient(n_, "A", 0, channels, channels, channels, channels, 0, 0));
+    ncchannels_set_fg_rgb8(&channels, 0x88, 0x99, 0x77);
+    ncchannels_set_bg_rgb(&channels, 0);
+    REQUIRE(0 < ncplane_gradient(n_, 0, 0, 0, 0, "A", 0, channels, channels, channels, channels));
     CHECK(0 == notcurses_render(nc_));
-    nccell d = CELL_TRIVIAL_INITIALIZER;
+    nccell d = NCCELL_TRIVIAL_INITIALIZER;
     CHECK(1 == ncplane_at_yx_cell(n_, 0, 0, &d));
     CHECK(channels == d.channels);
     REQUIRE(cell_simple_p(&d));
@@ -303,56 +301,51 @@ TEST_CASE("Fills") {
 
   // 1d gradients over multiple cells
   SUBCASE("Gradient1D") {
-    int sbytes;
     CHECK(0 == ncplane_set_fg_rgb(n_, 0x444444));
-    CHECK(1 == ncplane_putegc_yx(n_, 0, 0, "A", &sbytes));
+    CHECK(1 == ncplane_putegc_yx(n_, 0, 0, "A", nullptr));
     CHECK(0 == notcurses_render(nc_));
     CHECK(0 == ncplane_cursor_move_yx(n_, 0, 0));
     uint64_t chan1 = 0, chan2 = 0;
-    channels_set_fg_rgb8(&chan1, 0x88, 0x99, 0x77);
-    channels_set_fg_rgb8(&chan2, 0x77, 0x99, 0x88);
-    channels_set_bg_rgb(&chan1, 0);
-    channels_set_bg_rgb(&chan2, 0);
-    REQUIRE(0 < ncplane_gradient(n_, "A", 0, chan1, chan2, chan1, chan2, 0, 3));
+    ncchannels_set_fg_rgb8(&chan1, 0x88, 0x99, 0x77);
+    ncchannels_set_fg_rgb8(&chan2, 0x77, 0x99, 0x88);
+    ncchannels_set_bg_rgb(&chan1, 0);
+    ncchannels_set_bg_rgb(&chan2, 0);
+    CHECK(0 < ncplane_gradient(n_, 0, 0, 0, 3, "A", 0, chan1, chan2, chan1, chan2));
     CHECK(0 == notcurses_render(nc_));
-    nccell d = CELL_TRIVIAL_INITIALIZER;
+    nccell d = NCCELL_TRIVIAL_INITIALIZER;
     CHECK(1 == ncplane_at_yx_cell(n_, 0, 0, &d));
     CHECK(chan1 == d.channels);
-    REQUIRE(cell_simple_p(&d));
+    CHECK(cell_simple_p(&d));
     CHECK(htole('A') == d.gcluster);
-    CHECK(0 == ncplane_cursor_move_yx(n_, 0, 0));
-    REQUIRE(0 < ncplane_gradient(n_, "A", 0, chan2, chan1, chan2, chan1, 0, 3));
+    CHECK(0 < ncplane_gradient(n_, 0, 0, 0, 3, "A", 0, chan2, chan1, chan2, chan1));
     CHECK(1 == ncplane_at_yx_cell(n_, 0, 0, &d));
-    REQUIRE(cell_simple_p(&d));
+    CHECK(cell_simple_p(&d));
     CHECK(htole('A') == d.gcluster);
     CHECK(chan2 == d.channels);
   }
 
   // Unlike a typical gradient, a high gradient ought be able to do a vertical
-  // change in a single row.
+  // change in a single row (though not a single column).
   SUBCASE("HighGradient2Colors1Row") {
     uint32_t ul, ur, ll, lr;
     ul = ur = ll = lr = 0;
-    channel_set(&ul, 0xffffff);
-    channel_set(&lr, 0x000000);
-    channel_set(&ll, 0x00ffff);
-    channel_set(&ur, 0xff00ff);
-    int dimy, dimx;
-    ncplane_dim_yx(n_, &dimy, &dimx);
-    REQUIRE(0 < ncplane_highgradient_sized(n_, ul, ur, ll, lr, dimy, dimx));
+    ncchannel_set(&ul, 0xffffff);
+    ncchannel_set(&lr, 0x000000);
+    ncchannel_set(&ll, 0x00ffff);
+    ncchannel_set(&ur, 0xff00ff);
+    CHECK(0 > ncplane_gradient2x1(n_, 0, 0, 1, 1, ul, ur, ll, lr));
+    CHECK(0 < ncplane_gradient2x1(n_, 0, 0, 1, 2, ul, ur, ll, lr));
     CHECK(0 == notcurses_render(nc_));
   }
 
   SUBCASE("HighGradient") {
     uint32_t ul, ur, ll, lr;
     ul = ur = ll = lr = 0;
-    channel_set(&ul, 0xffffff);
-    channel_set(&lr, 0x000000);
-    channel_set(&ll, 0x00ffff);
-    channel_set(&ur, 0xff00ff);
-    int dimy, dimx;
-    ncplane_dim_yx(n_, &dimy, &dimx);
-    REQUIRE(0 < ncplane_highgradient_sized(n_, ul, ur, ll, lr, dimy, dimx));
+    ncchannel_set(&ul, 0xffffff);
+    ncchannel_set(&lr, 0x000000);
+    ncchannel_set(&ll, 0x00ffff);
+    ncchannel_set(&ur, 0xff00ff);
+    CHECK(0 < ncplane_gradient2x1(n_, 0, 0, 0, 0, ul, ur, ll, lr));
     CHECK(0 == notcurses_render(nc_));
   }
 
@@ -366,18 +359,19 @@ TEST_CASE("Fills") {
       .name = nullptr,
       .resizecb = nullptr,
       .flags = 0,
+      .margin_b = 0, .margin_r = 0,
     };
     struct ncplane* p1 = ncplane_create(n_, &nopts);
     REQUIRE(p1);
     // make sure glyphs replace nulls
     CHECK(0 < ncplane_putstr(p1, "0123456789"));
     CHECK(0 == ncplane_mergedown_simple(p1, n_));
-    nccell cbase = CELL_TRIVIAL_INITIALIZER;
-    nccell cp = CELL_TRIVIAL_INITIALIZER;
+    nccell cbase = NCCELL_TRIVIAL_INITIALIZER;
+    nccell cp = NCCELL_TRIVIAL_INITIALIZER;
     for(int i = 0 ; i < 10 ; ++i){
       CHECK(0 < ncplane_at_yx_cell(n_, 0, i, &cbase));
       CHECK(0 < ncplane_at_yx_cell(p1, 0, i, &cp));
-      CHECK(0 == cellcmp(n_, &cbase, p1, &cp));
+      CHECK(0 == nccellcmp(n_, &cbase, p1, &cp));
     }
     CHECK(0 == ncplane_cursor_move_yx(p1, 0, 0));
     // make sure glyphs replace glyps
@@ -386,7 +380,7 @@ TEST_CASE("Fills") {
     for(int i = 0 ; i < 10 ; ++i){
       CHECK(0 < ncplane_at_yx_cell(n_, 0, i, &cbase));
       CHECK(0 < ncplane_at_yx_cell(p1, 0, i, &cp));
-      CHECK(0 == cellcmp(n_, &cbase, p1, &cp));
+      CHECK(0 == nccellcmp(n_, &cbase, p1, &cp));
     }
     // make sure nulls do not replace glyphs
     auto p2 = ncplane_create(n_, &nopts);
@@ -395,7 +389,7 @@ TEST_CASE("Fills") {
     for(int i = 0 ; i < 10 ; ++i){
       CHECK(0 < ncplane_at_yx_cell(n_, 0, i, &cbase));
       CHECK(0 < ncplane_at_yx_cell(p1, 0, i, &cp));
-      CHECK(0 == cellcmp(n_, &cbase, p1, &cp));
+      CHECK(0 == nccellcmp(n_, &cbase, p1, &cp));
     }
     ncplane_destroy(p1);
   }
@@ -410,18 +404,19 @@ TEST_CASE("Fills") {
       .name = nullptr,
       .resizecb = nullptr,
       .flags = 0,
+      .margin_b = 0, .margin_r = 0,
     };
     auto p1 = ncplane_create(n_, &nopts);
     REQUIRE(p1);
     // make sure glyphs replace nulls
     CHECK(0 < ncplane_putstr(p1, "█▀▄▌▐🞵🞶🞷🞸🞹"));
     CHECK(0 == ncplane_mergedown_simple(p1, n_));
-    nccell cbase = CELL_TRIVIAL_INITIALIZER;
-    nccell cp = CELL_TRIVIAL_INITIALIZER;
+    nccell cbase = NCCELL_TRIVIAL_INITIALIZER;
+    nccell cp = NCCELL_TRIVIAL_INITIALIZER;
     for(int i = 0 ; i < 10 ; ++i){
       CHECK(0 < ncplane_at_yx_cell(n_, 0, i, &cbase));
       CHECK(0 < ncplane_at_yx_cell(p1, 0, i, &cp));
-      CHECK(0 == cellcmp(n_, &cbase, p1, &cp));
+      CHECK(0 == nccellcmp(n_, &cbase, p1, &cp));
     }
     ncplane_destroy(p1);
     CHECK(0 == notcurses_render(nc_));
@@ -429,22 +424,22 @@ TEST_CASE("Fills") {
     CHECK(0 == ncplane_cursor_move_yx(p3, 0, 0));
     // make sure glyphs replace glyps
     CHECK(0 < ncplane_putstr(p3, "🞵🞶🞷🞸🞹█▀▄▌▐"));
-    CHECK(0 == ncplane_mergedown_simple(p3, nullptr));
-    nccell c3 = CELL_TRIVIAL_INITIALIZER;
+    CHECK(0 == ncplane_mergedown_simple(p3, n_));
+    nccell c3 = NCCELL_TRIVIAL_INITIALIZER;
     for(int i = 0 ; i < 10 ; ++i){
       CHECK(0 < ncplane_at_yx_cell(n_, 0, i, &cbase));
       CHECK(0 < ncplane_at_yx_cell(p3, 0, i, &c3));
-      CHECK(0 == cellcmp(n_, &cbase, p3, &c3));
+      CHECK(0 == nccellcmp(n_, &cbase, p3, &c3));
     }
     CHECK(0 == notcurses_render(nc_));
     // make sure nulls do not replace glyphs
     auto p2 = ncplane_create(n_, &nopts);
-    CHECK(0 == ncplane_mergedown_simple(p2, nullptr));
+    CHECK(0 == ncplane_mergedown_simple(p2, n_));
     ncplane_destroy(p2);
     for(int i = 0 ; i < 10 ; ++i){
       CHECK(0 < ncplane_at_yx_cell(n_, 0, i, &cbase));
       CHECK(0 < ncplane_at_yx_cell(p3, 0, i, &c3));
-      CHECK(0 == cellcmp(n_, &cbase, p3, &c3));
+      CHECK(0 == nccellcmp(n_, &cbase, p3, &c3));
     }
     ncplane_destroy(p3);
     CHECK(0 == notcurses_render(nc_));
@@ -464,13 +459,14 @@ TEST_CASE("Fills") {
       .name = nullptr,
       .resizecb = nullptr,
       .flags = 0,
+      .margin_b = 0, .margin_r = 0,
     };
     struct ncplane* p1 = ncplane_create(n_, &nopts);
     REQUIRE(p1);
-    nccell c1 = CELL_TRIVIAL_INITIALIZER;
-    CHECK(0 < cell_load(p1, &c1, "█"));
-    CHECK(0 == cell_set_bg_rgb(&c1, 0x00ff00));
-    CHECK(0 == cell_set_fg_rgb(&c1, 0x0000ff));
+    nccell c1 = NCCELL_TRIVIAL_INITIALIZER;
+    CHECK(0 < nccell_load(p1, &c1, "█"));
+    CHECK(0 == nccell_set_bg_rgb(&c1, 0x00ff00));
+    CHECK(0 == nccell_set_fg_rgb(&c1, 0x0000ff));
     CHECK(0 < ncplane_polyfill_yx(p1, 0, 0, &c1));
     CHECK(0 == notcurses_render(nc_));
     struct ncplane_options n2opts = {
@@ -482,26 +478,28 @@ TEST_CASE("Fills") {
       .name = nullptr,
       .resizecb = nullptr,
       .flags = 0,
+      .margin_b = 0, .margin_r = 0,
     };
     auto p2 = ncplane_create(n_, &n2opts);
     REQUIRE(p2);
-    nccell c2 = CELL_TRIVIAL_INITIALIZER;
-    CHECK(0 < cell_load(p2, &c2, "🞶"));
-    CHECK(0 == cell_set_bg_rgb(&c2, 0x00ffff));
-    CHECK(0 == cell_set_fg_rgb(&c2, 0xff00ff));
+    nccell c2 = NCCELL_TRIVIAL_INITIALIZER;
+    CHECK(0 < nccell_load(p2, &c2, "🞶"));
+    CHECK(0 == nccell_set_bg_rgb(&c2, 0x00ffff));
+    CHECK(0 == nccell_set_fg_rgb(&c2, 0xff00ff));
     CHECK(0 < ncplane_polyfill_yx(p2, 0, 0, &c2));
     CHECK(0 == ncplane_mergedown_simple(p2, p1));
     CHECK(0 == notcurses_render(nc_));
-    for(int y = 0 ; y < DIMY ; ++y){
-      for(int x = 0 ; x < DIMX ; ++x){
+    for(unsigned y = 0 ; y < DIMY ; ++y){
+      for(unsigned x = 0 ; x < DIMX ; ++x){
         CHECK(0 < ncplane_at_yx_cell(p1, y, x, &c1));
         if(y < 1 || y > 5 || x < 1 || x > 5){
-          auto cstr = cell_strdup(p1, &c1);
+          auto cstr = nccell_strdup(p1, &c1);
+          REQUIRE(nullptr != cstr);
           CHECK(0 == strcmp(cstr, "█"));
           free(cstr);
         }else{
           CHECK(0 < ncplane_at_yx_cell(p2, y - 1, x - 1, &c2));
-          CHECK(0 == cellcmp(p1, &c1, p2, &c2));
+          CHECK(0 == nccellcmp(p1, &c1, p2, &c2));
         }
       }
     }
@@ -523,14 +521,15 @@ TEST_CASE("Fills") {
       .name = nullptr,
       .resizecb = nullptr,
       .flags = 0,
+      .margin_b = 0, .margin_r = 0,
     };
     struct ncplane* p1 = ncplane_create(n_, &nopts);
     REQUIRE(p1);
     uint64_t ul = 0, ur = 0, bl = 0, br = 0;
-    channels_set_fg_rgb(&ur, 0xff0000);
-    channels_set_fg_rgb(&bl, 0x00ff00);
-    channels_set_fg_rgb(&br, 0x0000ff);
-    ncplane_highgradient_sized(p1, ul, ur, bl, br, DIMY, DIMX);
+    ncchannels_set_fg_rgb(&ur, 0xff0000);
+    ncchannels_set_fg_rgb(&bl, 0x00ff00);
+    ncchannels_set_fg_rgb(&br, 0x0000ff);
+    ncplane_gradient2x1(p1, 0, 0, DIMY, DIMX, ul, ur, bl, br);
     CHECK(0 == notcurses_render(nc_));
     struct ncplane_options n2opts = {
       .y = 3,
@@ -541,27 +540,28 @@ TEST_CASE("Fills") {
       .name = nullptr,
       .resizecb = nullptr,
       .flags = 0,
+      .margin_b = 0, .margin_r = 0,
     };
     auto p2 = ncplane_create(n_, &n2opts);
     REQUIRE(p2);
-    ncplane_highgradient_sized(p2, br, bl, ur, ul, DIMY / 2, DIMX / 2);
+    ncplane_gradient2x1(p2, 0, 0, DIMY / 2, DIMX / 2, br, bl, ur, ul);
     CHECK(0 == ncplane_mergedown_simple(p2, p1));
     CHECK(0 == notcurses_render(nc_));
-    for(int y = 0 ; y < DIMY ; ++y){
-      for(int x = 0 ; x < DIMX ; ++x){
-        nccell c1 = CELL_TRIVIAL_INITIALIZER;
+    for(unsigned y = 0 ; y < DIMY ; ++y){
+      for(unsigned x = 0 ; x < DIMX ; ++x){
+        nccell c1 = NCCELL_TRIVIAL_INITIALIZER;
         CHECK(0 < ncplane_at_yx_cell(p1, y, x, &c1));
         if(y < 1 || y > 5 || x < 1 || x > 5){
-          auto cstr = cell_strdup(p1, &c1);
+          auto cstr = nccell_strdup(p1, &c1);
           CHECK(0 == strcmp(cstr, "▀"));
           free(cstr);
         }else{
-          nccell c2 = CELL_TRIVIAL_INITIALIZER;
+          nccell c2 = NCCELL_TRIVIAL_INITIALIZER;
           CHECK(0 < ncplane_at_yx_cell(p2, y - 1, x - 1, &c2));
-          CHECK(0 == cellcmp(p1, &c1, p2, &c2));
-          cell_release(p2, &c2);
+          CHECK(0 == nccellcmp(p1, &c1, p2, &c2));
+          nccell_release(p2, &c2);
         }
-        cell_release(p1, &c1);
+        nccell_release(p1, &c1);
       }
     }
     ncplane_destroy(p1);
@@ -571,9 +571,11 @@ TEST_CASE("Fills") {
 #ifdef USE_QRCODEGEN
   SUBCASE("QRCodes") {
     const char* qr = "a very simple qr code";
-    int dimy, dimx;
+    unsigned dimy, dimx;
     ncplane_dim_yx(n_, &dimy, &dimx);
-    CHECK(0 < ncplane_qrcode(n_, &dimy, &dimx, qr, strlen(qr)));
+    unsigned sdimy = dimy;
+    unsigned sdimx = dimx;
+    CHECK(0 < ncplane_qrcode(n_, &sdimy, &sdimx, qr, strlen(qr)));
     CHECK(0 == notcurses_render(nc_));
   }
 #endif
