@@ -1,11 +1,10 @@
 #include <array>
 #include <cstdlib>
 #include "main.h"
-#include "internal.h"
 
-const char SNAKE[] = "\xf0\x9f\x90\x8d"; // U+1F40D SNAKE
-const char SCORPION[] = "\xf0\x9f\xa6\x82"; // U+1F982 SCORPION
-const char FROG[] = "\xf0\x9f\x90\xb8"; // U+1F438 FROG FACE
+const char SNAKE[] = u8"\xf0\x9f\x90\x8d"; // U+1F40D SNAKE
+const char SCORPION[] = u8"\xf0\x9f\xa6\x82"; // U+1F982 SCORPION
+const char FROG[] = u8"\xf0\x9f\x90\xb8"; // U+1F438 FROG FACE
 
 TEST_CASE("Wide") {
   auto nc_ = testing_notcurses();
@@ -19,10 +18,9 @@ TEST_CASE("Wide") {
 
   // Verify we can emit a wide character, and it advances the cursor by 2
   SUBCASE("EmitWideAsian") {
-    const char* w = "\u5168";
-    int sbytes = 0;
-    CHECK(0 < ncplane_putegc(n_, w, &sbytes));
-    int x, y;
+    const char* w = u8"\u5168";
+    CHECK(0 < ncplane_putegc(n_, w, nullptr));
+    unsigned x, y;
     ncplane_cursor_yx(n_, &y, &x);
     CHECK(0 == y);
     CHECK(2 == x);
@@ -31,17 +29,16 @@ TEST_CASE("Wide") {
 
   // Verify a wide character is rejected with cursor on the last column
   SUBCASE("RejectWideAsian") {
-    const char* w = "\u5168";
-    int sbytes = 0;
-    int dimx;
+    const char* w = u8"\u5168";
+    unsigned dimx;
     ncplane_dim_yx(n_, nullptr, &dimx);
-    CHECK(0 < ncplane_putegc_yx(n_, 0, dimx - 3, w, &sbytes));
-    int x, y;
+    CHECK(0 < ncplane_putegc_yx(n_, 0, dimx - 3, w, nullptr));
+    unsigned x, y;
     ncplane_cursor_yx(n_, &y, &x);
     CHECK(0 == y);
     CHECK(dimx - 1 == x);
     // now it ought be rejected
-    CHECK(0 > ncplane_putegc(n_, w, &sbytes));
+    CHECK(0 > ncplane_putegc(n_, w, nullptr));
     // cursor ought remain where it was
     ncplane_cursor_yx(n_, &y, &x);
     CHECK(0 == y);
@@ -51,31 +48,26 @@ TEST_CASE("Wide") {
 
   // Verify a wide character is rejected when placed on the last column
   SUBCASE("RejectWideAsianPlaced") {
-    const char* w = "\u5168";
-    int sbytes = 0;
-    int dimx;
+    const char* w = u8"\u5168";
+    unsigned dimx;
     ncplane_dim_yx(n_, nullptr, &dimx);
     // now it ought be rejected
-    CHECK(0 > ncplane_putegc_yx(n_, 0, dimx - 1, w, &sbytes));
+    CHECK(0 > ncplane_putegc_yx(n_, 0, dimx - 1, w, nullptr));
     // cursor ought remain where it was
-    int y, x;
+    unsigned y, x;
     ncplane_cursor_yx(n_, &y, &x);
     CHECK(0 == y);
-    CHECK(dimx - 1 == x);
+    CHECK(0 == x);
     CHECK(0 == notcurses_render(nc_));
   }
 
   // half-width, double-width, huge-yet-narrow, all that crap
   SUBCASE("PlaneAtCursorInsane"){
-    const char EGC0[] = "\uffe0"; // fullwidth cent sign ￠
-    const char EGC1[] = "\u00c5"; // neutral A with ring above Å
-    const char EGC2[] = "\u20a9"; // half-width won ₩
-    const char EGC3[] = "\u212b"; // ambiguous angstrom Å
-#ifdef __APPLE__ // FIXME
-    const char EGC4[] = "\U00010E7D";
-#else
-    const char EGC4[] = "\ufdfd"; // neutral yet huge bismillah ﷽
-#endif
+    const char EGC0[] = u8"\uffe0"; // fullwidth cent sign ￠
+    const char EGC1[] = u8"\u00c5"; // neutral A with ring above Å
+    const char EGC2[] = u8"\u20a9"; // half-width won ₩
+    const char EGC3[] = u8"\u212b"; // ambiguous angstrom Å
+    const char EGC4[] = u8"\ufdfd"; // neutral yet huge bismillah ﷽
     std::array<nccell, 5> tcells;
     for(auto & tcell : tcells){
       nccell_init(&tcell);
@@ -92,7 +84,7 @@ TEST_CASE("Wide") {
     int x = 0;
     for(auto & tcell : tcells){
       CHECK(0 == ncplane_cursor_move_yx(n_, 0, x));
-      nccell testcell = CELL_TRIVIAL_INITIALIZER;
+      nccell testcell = NCCELL_TRIVIAL_INITIALIZER;
       CHECK(0 < ncplane_at_cursor_cell(n_, &testcell));
       CHECK(!strcmp(nccell_extended_gcluster(n_, &tcell), nccell_extended_gcluster(n_, &testcell)));
       CHECK(0 == testcell.stylemask);
@@ -114,28 +106,27 @@ TEST_CASE("Wide") {
     const char* w = FROG;
     const char* wbashed = SCORPION;
     const char bashed = 'X';
-    int sbytes = 0;
-    CHECK(0 < ncplane_putegc_yx(n_, 0, 1, wbashed, &sbytes));
+    CHECK(0 < ncplane_putegc_yx(n_, 0, 1, wbashed, nullptr));
     CHECK(0 < ncplane_putchar_yx(n_, 1, 1, bashed));
-    int x, y;
+    unsigned x, y;
     ncplane_cursor_yx(n_, &y, &x);
     CHECK(1 == y);
     CHECK(2 == x);
-    CHECK(0 < ncplane_putegc_yx(n_, 0, 0, w, &sbytes));
-    nccell c = CELL_TRIVIAL_INITIALIZER;
+    CHECK(0 < ncplane_putegc_yx(n_, 0, 0, w, nullptr));
+    nccell c = NCCELL_TRIVIAL_INITIALIZER;
     CHECK(0 < nccell_load(n_, &c, w));
     CHECK(0 < ncplane_putc_yx(n_, 1, 0, &c));
     nccell_release(n_, &c);
     ncplane_at_yx_cell(n_, 0, 0, &c);
     CHECK(0 == strcmp(nccell_extended_gcluster(n_, &c), FROG));
     ncplane_at_yx_cell(n_, 0, 1, &c);
-    CHECK(ncstrwidth(FROG) == 1 + nccell_double_wide_p(&c)); // should be wide
+    CHECK(ncstrwidth(FROG, NULL, NULL) == 1 + nccell_double_wide_p(&c)); // should be wide
     ncplane_at_yx_cell(n_, 0, 2, &c);
     CHECK(0 == strlen(nccell_extended_gcluster(n_, &c))); // should be nothing
     ncplane_at_yx_cell(n_, 1, 0, &c);
     CHECK(0 == strcmp(nccell_extended_gcluster(n_, &c), FROG));
     ncplane_at_yx_cell(n_, 1, 1, &c);
-    CHECK(ncstrwidth(FROG) == 1 + nccell_double_wide_p(&c)); //should be wide
+    CHECK(ncstrwidth(FROG, NULL, NULL) == 1 + nccell_double_wide_p(&c)); //should be wide
     CHECK(0 == notcurses_render(nc_)); // should be nothing
   }
 
@@ -144,16 +135,15 @@ TEST_CASE("Wide") {
   SUBCASE("WideCharAnnihilatesWideLeft") {
     const char* w = SNAKE;
     const char* wbashed = SCORPION;
-    int sbytes = 0;
-    int cols1 = ncplane_putegc_yx(n_, 0, 0, wbashed, &sbytes);
+    int cols1 = ncplane_putegc_yx(n_, 0, 0, wbashed, nullptr);
     CHECK(0 < cols1);
-    int cols2 = ncplane_putegc_yx(n_, 0, 1, w, &sbytes);
+    int cols2 = ncplane_putegc_yx(n_, 0, 1, w, nullptr);
     CHECK(0 < cols2);
-    int x, y;
+    unsigned x, y;
     ncplane_cursor_yx(n_, &y, &x);
     CHECK(0 == y);
     CHECK(1 + cols2 == x);
-    nccell c = CELL_TRIVIAL_INITIALIZER;
+    nccell c = NCCELL_TRIVIAL_INITIALIZER;
     ncplane_at_yx_cell(n_, 0, 0, &c);
     if(cols1 > 1){
       CHECK(0 == c.gcluster); // should be nothing
@@ -161,7 +151,7 @@ TEST_CASE("Wide") {
     ncplane_at_yx_cell(n_, 0, 1, &c);
     CHECK(0 == strcmp(nccell_extended_gcluster(n_, &c), SNAKE));
     ncplane_at_yx_cell(n_, 0, 2, &c);
-    CHECK(ncstrwidth(SNAKE) == 1 + nccell_double_wide_p(&c)); // should be wide
+    CHECK(ncstrwidth(SNAKE, NULL, NULL) == 1 + nccell_double_wide_p(&c)); // should be wide
     CHECK(0 == notcurses_render(nc_));
   }
 
@@ -171,18 +161,17 @@ TEST_CASE("Wide") {
     const char* cc = "X";
     const char* wbashedl = SNAKE;
     const char* wbashedr = SCORPION;
-    int sbytes = 0;
-    CHECK(0 < ncplane_putegc_yx(n_, 0, 0, wbashedl, &sbytes));
-    CHECK(0 < ncplane_putegc_yx(n_, 0, 2, wbashedr, &sbytes));
+    CHECK(0 < ncplane_putegc_yx(n_, 0, 0, wbashedl, nullptr));
+    CHECK(0 < ncplane_putegc_yx(n_, 0, 2, wbashedr, nullptr));
     CHECK(1 == ncplane_putchar_yx(n_, 0, 1, *cc));
     CHECK(1 == ncplane_putchar_yx(n_, 0, 2, *cc));
-    int x, y;
+    unsigned x, y;
     ncplane_cursor_yx(n_, &y, &x);
     CHECK(0 == y);
     CHECK(3 == x);
-    nccell c = CELL_TRIVIAL_INITIALIZER;
+    nccell c = NCCELL_TRIVIAL_INITIALIZER;
     ncplane_at_yx_cell(n_, 0, 0, &c);
-    if(ncstrwidth(wbashedl) > 1){
+    if(ncstrwidth(wbashedl, NULL, NULL) > 1){
       CHECK(0 == c.gcluster); // should be nothing
     }
     ncplane_at_yx_cell(n_, 0, 1, &c);
@@ -190,7 +179,7 @@ TEST_CASE("Wide") {
     ncplane_at_yx_cell(n_, 0, 2, &c);
     CHECK(0 == strcmp(cc, nccell_extended_gcluster(n_, &c))); // should be 'X'
     ncplane_at_yx_cell(n_, 0, 3, &c);
-    if(ncstrwidth(wbashedr) > 1){
+    if(ncstrwidth(wbashedr, NULL, NULL) > 1){
       CHECK(0 == strlen(nccell_extended_gcluster(n_, &c))); // should be nothing
     }
     CHECK(0 == notcurses_render(nc_));
@@ -202,25 +191,24 @@ TEST_CASE("Wide") {
     const char* cc = "X";
     const char* wsafel = SNAKE;
     const char* wsafer = SCORPION;
-    int sbytes = 0;
-    CHECK(0 < ncplane_putegc_yx(n_, 0, 0, wsafel, &sbytes));
-    CHECK(0 < ncplane_putegc_yx(n_, 0, 3, wsafer, &sbytes));
+    CHECK(0 < ncplane_putegc_yx(n_, 0, 0, wsafel, nullptr));
+    CHECK(0 < ncplane_putegc_yx(n_, 0, 3, wsafer, nullptr));
     CHECK(1 == ncplane_putchar_yx(n_, 0, 2, *cc));
-    int x, y;
+    unsigned x, y;
     ncplane_cursor_yx(n_, &y, &x);
     CHECK(0 == y);
     CHECK(3 == x);
-    nccell c = CELL_TRIVIAL_INITIALIZER;
+    nccell c = NCCELL_TRIVIAL_INITIALIZER;
     ncplane_at_yx_cell(n_, 0, 0, &c);
     CHECK(0 == strcmp(nccell_extended_gcluster(n_, &c), SNAKE));
     ncplane_at_yx_cell(n_, 0, 1, &c);
-    CHECK(ncstrwidth(SNAKE) == 1 + nccell_double_wide_p(&c));
+    CHECK(ncstrwidth(SNAKE, NULL, NULL) == 1 + nccell_double_wide_p(&c));
     ncplane_at_yx_cell(n_, 0, 2, &c);
     CHECK(0 == strcmp(cc, nccell_extended_gcluster(n_, &c))); // should be 'X'
     ncplane_at_yx_cell(n_, 0, 3, &c);
     CHECK(0 == strcmp(nccell_extended_gcluster(n_, &c), SCORPION));
     ncplane_at_yx_cell(n_, 0, 4, &c);
-    CHECK(ncstrwidth(SCORPION) == 1 + nccell_double_wide_p(&c));
+    CHECK(ncstrwidth(SCORPION, NULL, NULL) == 1 + nccell_double_wide_p(&c));
     CHECK(0 == notcurses_render(nc_));
   }
 
@@ -238,12 +226,12 @@ TEST_CASE("Wide") {
     };
     struct ncplane* ncp = ncplane_create(n_, &nopts);
     REQUIRE(ncp);
-    int dimx, dimy;
+    unsigned dimx, dimy;
     ncplane_dim_yx(n_, &dimy, &dimx);
     CHECK(0 == ncplane_rounded_box_sized(ncp, 0, 0, 3, 4, 0));
-    CHECK(ncstrwidth(SCORPION) == ncplane_putegc_yx(ncp, 1, 1, SCORPION, nullptr));
+    CHECK(ncstrwidth(SCORPION, NULL, NULL) == ncplane_putegc_yx(ncp, 1, 1, SCORPION, nullptr));
     CHECK(0 == notcurses_render(nc_));
-    nccell c = CELL_TRIVIAL_INITIALIZER;
+    nccell c = NCCELL_TRIVIAL_INITIALIZER;
     CHECK(0 < ncplane_at_yx_cell(ncp, 1, 0, &c));
     CHECK(!strcmp(nccell_extended_gcluster(ncp, &c), "│"));
     nccell_release(ncp, &c);
@@ -262,8 +250,8 @@ TEST_CASE("Wide") {
   }
 
   SUBCASE("RenderWides") {
-    CHECK(0 <= ncplane_putstr(n_, "\u5f62\u5168"));
-    nccell c = CELL_TRIVIAL_INITIALIZER;
+    CHECK(0 <= ncplane_putstr(n_, u8"\u5f62\u5168"));
+    nccell c = NCCELL_TRIVIAL_INITIALIZER;
     ncplane_at_yx_cell(n_, 0, 0, &c);
     CHECK(nccell_double_wide_p(&c));
     ncplane_at_yx_cell(n_, 0, 1, &c);
@@ -277,35 +265,41 @@ TEST_CASE("Wide") {
     CHECK(0 == notcurses_render(nc_));
     auto egc = notcurses_at_yx(nc_, 0, 0, &c.stylemask, &c.channels);
     REQUIRE(nullptr != egc);
-    CHECK(0 == strcmp("\u5f62", egc));
+    CHECK(0 == strcmp(u8"\u5f62", egc));
+    free(egc);
     egc = notcurses_at_yx(nc_, 0, 1, &c.stylemask, &c.channels);
     REQUIRE(nullptr != egc);
     CHECK(0 == strcmp("", egc));
+    free(egc);
     egc = notcurses_at_yx(nc_, 0, 2, &c.stylemask, &c.channels);
     REQUIRE(nullptr != egc);
-    CHECK(0 == strcmp("\u5168", egc));
+    CHECK(0 == strcmp(u8"\u5168", egc));
+    free(egc);
     egc = notcurses_at_yx(nc_, 0, 3, &c.stylemask, &c.channels);
     REQUIRE(nullptr != egc);
+    free(egc);
     egc = notcurses_at_yx(nc_, 0, 4, &c.stylemask, &c.channels);
     REQUIRE(nullptr != egc);
+    free(egc);
   }
 
   // If an ncplane is moved atop the right half of a wide glyph, the entire
   // glyph should be oblitrated.
   SUBCASE("PlaneStompsWideGlyph"){
-    nccell c = CELL_TRIVIAL_INITIALIZER;
+    nccell c = NCCELL_TRIVIAL_INITIALIZER;
     char* egc;
 
     // print two wide glyphs on the standard plane
-    int y, x;
-    ncplane_yx(n_, &y, &x);
-    CHECK(0 == y);
-    CHECK(0 == x);
-    CHECK(2 == ncplane_putstr(n_, "\xe5\x85\xa8")); // \u5168 (全)
+    int py, px;
+    ncplane_yx(n_, &py, &px);
+    CHECK(0 == py);
+    CHECK(0 == px);
+    CHECK(2 == ncplane_putstr(n_, u8"\xe5\x85\xa8")); // \u5168 (全)
+    unsigned y, x;
     ncplane_cursor_yx(n_, &y, &x);
     CHECK(0 == y);
     CHECK(2 == x);
-    CHECK(2 == ncplane_putstr(n_, "\xe5\xbd\xa2")); // \u5f62 (形)
+    CHECK(2 == ncplane_putstr(n_, u8"\xe5\xbd\xa2")); // \u5f62 (形)
     ncplane_cursor_yx(n_, &y, &x);
     CHECK(0 == y);
     CHECK(4 == x);
@@ -313,11 +307,11 @@ TEST_CASE("Wide") {
 
     // should be wide char 1
     CHECK(3 == ncplane_at_yx_cell(n_, 0, 0, &c));
-    CHECK(!strcmp("\xe5\x85\xa8", nccell_extended_gcluster(n_, &c)));
+    CHECK(!strcmp(u8"\xe5\x85\xa8", nccell_extended_gcluster(n_, &c)));
     CHECK(nccell_double_wide_p(&c));
     egc = notcurses_at_yx(nc_, 0, 0, &c.stylemask, &c.channels);
     REQUIRE(egc);
-    CHECK(!strcmp("\xe5\x85\xa8", egc));
+    CHECK(!strcmp(u8"\xe5\x85\xa8", egc));
     CHECK(nccell_double_wide_p(&c));
     free(egc);
     nccell_init(&c);
@@ -334,11 +328,11 @@ TEST_CASE("Wide") {
 
     // should be wide char 2
     REQUIRE(3 == ncplane_at_yx_cell(n_, 0, 2, &c));
-    CHECK(!strcmp("\xe5\xbd\xa2", nccell_extended_gcluster(n_, &c)));
+    CHECK(!strcmp(u8"\xe5\xbd\xa2", nccell_extended_gcluster(n_, &c)));
     CHECK(nccell_double_wide_p(&c));
     egc = notcurses_at_yx(nc_, 0, 2, &c.stylemask, &c.channels);
     REQUIRE(egc);
-    CHECK(!strcmp("\xe5\xbd\xa2", egc));
+    CHECK(!strcmp(u8"\xe5\xbd\xa2", egc));
     CHECK(nccell_double_wide_p(&c));
     free(egc);
     nccell_init(&c);
@@ -414,11 +408,10 @@ TEST_CASE("Wide") {
     };
     struct ncplane* p = ncplane_create(n_, &nopts);
     REQUIRE(nullptr != p);
-    nccell c = CELL_CHAR_INITIALIZER('X');
+    nccell c = NCCELL_CHAR_INITIALIZER('X');
     CHECK(0 == ncplane_perimeter(p, &c, &c, &c, &c, &c, &c, 0));
     ncplane_set_bg_rgb8(n_, 0x20, 0x20, 0x20);
-    int sbytes;
-    CHECK(2 == ncplane_putegc_yx(n_, 1, 1, "六", &sbytes));
+    CHECK(2 == ncplane_putegc_yx(n_, 1, 1, "六", nullptr));
     uint64_t channels = 0;
     ncchannels_set_bg_alpha(&channels, NCALPHA_BLEND);
     ncchannels_set_bg_rgb8(&channels, 0x80, 0xf0, 0x10);
@@ -434,9 +427,9 @@ TEST_CASE("Wide") {
     REQUIRE(nullptr != egc);
     CHECK(0 == strcmp(" ", egc));
     free(egc);
-    nccell cl = CELL_TRIVIAL_INITIALIZER, cr = CELL_TRIVIAL_INITIALIZER;
+    nccell cl = NCCELL_TRIVIAL_INITIALIZER, cr = NCCELL_TRIVIAL_INITIALIZER;
     CHECK(3 == ncplane_at_yx_cell(n_, 1, 1, &cl));
-    CHECK(0 == strcmp("六", nccell_extended_gcluster(n_, &cl)));
+    CHECK(0 == strcmp(u8"六", nccell_extended_gcluster(n_, &cl)));
     CHECK(0 == ncplane_at_yx_cell(n_, 1, 2, &cr));
     CHECK(cell_simple_p(&cr));
     CHECK(0 == cr.gcluster);
@@ -466,19 +459,19 @@ TEST_CASE("Wide") {
     REQUIRE(nullptr != topp);
     CHECK(0 == ncplane_set_bg_rgb8(topp, 0, 0xff, 0));
     CHECK(4 == ncplane_putstr(topp, "abcd"));
-    CHECK(8 == ncplane_putstr(n_, "六六六六"));
+    CHECK(8 == ncplane_putstr(n_, u8"六六六六"));
     CHECK(0 == notcurses_render(nc_));
     uint16_t stylemask;
     uint64_t channels;
     char* egc;
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 0, &stylemask, &channels)));
-    CHECK(0 == strcmp(egc, "六"));
+    CHECK(0 == strcmp(egc, u8"六"));
     free(egc);
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 1, &stylemask, &channels)));
     CHECK(0 == strcmp(egc, ""));
     free(egc);
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 2, &stylemask, &channels)));
-    CHECK(0 == strcmp(egc, "六"));
+    CHECK(0 == strcmp(egc, u8"六"));
     free(egc);
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 3, &stylemask, &channels)));
     CHECK(0 == strcmp(egc, ""));
@@ -498,7 +491,7 @@ TEST_CASE("Wide") {
     CHECK(0 == ncplane_move_yx(topp, 0, 3));
     CHECK(0 == notcurses_render(nc_));
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 0, &stylemask, &channels)));
-    CHECK(0 == strcmp(egc, "六"));
+    CHECK(0 == strcmp(egc, u8"六"));
     free(egc);
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 1, &stylemask, &channels)));
     CHECK(0 == strcmp(egc, ""));
@@ -524,7 +517,7 @@ TEST_CASE("Wide") {
     CHECK(0 == ncplane_move_yx(topp, 0, 2));
     CHECK(0 == notcurses_render(nc_));
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 0, &stylemask, &channels)));
-    CHECK(0 == strcmp(egc, "六"));
+    CHECK(0 == strcmp(egc, u8"六"));
     free(egc);
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 1, &stylemask, &channels)));
     CHECK(0 == strcmp(egc, ""));
@@ -542,7 +535,7 @@ TEST_CASE("Wide") {
     CHECK(0 == strcmp(egc, "d"));
     free(egc);
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 6, &stylemask, &channels)));
-    CHECK(0 == strcmp(egc, "六"));
+    CHECK(0 == strcmp(egc, u8"六"));
     free(egc);
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 7, &stylemask, &channels)));
     CHECK(0 == strcmp(egc, ""));
@@ -568,7 +561,7 @@ TEST_CASE("Wide") {
     CHECK(0 == strcmp(egc, ""));
     free(egc);
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 6, &stylemask, &channels)));
-    CHECK(0 == strcmp(egc, "六"));
+    CHECK(0 == strcmp(egc, u8"六"));
     free(egc);
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 7, &stylemask, &channels)));
     CHECK(0 == strcmp(egc, ""));
@@ -588,13 +581,13 @@ TEST_CASE("Wide") {
     CHECK(0 == strcmp(egc, "d"));
     free(egc);
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 4, &stylemask, &channels)));
-    CHECK(0 == strcmp(egc, "六"));
+    CHECK(0 == strcmp(egc, u8"六"));
     free(egc);
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 5, &stylemask, &channels)));
     CHECK(0 == strcmp(egc, ""));
     free(egc);
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 6, &stylemask, &channels)));
-    CHECK(0 == strcmp(egc, "六"));
+    CHECK(0 == strcmp(egc, u8"六"));
     free(egc);
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 7, &stylemask, &channels)));
     CHECK(0 == strcmp(egc, ""));
@@ -620,33 +613,33 @@ TEST_CASE("Wide") {
     struct ncplane* topp = ncplane_create(n_, &nopts);
     REQUIRE(nullptr != topp);
     CHECK(0 == ncplane_set_bg_rgb8(topp, 0, 0xff, 0));
-    CHECK(4 == ncplane_putstr(topp, "次次"));
+    CHECK(4 == ncplane_putstr(topp, u8"次次"));
     CHECK(0 == ncplane_cursor_move_yx(n_, 0, 0));
-    CHECK(8 == ncplane_putstr(n_, "六六六六"));
+    CHECK(8 == ncplane_putstr(n_, u8"六六六六"));
     CHECK(0 == notcurses_render(nc_));
     uint16_t stylemask;
     uint64_t channels;
     char* egc;
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 0, &stylemask, &channels)));
-    CHECK(0 == strcmp(egc, "六"));
+    CHECK(0 == strcmp(egc, u8"六"));
     free(egc);
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 1, &stylemask, &channels)));
     CHECK(0 == strcmp(egc, ""));
     free(egc);
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 2, &stylemask, &channels)));
-    CHECK(0 == strcmp(egc, "六"));
+    CHECK(0 == strcmp(egc, u8"六"));
     free(egc);
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 3, &stylemask, &channels)));
     CHECK(0 == strcmp(egc, ""));
     free(egc);
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 4, &stylemask, &channels)));
-    CHECK(0 == strcmp(egc, "次"));
+    CHECK(0 == strcmp(egc, u8"次"));
     free(egc);
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 5, &stylemask, &channels)));
     CHECK(0 == strcmp(egc, ""));
     free(egc);
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 6, &stylemask, &channels)));
-    CHECK(0 == strcmp(egc, "次"));
+    CHECK(0 == strcmp(egc, u8"次"));
     free(egc);
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 7, &stylemask, &channels)));
     CHECK(0 == strcmp(egc, ""));
@@ -654,7 +647,7 @@ TEST_CASE("Wide") {
     CHECK(0 == ncplane_move_yx(topp, 0, 3));
     CHECK(0 == notcurses_render(nc_));
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 0, &stylemask, &channels)));
-    CHECK(0 == strcmp(egc, "六"));
+    CHECK(0 == strcmp(egc, u8"六"));
     free(egc);
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 1, &stylemask, &channels)));
     CHECK(0 == strcmp(egc, ""));
@@ -663,13 +656,13 @@ TEST_CASE("Wide") {
     CHECK(0 == strcmp(egc, " "));
     free(egc);
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 3, &stylemask, &channels)));
-    CHECK(0 == strcmp(egc, "次"));
+    CHECK(0 == strcmp(egc, u8"次"));
     free(egc);
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 4, &stylemask, &channels)));
     CHECK(0 == strcmp(egc, ""));
     free(egc);
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 5, &stylemask, &channels)));
-    CHECK(0 == strcmp(egc, "次"));
+    CHECK(0 == strcmp(egc, u8"次"));
     free(egc);
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 6, &stylemask, &channels)));
     CHECK(0 == strcmp(egc, ""));
@@ -680,25 +673,25 @@ TEST_CASE("Wide") {
     CHECK(0 == ncplane_move_yx(topp, 0, 2));
     CHECK(0 == notcurses_render(nc_));
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 0, &stylemask, &channels)));
-    CHECK(0 == strcmp(egc, "六"));
+    CHECK(0 == strcmp(egc, u8"六"));
     free(egc);
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 1, &stylemask, &channels)));
     CHECK(0 == strcmp(egc, ""));
     free(egc);
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 2, &stylemask, &channels)));
-    CHECK(0 == strcmp(egc, "次"));
+    CHECK(0 == strcmp(egc, u8"次"));
     free(egc);
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 3, &stylemask, &channels)));
     CHECK(0 == strcmp(egc, ""));
     free(egc);
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 4, &stylemask, &channels)));
-    CHECK(0 == strcmp(egc, "次"));
+    CHECK(0 == strcmp(egc, u8"次"));
     free(egc);
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 5, &stylemask, &channels)));
     CHECK(0 == strcmp(egc, ""));
     free(egc);
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 6, &stylemask, &channels)));
-    CHECK(0 == strcmp(egc, "六"));
+    CHECK(0 == strcmp(egc, u8"六"));
     free(egc);
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 7, &stylemask, &channels)));
     CHECK(0 == strcmp(egc, ""));
@@ -709,13 +702,13 @@ TEST_CASE("Wide") {
     CHECK(0 == strcmp(egc, " "));
     free(egc);
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 1, &stylemask, &channels)));
-    CHECK(0 == strcmp(egc, "次"));
+    CHECK(0 == strcmp(egc, u8"次"));
     free(egc);
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 2, &stylemask, &channels)));
     CHECK(0 == strcmp(egc, ""));
     free(egc);
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 3, &stylemask, &channels)));
-    CHECK(0 == strcmp(egc, "次"));
+    CHECK(0 == strcmp(egc, u8"次"));
     free(egc);
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 4, &stylemask, &channels)));
     CHECK(0 == strcmp(egc, ""));
@@ -724,7 +717,7 @@ TEST_CASE("Wide") {
     CHECK(0 == strcmp(egc, ""));
     free(egc);
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 6, &stylemask, &channels)));
-    CHECK(0 == strcmp(egc, "六"));
+    CHECK(0 == strcmp(egc, u8"六"));
     free(egc);
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 7, &stylemask, &channels)));
     CHECK(0 == strcmp(egc, ""));
@@ -732,25 +725,25 @@ TEST_CASE("Wide") {
     CHECK(0 == ncplane_move_yx(topp, 0, 0));
     CHECK(0 == notcurses_render(nc_));
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 0, &stylemask, &channels)));
-    CHECK(0 == strcmp(egc, "次"));
+    CHECK(0 == strcmp(egc, u8"次"));
     free(egc);
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 1, &stylemask, &channels)));
     CHECK(0 == strcmp(egc, ""));
     free(egc);
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 2, &stylemask, &channels)));
-    CHECK(0 == strcmp(egc, "次"));
+    CHECK(0 == strcmp(egc, u8"次"));
     free(egc);
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 3, &stylemask, &channels)));
     CHECK(0 == strcmp(egc, ""));
     free(egc);
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 4, &stylemask, &channels)));
-    CHECK(0 == strcmp(egc, "六"));
+    CHECK(0 == strcmp(egc, u8"六"));
     free(egc);
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 5, &stylemask, &channels)));
     CHECK(0 == strcmp(egc, ""));
     free(egc);
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 6, &stylemask, &channels)));
-    CHECK(0 == strcmp(egc, "六"));
+    CHECK(0 == strcmp(egc, u8"六"));
     free(egc);
     REQUIRE((egc = notcurses_at_yx(nc_, 0, 7, &stylemask, &channels)));
     CHECK(0 == strcmp(egc, ""));
@@ -776,7 +769,7 @@ TEST_CASE("Wide") {
     struct ncplane* topp = ncplane_create(n_, &nopts);
     REQUIRE(nullptr != topp);
     CHECK(0 == ncplane_set_bg_rgb8(topp, 0, 0xff, 0));
-    CHECK(4 == ncplane_putstr(topp, "次次"));
+    CHECK(4 == ncplane_putstr(topp, u8"次次"));
     CHECK(8 == ncplane_putstr(n_, "abcdefgh"));
     CHECK(0 == notcurses_render(nc_));
     uint16_t stylemask;
@@ -918,7 +911,7 @@ TEST_CASE("Wide") {
   // the NUL backstop for long inlined UTF8).
   // U+1F427 PINCHED FINGERS → UTF8(f0 9f a4 8c)
   SUBCASE("ItalicEmoji") {
-    nccell c = CELL_TRIVIAL_INITIALIZER;
+    nccell c = NCCELL_TRIVIAL_INITIALIZER;
     nccell_load(n_, &c, "\U0001F427");
     CHECK(0xa7909ff0 == htole(c.gcluster));
     nccell_on_styles(&c, NCSTYLE_ITALIC);
@@ -937,7 +930,7 @@ TEST_CASE("Wide") {
   }
 
   SUBCASE("StyleMaxEmoji") {
-    nccell c = CELL_TRIVIAL_INITIALIZER;
+    nccell c = NCCELL_TRIVIAL_INITIALIZER;
     nccell_load(n_, &c, "\U0001F427");
     CHECK(0xa7909ff0 == htole(c.gcluster));
     nccell_on_styles(&c, NCSTYLE_MASK);
@@ -993,18 +986,26 @@ TEST_CASE("Wide") {
   }
 
   // fill the screen with un-inlineable EGCs
-#ifndef __APPLE__  // FIXME
   SUBCASE("OfflineEGCs") {
-    nccell c = CELL_TRIVIAL_INITIALIZER;
-    const char egc[] = "\U0001F471\u200D\u2640"; // all one EGC
+    nccell c = NCCELL_TRIVIAL_INITIALIZER;
+    const char egc[] = u8"\U0001F471\u200D\u2640"; // all one EGC
     CHECK(0 < nccell_load(n_, &c, egc));
     ncplane_set_scrolling(n_, true);
-    for(int i = 0 ; i < 100 ; ++i){ // FIXME fill up stdplane
+    unsigned dimx, dimy;
+    ncplane_dim_yx(n_, &dimy, &dimx);
+    unsigned cy;
+    do{
       CHECK(0 < ncplane_putc(n_, &c));
-    }
+      unsigned cx;
+      ncplane_cursor_yx(n_, &cy, &cx);
+      if(cx + 2 >= dimx){
+        CHECK(0 < ncplane_putchar(n_, '\n'));
+        ncplane_cursor_yx(n_, &cy, &cx);
+      }
+    }while(cy + 1 < dimy);
+    nccell_release(n_, &c);
     CHECK(0 == notcurses_render(nc_));
   }
-#endif
 
   SUBCASE("Putwc") {
     wchar_t w = L'\u2658';
